@@ -70,10 +70,8 @@ void tunnelbauer_t::register_besch(tunnel_besch_t *besch)
 // now we have to convert old tunnel to new ones ...
 bool tunnelbauer_t::laden_erfolgreich()
 {
-	stringhashtable_iterator_tpl<tunnel_besch_t *>iter(tunnel_by_name);
-	while(  iter.next()  ) {
-		tunnel_besch_t* besch = iter.get_current_value();
-
+	FOR(stringhashtable_tpl<tunnel_besch_t*>, const& i, tunnel_by_name) {
+		tunnel_besch_t* const besch = i.value;
 		if(besch->get_topspeed()==0) {
 			// old style, need to convert
 			if(strcmp(besch->get_name(),"RoadTunnel")==0) {
@@ -112,10 +110,8 @@ const tunnel_besch_t *tunnelbauer_t::find_tunnel(const waytype_t wtyp, const sin
 {
 	const tunnel_besch_t *find_besch=NULL;
 
-	stringhashtable_iterator_tpl<tunnel_besch_t *>iter(tunnel_by_name);
-	while(  iter.next()  ) {
-		tunnel_besch_t* besch = iter.get_current_value();
-
+	FOR(stringhashtable_tpl<tunnel_besch_t*>, const& i, tunnel_by_name) {
+		tunnel_besch_t* const besch = i.value;
 		if(besch->get_waytype() == wtyp) {
 			if(time==0  ||  (besch->get_intro_year_month()<=time  &&  besch->get_retire_year_month()>time)) {
 				if(find_besch==NULL  ||
@@ -153,9 +149,8 @@ void tunnelbauer_t::fill_menu(werkzeug_waehler_t* wzw, const waytype_t wtyp, sin
 	const uint16 time=welt->get_timeline_year_month();
 	vector_tpl<const tunnel_besch_t*> matching(tunnel_by_name.get_count());
 
-	stringhashtable_iterator_tpl<tunnel_besch_t *>iter(tunnel_by_name);
-	while(  iter.next()  ) {
-		tunnel_besch_t* besch = iter.get_current_value();
+	FOR(stringhashtable_tpl<tunnel_besch_t*>, const& i, tunnel_by_name) {
+		tunnel_besch_t* const besch = i.value;
 		if (besch->get_waytype() == wtyp && (
 					time == 0 ||
 					(besch->get_intro_year_month() <= time && time < besch->get_retire_year_month())
@@ -164,8 +159,8 @@ void tunnelbauer_t::fill_menu(werkzeug_waehler_t* wzw, const waytype_t wtyp, sin
 		}
 	}
 	// now sorted ...
-	for (vector_tpl<const tunnel_besch_t*>::const_iterator i = matching.begin(), end = matching.end(); i != end; ++i) {
-		wzw->add_werkzeug( (*i)->get_builder() );
+	FOR(vector_tpl<tunnel_besch_t const*>, const i, matching) {
+		wzw->add_werkzeug(i->get_builder());
 	}
 }
 
@@ -369,7 +364,7 @@ DBG_MESSAGE("tunnelbauer_t::baue()","build from (%d,%d,%d) to (%d,%d,%d) ", pos.
 }
 
 
-const weg_besch_t *tunnelbauer_t::baue_einfahrt(karte_t *welt, spieler_t *sp, koord3d end, koord zv, const tunnel_besch_t *besch, const weg_besch_t *weg_besch, int &cost)
+void tunnelbauer_t::baue_einfahrt(karte_t *welt, spieler_t *sp, koord3d end, koord zv, const tunnel_besch_t *besch, const weg_besch_t *weg_besch, int &cost)
 {
 	grund_t *alter_boden = welt->lookup(end);
 	ribi_t::ribi ribi = alter_boden->get_weg_ribi_unmasked(besch->get_waytype()) | ribi_typ(zv);
@@ -438,7 +433,6 @@ const weg_besch_t *tunnelbauer_t::baue_einfahrt(karte_t *welt, spieler_t *sp, ko
 		spieler_t::add_maintenance( sp,  besch->get_wartung() );
 	}
 	cost += besch->get_preis();
-	return weg->get_besch();
 }
 
 
@@ -482,7 +476,7 @@ const char *tunnelbauer_t::remove(karte_t *welt, spieler_t *sp, koord3d start, w
 		// Nachbarn raussuchen
 		for(int r = 0; r < 4; r++) {
 			if((zv == koord::invalid || zv == koord::nsow[r]) &&
-				from->get_neighbour(to, wegtyp, koord::nsow[r]) &&
+				from->get_neighbour(to, wegtyp, ribi_t::nsow[r]) &&
 				!marker.ist_markiert(to))
 			{
 				tmp_list.insert(to->get_pos());
