@@ -65,6 +65,7 @@
 
 #include "../gui/karte.h"	// for debugging
 #include "../gui/werkzeug_waehler.h"
+#include "../gui/messagebox.h"
 
 #ifdef DEBUG_ROUTES
 #include "../simsys.h"
@@ -153,8 +154,8 @@ const weg_besch_t* wegbauer_t::weg_search(const waytype_t wtyp, const sint32 spe
 {
 	const weg_besch_t* best = NULL;
 	bool best_allowed = false; // Does the best way fulfill the timeline?
-	for(  stringhashtable_iterator_tpl<const weg_besch_t*> iter(alle_wegtypen); iter.next();  ) {
-		const weg_besch_t* const test = iter.get_current_value();
+	FOR(stringhashtable_tpl<weg_besch_t const*>, const& i, alle_wegtypen) {
+		weg_besch_t const* const test = i.value;
 		if(  ((test->get_wtyp()==wtyp  &&
 			(test->get_styp()==system_type  ||  system_type==weg_t::type_all))  ||  (test->get_wtyp()==track_wt  &&  test->get_styp()==weg_t::type_tram  &&  wtyp==tram_wt))
 			&&  test->get_cursor()->get_bild_nr(1)!=IMG_LEER  ) {
@@ -179,8 +180,8 @@ const weg_besch_t* wegbauer_t::weg_search(const waytype_t wtyp, const sint32 spe
 const weg_besch_t *wegbauer_t::get_earliest_way(const waytype_t wtyp)
 {
 	const weg_besch_t *besch = NULL;
-	for(  stringhashtable_iterator_tpl<const weg_besch_t*> iter(alle_wegtypen); iter.next();  ) {
-		const weg_besch_t* const test = iter.get_current_value();
+	FOR(stringhashtable_tpl<weg_besch_t const*>, const& i, alle_wegtypen) {
+		weg_besch_t const* const test = i.value;
 		if(  test->get_wtyp()==wtyp  &&  (besch==NULL  ||  test->get_intro_year_month()<besch->get_intro_year_month())  ) {
 			besch = test;
 		}
@@ -193,8 +194,8 @@ const weg_besch_t *wegbauer_t::get_earliest_way(const waytype_t wtyp)
 const weg_besch_t *wegbauer_t::get_latest_way(const waytype_t wtyp)
 {
 	const weg_besch_t *besch = NULL;
-	for(  stringhashtable_iterator_tpl<const weg_besch_t*> iter(alle_wegtypen); iter.next();  ) {
-		const weg_besch_t* const test = iter.get_current_value();
+	FOR(stringhashtable_tpl<weg_besch_t const*>, const& i, alle_wegtypen) {
+		weg_besch_t const* const test = i.value;
 		if(  test->get_wtyp()==wtyp  &&  (besch==NULL  ||  test->get_retire_year_month()>besch->get_retire_year_month())  ) {
 			besch = test;
 		}
@@ -203,6 +204,23 @@ const weg_besch_t *wegbauer_t::get_latest_way(const waytype_t wtyp)
 }
 
 
+// ture if the way is available with timely
+bool wegbauer_t::waytype_available( const waytype_t wtyp, uint16 time )
+{
+	if(  time==0  ) {
+		return true;
+	}
+
+	FOR(stringhashtable_tpl<weg_besch_t const*>, const& i, alle_wegtypen) {
+		weg_besch_t const* const test = i.value;
+		if(  test->get_wtyp()==wtyp  &&  test->get_intro_year_month()<=time  &&  test->get_retire_year_month()>time  ) {
+			return true;
+		}
+	}
+	return false;
+}
+
+
 
 const weg_besch_t * wegbauer_t::get_besch(const char * way_name,const uint16 time)
 {
@@ -223,9 +241,8 @@ void wegbauer_t::neuer_monat(karte_t *welt)
 	if(current_month!=0) {
 		// check, what changed
 		slist_tpl <const weg_besch_t *> matching;
-		stringhashtable_iterator_tpl<const weg_besch_t *> iter(alle_wegtypen);
-		while(iter.next()) {
-			const weg_besch_t * besch = iter.get_current_value();
+		FOR(stringhashtable_tpl<weg_besch_t const*>, const& i, alle_wegtypen) {
+			weg_besch_t const* const besch = i.value;
 			cbuffer_t buf;
 
 			const uint16 intro_month = besch->get_intro_year_month();
@@ -269,9 +286,8 @@ void wegbauer_t::fill_menu(werkzeug_waehler_t *wzw, const waytype_t wtyp, const
 	// list of matching types (sorted by speed)
 	vector_tpl<const weg_besch_t*> matching;
 
-	stringhashtable_iterator_tpl<const weg_besch_t*> iter(alle_wegtypen);
-	while(iter.next()) {
-		const weg_besch_t* besch = iter.get_current_value();
+	FOR(stringhashtable_tpl<weg_besch_t const*>, const& i, alle_wegtypen) {
+		weg_besch_t const* const besch = i.value;
 		if (besch->get_styp() == styp &&
 				besch->get_wtyp() == wtyp &&
 				besch->get_builder() && (
@@ -284,8 +300,8 @@ void wegbauer_t::fill_menu(werkzeug_waehler_t *wzw, const waytype_t wtyp, const
 	std::sort(matching.begin(), matching.end(), compare_ways);
 
 	// now add sorted ways ...
-	for (vector_tpl<const weg_besch_t*>::const_iterator i = matching.begin(), end = matching.end(); i != end; ++i) {
-		wzw->add_werkzeug( (*i)->get_builder() );
+	FOR(vector_tpl<weg_besch_t const*>, const i, matching) {
+		wzw->add_werkzeug(i->get_builder());
 	}
 }
 
@@ -505,7 +521,7 @@ bool wegbauer_t::is_allowed_step( const grund_t *from, const grund_t *to, long *
 
 	// universal check for elevated things ...
 	if(bautyp&elevated_flag) {
-		if(to->hat_weg(air_wt)  ||  to->ist_wasser()  ||  !check_for_leitung(zv,to)  || (!to->ist_karten_boden() && to->get_typ()!=grund_t::monorailboden) ||  to->get_typ()==grund_t::brueckenboden  ||  to->get_typ()==grund_t::tunnelboden) {
+		if(to->hat_weg(air_wt)  ||  welt->lookup_hgt(to->get_pos().get_2d())<welt->get_grundwasser()  ||  !check_for_leitung(zv,to)  || (!to->ist_karten_boden() && to->get_typ()!=grund_t::monorailboden) ||  to->get_typ()==grund_t::brueckenboden  ||  to->get_typ()==grund_t::tunnelboden) {
 			// no suitable ground below!
 			return false;
 		}
@@ -720,7 +736,7 @@ bool wegbauer_t::is_allowed_step( const grund_t *from, const grund_t *to, long *
 				*costs = s.way_count_straight;
 				if(  !to->get_leitung()  ) {
 					// extra malus for not following an existing line or going on ways
-					*costs += s.way_count_double_curve + to->hat_wege() ? 8 : 0; // prefer existing powerlines
+					*costs += s.way_count_double_curve + (to->hat_wege() ? 8 : 0); // prefer existing powerlines
 				}
 			}
 		break;
@@ -763,7 +779,7 @@ bool wegbauer_t::is_allowed_step( const grund_t *from, const grund_t *to, long *
 		case luft: // hsiegeln: runway
 			{
 				const weg_t *w = to->get_weg(air_wt);
-				if(  w  &&  w->get_besch()->get_styp()!=besch->get_styp()  &&  ribi_t::ist_einfach(w->get_ribi_unmasked())  ) {
+				if(  w  &&  w->get_besch()->get_styp()==1  &&  besch->get_styp()!=1 &&  ribi_t::ist_einfach(w->get_ribi_unmasked())  ) {
 					// cannot go over the end of a runway with a taxiway
 					return false;
 				}
@@ -854,10 +870,7 @@ void wegbauer_t::do_terraforming()
 {
 	uint32 last_terraformed = terraform_index.get_count();
 
-	for(uint32 k=0; k<terraform_index.get_count(); k++) {
-		// index in route
-		uint32 i = terraform_index[k];
-
+	FOR(vector_tpl<uint32>, const i, terraform_index) { // index in route
 		grund_t *from = welt->lookup(route[i]);
 		uint8 from_slope = from->get_grund_hang();
 
@@ -1090,8 +1103,7 @@ void wegbauer_t::route_fuer(bautyp_t wt, const weg_besch_t *b, const tunnel_besc
 void get_mini_maxi( const vector_tpl<koord3d> &ziel, koord3d &mini, koord3d &maxi )
 {
 	mini = maxi = ziel[0];
-	for( uint32 i = 1; i < ziel.get_count(); i++ ) {
-		const koord3d &current = ziel[i];
+	FOR(vector_tpl<koord3d>, const& current, ziel) {
 		if( current.x < mini.x ) {
 			mini.x = current.x;
 		} else if( current.x > maxi.x ) {
@@ -1123,8 +1135,8 @@ long wegbauer_t::intern_calc_route(const vector_tpl<koord3d> &start, const vecto
 
 	// check for existing koordinates
 	bool has_target_ground = false;
-	for( uint32 i = 0; i < ziel.get_count(); i++ ) {
-		has_target_ground |= welt->lookup(ziel[i]) != NULL;
+	FOR(vector_tpl<koord3d>, const& i, ziel) {
+		has_target_ground |= welt->lookup(i) != 0;
 	}
 	if( !has_target_ground ) {
 		return -1;
@@ -1169,8 +1181,8 @@ long wegbauer_t::intern_calc_route(const vector_tpl<koord3d> &start, const vecto
 	uint32 step = 0;
 	const grund_t* gr=NULL;
 
-	for( uint32 i = 0; i < start.get_count(); i++ ) {
-		gr = welt->lookup(start[i]);
+	FOR(vector_tpl<koord3d>, const& i, start) {
+		gr = welt->lookup(i);
 
 		// is valid ground?
 		long dummy;
@@ -1183,7 +1195,7 @@ long wegbauer_t::intern_calc_route(const vector_tpl<koord3d> &start, const vecto
 
 		tmp->parent = NULL;
 		tmp->gr = gr;
-		tmp->f = calc_distance(start[i], mini, maxi);
+		tmp->f = calc_distance(i, mini, maxi);
 		tmp->g = 0;
 		tmp->dir = 0;
 		tmp->count = 0;
@@ -1251,7 +1263,7 @@ DBG_DEBUG("insert to close","(%i,%i,%i)  f=%i",gr->get_pos().x,gr->get_pos().y,g
 
 			bool do_terraform = false;
 			const koord zv(r);
-			if(!gr->get_neighbour(to,invalid_wt,zv)  ||  !check_slope(gr, to)) {
+			if(!gr->get_neighbour(to,invalid_wt,r)  ||  !check_slope(gr, to)) {
 				// slopes do not match
 				// terraforming enabled?
 				if (bautyp==river  ||  (bautyp & terraform_flag) == 0) {
@@ -1291,12 +1303,11 @@ DBG_DEBUG("insert to close","(%i,%i,%i)  f=%i",gr->get_pos().x,gr->get_pos().y,g
 		}
 
 		// now check all valid ones ...
-		for(uint32 r=0; r<next_gr.get_count(); r++) {
-
-			to = next_gr[r].gr;
+		FOR(vector_tpl<next_gr_t>, const& r, next_gr) {
+			to = r.gr;
 
 			// new values for cost g
-			uint32 new_g = tmp->g + next_gr[r].cost;
+			uint32 new_g = tmp->g + r.cost;
 
 			settings_t const& s = welt->get_settings();
 			// check for curves (usually, one would need the lastlast and the last;
@@ -1339,7 +1350,7 @@ DBG_DEBUG("insert to close","(%i,%i,%i)  f=%i",gr->get_pos().x,gr->get_pos().y,g
 				new_g += s.way_count_double_curve;
 			}
 
-			if(new_dist == 0  &&  (next_gr[r].flag & terraform)) {
+			if (new_dist == 0 && r.flag & terraform) {
 				// no terraforming near target
 				continue;
 			}
@@ -1373,7 +1384,7 @@ DBG_DEBUG("insert to close","(%i,%i,%i)  f=%i",gr->get_pos().x,gr->get_pos().y,g
 			k->f = new_f;
 			k->dir = current_dir;
 			// count is unused here, use it as flag-variable instead
-			k->count = next_gr[r].flag;
+			k->count = r.flag;
 
 			queue.insert( k );
 
@@ -1453,12 +1464,12 @@ void wegbauer_t::intern_calc_straight_route(const koord3d start, const koord3d z
 
 		bool do_terraform = false;
 		// shortest way
-		koord diff;
+		ribi_t::ribi diff;
 		if(abs(pos.x-ziel.x)>=abs(pos.y-ziel.y)) {
-			diff = (pos.x>ziel.x) ? koord(-1,0) : koord(1,0);
+			diff = (pos.x>ziel.x) ? ribi_t::west : ribi_t::ost;
 		}
 		else {
-			diff = (pos.y>ziel.y) ? koord(0,-1) : koord(0,1);
+			diff = (pos.y>ziel.y) ? ribi_t::nord : ribi_t::sued;
 		}
 		if(bautyp&tunnel_flag) {
 #ifdef ONLY_TUNNELS_BELOW_GROUND
@@ -1477,7 +1488,7 @@ void wegbauer_t::intern_calc_straight_route(const koord3d start, const koord3d z
 				bd_von_new = true;
 			}
 			// take care of slopes
-			pos.z = bd_von->get_vmove(ribi_typ(diff));
+			pos.z = bd_von->get_vmove(diff);
 
 			// check next tile
 			grund_t *bd_nach = welt->lookup(pos + diff);
@@ -1494,7 +1505,7 @@ void wegbauer_t::intern_calc_straight_route(const koord3d start, const koord3d z
 				bd_nach_new = true;
 			}
 			// check for tunnel and right slope
-			ok = ok && bd_nach->ist_tunnel() && bd_nach->get_vmove(ribi_typ(-diff))==pos.z;
+			ok = ok && bd_nach->ist_tunnel() && bd_nach->get_vmove(ribi_t::rueckwaerts(diff))==pos.z;
 			// all other checks are done here (crossings, stations etc)
 			ok = ok && is_allowed_step(bd_von, bd_nach, &dummy_cost);
 
@@ -1533,7 +1544,7 @@ void wegbauer_t::intern_calc_straight_route(const koord3d start, const koord3d z
 					ok = false;
 					if (check_terraform) {
 						bd_nach = welt->lookup_kartenboden(bd_von->get_pos().get_2d() + diff);
-						if (bd_nach==NULL  ||  (check_slope(bd_von, bd_nach)  &&  bd_von->get_vmove(ribi_typ(diff))!=bd_nach->get_vmove(ribi_typ(-diff)))) {
+						if (bd_nach==NULL  ||  (check_slope(bd_von, bd_nach)  &&  bd_von->get_vmove(diff)!=bd_nach->get_vmove(ribi_t::rueckwaerts(diff)))) {
 							ok = false;
 						}
 						else {
@@ -1555,7 +1566,7 @@ void wegbauer_t::intern_calc_straight_route(const koord3d start, const koord3d z
 		if (do_terraform) {
 			terraform_index.append(route.get_count()-2);
 		}
-DBG_MESSAGE("wegbauer_t::calc_straight_route()","step %i,%i = %i",diff.x,diff.y,ok);
+		DBG_MESSAGE("wegbauer_t::calc_straight_route()","step %s = %i",koord(diff).get_str(),ok);
 	}
 	ok = ok && ( target_3d ? pos==ziel : pos.get_2d()==ziel.get_2d() );
 
@@ -1573,6 +1584,9 @@ DBG_MESSAGE("wegbauer_t::intern_calc_straight_route()","found straight route max
 // special for starting/landing runways
 bool wegbauer_t::intern_calc_route_runways(koord3d start3d, const koord3d ziel3d)
 {
+	route.clear();
+	terraform_index.clear();
+
 	const koord start=start3d.get_2d();
 	const koord ziel=ziel3d.get_2d();
 	// check for straight line!
@@ -1649,7 +1663,7 @@ bool wegbauer_t::intern_calc_route_runways(koord3d start3d, const koord3d ziel3d
 void wegbauer_t::calc_straight_route(koord3d start, const koord3d ziel)
 {
 	DBG_MESSAGE("wegbauer_t::calc_straight_route()","from %d,%d,%d to %d,%d,%d",start.x,start.y,start.z, ziel.x,ziel.y,ziel.z );
-	if(bautyp==luft  &&  besch->get_topspeed()>=250) {
+	if(bautyp==luft  &&  besch->get_styp()==1) {
 		// these are straight anyway ...
 		intern_calc_route_runways(start, ziel);
 	}
@@ -1837,10 +1851,7 @@ sint64 wegbauer_t::calc_costs()
 	// calculate costs for terraforming
 	uint32 last_terraformed = terraform_index.get_count();
 
-	for(uint32 k=0; k<terraform_index.get_count(); k++) {
-		// index in route
-		uint32 i = terraform_index[k];
-
+	FOR(vector_tpl<uint32>, const i, terraform_index) { // index in route
 		grund_t *from = welt->lookup(route[i]);
 		uint8 from_slope = from->get_grund_hang();
 
@@ -1863,6 +1874,7 @@ sint64 wegbauer_t::calc_costs()
 
 	for(uint32 i=0; i<get_count(); i++) {
 		sint32 old_speedlimit = -1;
+		sint32 replace_cost = 0;
 
 		const grund_t* gr = welt->lookup(route[i] + offset);
 		if( gr ) {
@@ -1882,6 +1894,7 @@ sint64 wegbauer_t::calc_costs()
 				}
 				else {
 					if (weg_t const* const weg = gr->get_weg(besch->get_wtyp())) {
+						replace_cost = weg->get_besch()->get_preis();
 						if( weg->get_besch() == besch ) {
 							continue; // Nothing to pay on this tile.
 						}
@@ -1911,7 +1924,7 @@ sint64 wegbauer_t::calc_costs()
 			}
 		}
 		if(  !keep_existing_faster_ways  ||  old_speedlimit < new_speedlimit  ) {
-			costs += single_cost;
+			costs += max(single_cost, replace_cost);
 		}
 
 		// last tile cannot be start of tunnel/bridge
@@ -2010,18 +2023,17 @@ bool wegbauer_t::baue_tunnelboden()
 
 void wegbauer_t::baue_elevated()
 {
-	for(  uint32 i=0;  i<get_count();  i++  ) {
+	FOR(koord3d_vector_t, & i, route) {
+		planquadrat_t* const plan = welt->access(i.get_2d());
 
-		planquadrat_t* plan = welt->access(route[i].get_2d());
-
-		grund_t* gr0= plan->get_boden_in_hoehe(route[i].z);
-		route[i].z += Z_TILE_STEP;
-		grund_t* gr = plan->get_boden_in_hoehe(route[i].z);
+		grund_t* const gr0 = plan->get_boden_in_hoehe(i.z);
+		i.z += Z_TILE_STEP;
+		grund_t* const gr  = plan->get_boden_in_hoehe(i.z);
 
 		if(gr==NULL) {
 			hang_t::typ hang = gr0 ? gr0->get_grund_hang() : 0;
 			// add new elevated ground
-			monorailboden_t* monorail = new monorailboden_t(welt, route[i], hang);
+			monorailboden_t* const monorail = new monorailboden_t(welt, i, hang);
 			plan->boden_hinzufuegen(monorail);
 			monorail->calc_bild();
 		}
@@ -2137,7 +2149,11 @@ void wegbauer_t::baue_schiene()
 
 				// do not touch fences, tram way etc. if there is already same way with different type
 				// keep faster ways or if it is the same way ... (@author prissi)
-				if(  weg->get_besch()==besch  ||  (besch->get_styp()==0 && weg->get_besch()->get_styp()==7 && gr->has_two_ways())  ||  keep_existing_ways  ||  (keep_existing_faster_ways  &&  weg->get_besch()->get_topspeed()>besch->get_topspeed()) || (gr->get_typ()==grund_t::monorailboden  &&  (bautyp&elevated_flag)==0)  ) {
+				if (weg->get_besch() == besch                                                               ||
+						(besch->get_styp() == 0 && weg->get_besch()->get_styp() == 7 && gr->has_two_ways())     ||
+						keep_existing_ways                                                                      ||
+						(keep_existing_faster_ways && weg->get_besch()->get_topspeed() > besch->get_topspeed()) ||
+						(gr->get_typ() == grund_t::monorailboden && !(bautyp & elevated_flag))) {
 					//nothing to be done
 					change_besch = false;
 				}
@@ -2184,7 +2200,7 @@ void wegbauer_t::baue_schiene()
 				if(besch->get_wtyp()==water_wt  &&  gr->get_hoehe()==welt->get_grundwasser()) {
 					grund_t *sea = welt->lookup_kartenboden(gr->get_pos().get_2d() - koord( ribi_typ(gr->get_grund_hang() ) ));
 					if (sea  &&  sea->ist_wasser()) {
-						gr->weg_erweitern(water_wt, ribi_t::rueckwaerts(ribi));
+						gr->weg_erweitern(water_wt, ribi_t::doppelt(ribi_typ(gr->get_grund_hang() )));
 						sea->calc_bild();
 					}
 				}
@@ -2344,9 +2360,8 @@ void wegbauer_t::baue_fluss()
 		route_t to_the_sea;
 		fluss_fahrer_t ff;
 		if (to_the_sea.find_route(welt, welt->lookup_kartenboden(route[start_n].get_2d())->get_pos(), &ff, 0, ribi_t::alle, 0x7FFFFFFF)) {
-			for(  uint32 idx=0;  idx<to_the_sea.get_count();  idx++  ) {
-				weg_t* w = welt->lookup(to_the_sea.get_route()[idx])->get_weg(water_wt);
-				if(w) {
+			FOR(koord3d_vector_t, const& i, to_the_sea.get_route()) {
+				if (weg_t* const w = welt->lookup(i)->get_weg(water_wt)) {
 					int type;
 					for(  type=umgebung_t::river_types-1;  type>0;  type--  ) {
 						// llokup type
