@@ -16,7 +16,7 @@ void ground_writer_t::write_obj(FILE* fp, obj_node_t& parent, tabfileobj_t& obj)
 	slist_tpl<slist_tpl<std::string> > keys;
 	// summer images
 	for (int hangtyp = 0; hangtyp < 128; hangtyp++) {
-		keys.append(slist_tpl<std::string>());
+		keys.append();
 
 		for (int phase = 0; ; phase++) {
 			char buf[40];
