@@ -3,10 +3,12 @@
  */
 
 #include <stdio.h>
+#include <algorithm>
 
 #include "../simdebug.h"
 #include "../simgraph.h"
-#include "dingliste.h"
+#include "../simworld.h"
+
 #include "../bauer/hausbauer.h"
 #include "../dings/dummy.h"
 #include "../dings/wolke.h"
@@ -19,9 +21,6 @@
 #include "../dings/tunnel.h"
 #include "../dings/gebaeude.h"
 #include "../dings/signal.h"
-#ifdef LAGER_NOT_IN_USE
-#include "../dings/lagerhaus.h"
-#endif
 #include "../dings/label.h"
 #include "../dings/gebaeude.h"
 #include "../dings/leitung2.h"
@@ -46,6 +45,8 @@
 #include "../dataobj/freelist.h"
 #include "../dataobj/umgebung.h"
 
+#include "dingliste.h"
+
 /* All things including ways are stored in this structure.
  * The entries are packed, i.e. the first free entry is at the top.
  * To save memory, a single element (like in the case for houses or a single tree)
@@ -302,7 +303,7 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 
 	// if we have two ways, the way at index 0 is ALWAYS the road!
 	// however ships and planes may be where not way is below ...
-	if(start!=0  &&  obj.some[0]->is_way()  &&  ((weg_t *)obj.some[0])->get_waytype()==road_wt) {
+	if(start!=0  &&  obj.some[0]->get_typ()==ding_t::way  &&  ((weg_t *)obj.some[0])->get_waytype()==road_wt) {
 
 		const uint8 fahrtrichtung = ((vehikel_basis_t*)ding)->get_fahrtrichtung();
 
@@ -416,6 +417,27 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 	return false;
 }
 
+/**
+ * @returns true if tree1 must be sorted before tree2 (tree1 stands behind tree2)
+ */
+bool compare_trees(const ding_t *tree1, const ding_t *tree2)
+{
+	// the tree with larger yoff is in front
+	sint8 diff = tree2->get_yoff() - tree1->get_yoff();
+	if (diff==0) {
+		// .. or the one that is the left most (ie xoff is small)
+		diff = tree1->get_xoff() - tree2->get_xoff();
+	}
+	return diff>0;
+}
+
+
+void dingliste_t::sort_trees(uint8 index, uint8 count)
+{
+	if(top>=index+count) {
+		std::sort(&obj.some[index], &obj.some[index+count], compare_trees);
+	}
+}
 
 
 bool dingliste_t::add(ding_t* ding)
@@ -468,11 +490,9 @@ bool dingliste_t::add(ding_t* ding)
 			/* trees are a little tricky, since they cast a shadow
 			 * therefore the y-order must be correct!
 			 */
-			const sint8 offset = ding->get_yoff() + ding->get_xoff();
-
 			for(  ;  i<top;  i++) {
 				baum_t const* const tree = ding_cast<baum_t>(obj.some[i]);
-				if (!tree || tree->get_yoff() + tree->get_xoff() > offset) {
+				if (!tree  ||  compare_trees(ding, tree)) {
 					break;
 				}
 			}
@@ -1005,8 +1025,8 @@ void dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 		ding_t *save[256];
 		sint32 max_object_index = 0;
 		for(  uint16 i=0;  i<top;  i++  ) {
-			ding_t *d=bei(i);
-			if(d->is_way()
+			ding_t *d = bei((uint8)i);
+			if(d->get_typ()==ding_t::way
 				// do not save smoke
 				||  d->get_typ()==ding_t::raucher
 				||  d->get_typ()==ding_t::sync_wolke
@@ -1078,41 +1098,73 @@ void dingliste_t::dump() const
 }
 
 
+/** display all things, faster, but will lead to clipping errors
+ *  @author prissi
+ */
+void dingliste_t::display_dinge_quick_and_dirty( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool is_global ) const
+{
+	if(capacity==0) {
+		return;
+	}
+	else if(capacity==1) {
+		if(start_offset==0) {
+			obj.one->display(xpos, ypos);
+			obj.one->display_after(xpos, ypos, is_global);
+			if(is_global) {
+				obj.one->clear_flag(ding_t::dirty);
+			}
+		}
+		return;
+	}
+
+	for(uint8 n=start_offset; n<top; n++) {
+		// ist dort ein objekt ?
+		obj.some[n]->display(xpos, ypos );
+	}
+	// foreground (needs to be done backwards!
+	for (size_t n = top; n-- != 0;) {
+		obj.some[n]->display_after(xpos, ypos, is_global);
+		if(is_global) {
+			obj.some[n]->clear_flag(ding_t::dirty);
+		}
+	}
+}
+
+
 /**
  * Routine to display background images of non-moving things
  * powerlines have to be drawn after vehicles (and thus are in the obj-array inserted after vehicles)
- * @param reset_dirty will be only true for the main display; all miniworld windows should still reset main window
  * @return the index of the first moving thing (or powerline)
  *
  * dingliste_t::display_dinge_bg() .. called by the methods in grund_t
  * local_display_dinge_bg()        .. local function to avoid code duplication, returns false if the first non-valid obj is reached
  * @author Dwachs
  */
-inline bool local_display_dinge_bg(const ding_t *ding, const sint16 xpos, const sint16 ypos, const bool reset_dirty )
+inline bool local_display_dinge_bg(const ding_t *ding, const sint16 xpos, const sint16 ypos)
 {
 	const bool display_ding = !ding->is_moving();
 	if (display_ding) {
-		ding->display(xpos, ypos, reset_dirty );
+		ding->display(xpos, ypos);
 	}
 	return display_ding;
 }
 
 
-uint8 dingliste_t::display_dinge_bg( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty ) const
+uint8 dingliste_t::display_dinge_bg( const sint16 xpos, const sint16 ypos, const uint8 start_offset) const
 {
 	if(start_offset>=top) {
 		return start_offset;
 	}
 
 	if(capacity==1) {
-		if(local_display_dinge_bg(obj.one, xpos, ypos, reset_dirty)) {
+		if(local_display_dinge_bg(obj.one, xpos, ypos)) {
 			return 1;
 		}
 		return 0;
 	}
 
 	for(uint8 n=start_offset; n<top; n++) {
-		if (!local_display_dinge_bg(obj.some[n], xpos, ypos, reset_dirty)) {
+		if (!local_display_dinge_bg(obj.some[n], xpos, ypos)) {
 			return n;
 		}
 	}
@@ -1125,14 +1177,13 @@ uint8 dingliste_t::display_dinge_bg( const sint16 xpos, const sint16 ypos, const
  * .. vehicles are draws if driving in direction ribi (with special treatment of flying aircrafts)
  * .. clips vehicle only along relevant edges (depends on ribi and vehicle direction)
  * @param ontile if true then vehicles are on the tile that defines the clipping
- * @param reset_dirty will be only true for the main display; all miniworld windows should still reset main window
  * @return the index of the first non-moving thing
  *
  * dingliste_t::display_dinge_vh() .. called by the methods in grund_t
  * local_display_dinge_vh()        .. local function to avoid code duplication, returns false if the first non-valid obj is reached
  * @author Dwachs
  */
-inline bool local_display_dinge_vh(const ding_t *ding, const sint16 xpos, const sint16 ypos, const bool reset_dirty, const ribi_t::ribi ribi, const bool ontile)
+inline bool local_display_dinge_vh(const ding_t *ding, const sint16 xpos, const sint16 ypos, const ribi_t::ribi ribi, const bool ontile)
 {
 	vehikel_basis_t const* const v = ding_cast<vehikel_basis_t>(ding);
 	aircraft_t      const*       a;
@@ -1142,7 +1193,7 @@ inline bool local_display_dinge_vh(const ding_t *ding, const sint16 xpos, const
 			// activate clipping only for our direction masked by the ribi argument
 			// use non-convex clipping (16) only if we are on the currently drawn tile or its n/w neighbours
 			activate_ribi_clip( ((veh_ribi|ribi_t::rueckwaerts(veh_ribi))&ribi)  |  (ontile  ||  ribi==ribi_t::nord || ribi==ribi_t::west ? 16 : 0));
-			ding->display(xpos, ypos, reset_dirty );
+			ding->display(xpos, ypos);
 		}
 		return true;
 	}
@@ -1153,14 +1204,14 @@ inline bool local_display_dinge_vh(const ding_t *ding, const sint16 xpos, const
 }
 
 
-uint8 dingliste_t::display_dinge_vh( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty, const ribi_t::ribi ribi, const bool ontile ) const
+uint8 dingliste_t::display_dinge_vh( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const ribi_t::ribi ribi, const bool ontile ) const
 {
 	if(start_offset>=top) {
 		return start_offset;
 	}
 
 	if(capacity==1) {
-		if(local_display_dinge_vh(obj.one, xpos, ypos, reset_dirty, ribi, ontile)) {
+		if(local_display_dinge_vh(obj.one, xpos, ypos, ribi, ontile)) {
 			return 1;
 		}
 		else {
@@ -1170,7 +1221,7 @@ uint8 dingliste_t::display_dinge_vh( const sint16 xpos, const sint16 ypos, const
 
 	uint8 nr_v = start_offset;
 	for(uint8 n=start_offset; n<top; n++) {
-		if (local_display_dinge_vh(obj.some[n], xpos, ypos, reset_dirty, ribi, ontile)) {
+		if (local_display_dinge_vh(obj.some[n], xpos, ypos, ribi, ontile)) {
 			nr_v = n;
 		}
 		else {
@@ -1185,19 +1236,19 @@ uint8 dingliste_t::display_dinge_vh( const sint16 xpos, const sint16 ypos, const
 /**
  * Routine to draw foreground images of everything on the tile (no clipping) and powerlines
  * @param start_offset .. draws also background images of all objects with index>=start_offset
- * @param reset_dirty will be only true for the main display; all miniworld windows should still reset main window
+ * @param is_global will be only true for the main display; all miniworld windows should still reset main window
  */
-void dingliste_t::display_dinge_fg( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty ) const
+void dingliste_t::display_dinge_fg( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool is_global ) const
 {
 	if(capacity==0) {
 		return;
 	}
 	else if(capacity==1) {
 		if(start_offset==0) {
-			obj.one->display(xpos, ypos, reset_dirty );
+			obj.one->display(xpos, ypos);
 		}
-		obj.one->display_after(xpos, ypos, reset_dirty );
-		if(reset_dirty) {
+		obj.one->display_after(xpos, ypos, is_global);
+		if(is_global) {
 			obj.one->clear_flag(ding_t::dirty);
 		}
 		return;
@@ -1205,12 +1256,12 @@ void dingliste_t::display_dinge_fg( const sint16 xpos, const sint16 ypos, const
 
 	for(uint8 n=start_offset; n<top; n++) {
 		// ist dort ein objekt ?
-		obj.some[n]->display(xpos, ypos, reset_dirty );
+		obj.some[n]->display(xpos, ypos);
 	}
 	// foreground (needs to be done backwards!
-	for(int n=top-1; n>=0;  n--) {
-		obj.some[n]->display_after(xpos, ypos, reset_dirty );
-		if(reset_dirty) {
+	for (size_t n = top; n-- != 0;) {
+		obj.some[n]->display_after(xpos, ypos, is_global);
+		if(is_global) {
 			obj.some[n]->clear_flag(ding_t::dirty);
 		}
 	}
