@@ -11,20 +11,14 @@
 #include "../simmenu.h"
 #include "../simsys.h"
 #include "../simversion.h"
+#include "../simwin.h"
+#include "../simmesg.h"
 #include "../dataobj/umgebung.h"
 #include "../player/simplay.h"
+#include "../gui/player_frame_t.h"
 #include "../utils/cbuffer_t.h"
+#include "../utils/csv.h"
 
-// for chdir
-#ifdef _WIN32
-#include <direct.h>
-#else
-#include <unistd.h>
-#endif
-
-#ifdef DO_NOT_SEND_HASHES
-static vector_tpl<uint16>hashes_ok;	// bit set, if this player on that client is not protected
-#endif
 
 network_command_t* network_command_t::read_from_packet(packet_t *p)
 {
@@ -37,6 +31,8 @@ network_command_t* network_command_t::read_from_packet(packet_t *p)
 	network_command_t* nwc = NULL;
 	switch (p->get_id()) {
 		case NWC_GAMEINFO:    nwc = new nwc_gameinfo_t(); break;
+		case NWC_NICK:	      nwc = new nwc_nick_t(); break;
+		case NWC_CHAT:	      nwc = new nwc_chat_t(); break;
 		case NWC_JOIN:	      nwc = new nwc_join_t(); break;
 		case NWC_SYNC:        nwc = new nwc_sync_t(); break;
 		case NWC_GAME:        nwc = new nwc_game_t(); break;
@@ -45,6 +41,8 @@ network_command_t* network_command_t::read_from_packet(packet_t *p)
 		case NWC_CHECK:       nwc = new nwc_check_t(); break;
 		case NWC_PAKSETINFO:  nwc = new nwc_pakset_info_t(); break;
 		case NWC_SERVICE:     nwc = new nwc_service_t(); break;
+		case NWC_AUTH_PLAYER: nwc = new nwc_auth_player_t(); break;
+		case NWC_CHG_PLAYER:  nwc = new nwc_chg_player_t(); break;
 		default:
 			dbg->warning("network_command_t::read_from_socket", "received unknown packet id %d", p->get_id());
 	}
@@ -115,12 +113,270 @@ bool nwc_gameinfo_t::execute(karte_t *welt)
 }
 
 
-SOCKET nwc_join_t::pending_join_client = INVALID_SOCKET;
+void nwc_nick_t::rdwr()
+{
+	network_command_t::rdwr();
+	packet->rdwr_str(nickname);
+}
 
 
+/**
+ * if server: checks whether nickname is taken and generates default nick
+ */
+bool nwc_nick_t::execute(karte_t *welt)
+{
+	if(umgebung_t::server) {
+		uint32 client_id = socket_list_t::get_client_id(packet->get_sender());
+
+		if(nickname==NULL) {
+			goto generate_default_nick;
+		}
+
+		// check for same nick
+		for(uint32 i = 0; i<socket_list_t::get_count(); i++) {
+			socket_info_t& info = socket_list_t::get_client(i);
+			if ( (info.state == socket_info_t::playing  ||  i==0)
+				&&  i != client_id
+				&&  (nickname == info.nickname.c_str()  ||  nickname == "Admin")  )
+			{
+				goto generate_default_nick;
+			}
+		}
+		if (id == NWC_NICK) {
+			// do not call this tool if called by nwc_join_t::execute
+			nwc_nick_t::server_tools(welt, client_id, CHANGE_NICK, nickname);
+		}
+		return true;
+
+generate_default_nick:
+		// nick exists already
+		// generate default nick
+		cbuffer_t buf;
+		buf.printf("Client#%d", client_id);
+		nickname = (const char*)buf;
+		return true;
+	}
+	else {
+		umgebung_t::nickname = nickname!=NULL ? nickname.c_str() : "(null)";
+	}
+	return true;
+}
+
+
+void nwc_nick_t::server_tools(karte_t *welt, uint32 client_id, uint8 what, const char* nick)
+{
+	if (!socket_list_t::is_valid_client_id(client_id)) {
+		return;
+	}
+	socket_info_t &info = socket_list_t::get_client(client_id);
+
+	cbuffer_t buf;
+	switch(what) {
+		case WELCOME: {
+			// welcome message
+			buf.printf(translator::translate("Welcome, %s!", welt->get_settings().get_name_language_id()),
+				   info.nickname.c_str());
+
+			// Log chat message - please don't change order of fields
+			CSV_t csv;
+			csv.add_field( "connect" );
+			csv.add_field( client_id );
+			csv.add_field( info.address.get_str() );
+			csv.add_field( info.nickname.c_str() );
+			dbg->warning( "__ChatLog__", "%s", csv.get_str() );
+
+			break;
+		}
+		case CHANGE_NICK: {
+			if (nick==NULL  ||  info.nickname == nick) {
+				return;
+			}
+			// change nickname
+			buf.printf(translator::translate("%s now known as %s.", welt->get_settings().get_name_language_id()),
+				   info.nickname.c_str(), nick);
+
+			// Log chat message - please don't change order of fields
+			CSV_t csv;
+			csv.add_field( "namechange" );
+			csv.add_field( client_id );
+			csv.add_field( info.address.get_str() );
+			csv.add_field( info.nickname.c_str() );
+			csv.add_field( nick );
+			dbg->warning( "__ChatLog__", "%s", csv.get_str() );
+
+			info.nickname = nick;
+
+			if (client_id > 0) {
+				// send new nickname back to client
+				nwc_nick_t nwc(nick);
+				nwc.send(info.socket);
+			}
+			else {
+				// human at server
+				umgebung_t::nickname = nick;
+			}
+			break;
+		}
+		case FAREWELL: {
+			buf.printf(translator::translate("%s has left.", welt->get_settings().get_name_language_id()),
+				   info.nickname.c_str());
+
+			// Log chat message - please don't change order of fields
+			CSV_t csv;
+			csv.add_field( "disconnect" );
+			csv.add_field( client_id );
+			csv.add_field( info.address.get_str() );
+			csv.add_field( info.nickname.c_str() );
+			dbg->warning( "__ChatLog__", "%s", csv.get_str() );
+
+			break;
+		}
+		default: return;
+	}
+	werkzeug_t *w = create_tool( WKZ_ADD_MESSAGE_TOOL | SIMPLE_TOOL );
+	w->set_default_param( buf );
+	// queue tool for network
+	nwc_tool_t *nwc = new nwc_tool_t(NULL, w, koord3d::invalid, 0, welt->get_map_counter(), true);
+	network_send_server(nwc);
+	// since init always returns false, it is save to delete immediately
+	delete w;
+}
+
+
+void nwc_chat_t::rdwr()
+{
+	network_command_t::rdwr();
+	packet->rdwr_str( message );
+	packet->rdwr_byte( player_nr );
+	packet->rdwr_str( clientname );
+	packet->rdwr_str( destination );
+
+	dbg->warning("nwc_chat_t::rdwr", "rdwr message=%s plnr=%d clientname=%s destination=%s", message.c_str(), player_nr, clientname.c_str(), destination.c_str());
+}
+
+
+void nwc_chat_t::add_message (karte_t* welt) const
+{
+	dbg->warning("nwc_chat_t::add_message", "");
+	cbuffer_t buf;  // Output which will be printed to chat window
+
+	PLAYER_COLOR_VAL color = player_nr < PLAYER_UNOWNED  ?  welt->get_spieler( player_nr )->get_player_color1()  :  COL_WHITE;
+	uint16 flag = message_t::chat;
+
+	if (  destination == NULL  ) {
+		if (  player_nr < PLAYER_UNOWNED  ) {
+			buf.printf( "%s <%s>: %s", clientname.c_str(), welt->get_spieler( player_nr )->get_name(), message.c_str() );
+		}
+		else {
+			buf.printf( "%s: %s", clientname.c_str(), message.c_str() );
+		}
+	}
+	else {
+		// Whisper, do not store message in savegame
+		flag |= message_t::local_flag;
+		if (  player_nr < PLAYER_UNOWNED  ) {
+			buf.printf( "%s <%s> --> %s: %s", clientname.c_str(), welt->get_spieler( player_nr )->get_name(), destination.c_str(), message.c_str() );
+		}
+		else {
+			buf.printf( "%s --> %s: %s", clientname.c_str(), destination.c_str(), message.c_str() );
+		}
+	}
+	welt->get_message()->add_message( buf.get_str(), koord::invalid, flag, color, IMG_LEER );
+}
+
+
+bool nwc_chat_t::execute (karte_t* welt)
+{
+	if (  message == NULL  ) {
+		dbg->warning("nwc_chat_t::execute", "null message");
+		return true;
+	}
+
+
+	// Relay message to all listening clients
+	if (  umgebung_t::server  ) {
+		uint32 client_id = socket_list_t::get_client_id( packet->get_sender() );
+
+		dbg->warning("nwc_chat_t::execute", "server, client id: %d", client_id);
+
+		// Invalid client_id means this is a message from the server to itself
+		if (  !socket_list_t::is_valid_client_id( client_id )  ) {
+			// Act on message as if we are a client
+			add_message(welt);
+			return true;
+		}
+
+		// Clients can only send messages as companies they have unlocked
+		if (  player_nr < PLAYER_UNOWNED  &&  !socket_list_t::get_client( client_id ).is_player_unlocked( player_nr )  ) {
+			dbg->warning("nwc_chat_t::execute", "attempt to send message as locked company by client %d, redirecting to PLAYER_UNOWNED", client_id);
+			player_nr = PLAYER_UNOWNED;
+		}
+
+		// Otherwise forward message as appropriate
+		socket_info_t &info = socket_list_t::get_client( client_id );
+
+		nwc_chat_t* nwchat = new nwc_chat_t( message, player_nr, info.nickname.c_str(), destination );
+
+		if (  destination == NULL  ) {
+			network_send_all( nwchat, false );
+
+			// Log chat message - please don't change order of fields
+			CSV_t csv;
+			csv.add_field( "chat" );
+			csv.add_field( client_id );
+			csv.add_field( info.address.get_str() );
+			csv.add_field( info.nickname.c_str() );
+			csv.add_field( player_nr );
+			csv.add_field( player_nr < PLAYER_UNOWNED ? welt->get_spieler( player_nr )->get_name() : "" );
+			csv.add_field( message.c_str() );
+			dbg->warning( "__ChatLog__", "%s", csv.get_str() );
+		}
+		else {
+			// Send to a specific client
+			// Look up a client with a matching name, if none matches
+			// send a message back saying that client doesn't exist
+
+			// Check if destination nick exists
+			for (  uint32 i = 0;  i < socket_list_t::get_count();  i++  ) {
+				socket_info_t& dest_info = socket_list_t::get_client(i);
+				if (  (dest_info.state == socket_info_t::playing  ||  i == 0  )
+					&&  i != client_id
+					&&  destination == dest_info.nickname.c_str()  )
+				{
+					nwchat->send( dest_info.socket );
+				}
+			}
+
+			// TODO also send a copy back to sending client for logging
+
+			delete nwchat;
+
+			// Log chat message - please don't change order of fields
+			CSV_t csv;
+			csv.add_field( "private" );
+			csv.add_field( client_id );
+			csv.add_field( info.address.get_str() );
+			csv.add_field( info.nickname.c_str() );
+			csv.add_field( player_nr );
+			csv.add_field( player_nr < PLAYER_UNOWNED ? welt->get_spieler( player_nr )->get_name() : "" );
+			csv.add_field( destination.c_str() );
+			csv.add_field( message.c_str() );
+			dbg->warning( "__ChatLog__", "%s", csv.get_str() );
+		}
+	}
+	else {
+		add_message(welt);
+	}
+	return true;
+}
+
+
+
+SOCKET nwc_join_t::pending_join_client = INVALID_SOCKET;
+
 void nwc_join_t::rdwr()
 {
-	network_command_t::rdwr();
+	nwc_nick_t::rdwr();
 	packet->rdwr_long(client_id);
 	packet->rdwr_byte(answer);
 }
@@ -133,6 +389,14 @@ bool nwc_join_t::execute(karte_t *welt)
 		// TODO: check whether we can send a file
 		nwc_join_t nwj;
 		nwj.client_id = socket_list_t::get_client_id(packet->get_sender());
+		//save nickname
+		if (socket_list_t::is_valid_client_id(nwj.client_id)) {
+			// check nickname
+			nwc_nick_t::execute(welt);
+			nwj.nickname = nickname;
+			socket_list_t::get_client(nwj.client_id).nickname = nickname;
+		}
+
 		// no other joining process active?
 		nwj.answer = socket_list_t::get_client(nwj.client_id).is_active()  &&  pending_join_client == INVALID_SOCKET ? 1 : 0;
 		DBG_MESSAGE( "nwc_join_t::execute", "client_id=%i active=%i pending_join_client=%i active=%d", socket_list_t::get_client_id(packet->get_sender()), socket_list_t::get_client(nwj.client_id).is_active(), pending_join_client, nwj.answer );
@@ -179,7 +443,7 @@ vector_tpl<uint32> nwc_ready_t::all_map_counters(MAX_MAP_COUNTERS);
 void nwc_ready_t::append_map_counter(uint32 map_counter_)
 {
 	if (all_map_counters.get_count() == MAX_MAP_COUNTERS) {
-		all_map_counters.remove_at( all_map_counters.get_count()-1 );
+		all_map_counters.pop_back();
 	}
 	all_map_counters.insert_at(0, map_counter_);
 }
@@ -205,8 +469,8 @@ bool nwc_ready_t::execute(karte_t *welt)
 			return true;
 		}
 		// check the validity of the map counter
-		for(  uint32 i=0;  i<all_map_counters.get_count();  ++i  ) {
-			if(  all_map_counters[i]==map_counter  ) {
+		FOR(vector_tpl<uint32>, const i, all_map_counters) {
+			if (i == map_counter) {
 				// unpause the sender by sending nwc_ready_t back
 				nwc_ready_t nwc(sync_step, map_counter, checklist);
 				if(  !nwc.send( get_sender())  ) {
@@ -248,6 +512,97 @@ void nwc_game_t::rdwr()
 	packet->rdwr_long(len);
 }
 
+
+
+void nwc_auth_player_t::rdwr()
+{
+	network_command_t::rdwr();
+	for(uint32 i=0; i<20; i++) {
+		packet->rdwr_byte( hash[i] );
+	}
+	packet->rdwr_byte( player_nr );
+	packet->rdwr_short(player_unlocked);
+}
+
+
+bool nwc_auth_player_t::execute(karte_t *welt)
+{
+	dbg->message("nwc_auth_player_t::execute","plnr = %d  unlock = %d  our_client_id = %d", player_nr, player_unlocked, our_client_id);
+
+	if(  umgebung_t::server  &&  !(our_client_id==0  &&  player_nr==255)) {
+		// sent to server, and not sent to player playing on server
+		if (socket_list_t::is_valid_client_id(our_client_id)) {
+
+			// player activated for this client?
+			socket_info_t &info = socket_list_t::get_client(our_client_id);
+			if (info.is_player_unlocked(player_nr)) {
+				dbg->message("nwc_auth_player_t::execute","set pwd for plnr = %d", player_nr);
+
+				// change password
+				if (welt->get_spieler(player_nr)->access_password_hash() != hash) {
+					welt->get_spieler(player_nr)->access_password_hash() = hash;
+					// unlock all clients if new password is empty
+					// otherwise lock all
+					socket_list_t::unlock_player_all(player_nr, hash.empty(), our_client_id);
+				}
+			}
+			else if (player_nr < PLAYER_UNOWNED) {
+				// check password
+				if (welt->get_spieler(player_nr)->access_password_hash() == hash) {
+
+					dbg->message("nwc_auth_player_t::execute","unlock plnr = %d at our_client_id = %d", player_nr, our_client_id);
+
+					info.unlock_player(player_nr);
+					if (our_client_id == 0) {
+						// unlock player on the server
+						welt->get_spieler(player_nr)->unlock(true, false);
+					}
+				}
+				if (our_client_id != 0) {
+					// send unlock-info to player on the client (to clear unlock_pending flag)
+					nwc_auth_player_t nwc;
+					nwc.player_unlocked = info.player_unlocked;
+					nwc.send( get_sender());
+				}
+			}
+		}
+	}
+	else {
+		for(uint8 i=0; i<PLAYER_UNOWNED; i++) {
+			if (spieler_t *sp = welt->get_spieler(i)) {
+				sp->unlock( player_unlocked & (1<<i), false);
+			}
+		}
+	}
+	// update the player window
+	ki_kontroll_t* playerwin = (ki_kontroll_t*)win_get_magic(magic_ki_kontroll_t);
+	if (playerwin) {
+		playerwin->update_data();
+	}
+	return true;
+}
+
+
+void nwc_auth_player_t::init_player_lock_server(karte_t *welt)
+{
+	uint16 player_unlocked = 0;
+	for(uint8 i=0; i<PLAYER_UNOWNED; i++) {
+		// player not activated or password matches stored password
+		spieler_t *sp = welt->get_spieler(i);
+		if (sp == NULL  ||  sp->access_password_hash() == welt->get_player_password_hash(i) ) {
+			player_unlocked |= 1<<i;
+		}
+		if (sp) {
+			sp->unlock( player_unlocked & (1<<i), false);
+		}
+	}
+	// get the local server socket
+	socket_info_t &info = socket_list_t::get_client(0);
+	info.player_unlocked = player_unlocked;
+	dbg->message("nwc_auth_player_t::init_player_lock_server", "new = %d", player_unlocked);
+}
+
+
 network_world_command_t::network_world_command_t(uint16 id, uint32 sync_step, uint32 map_counter)
 : network_command_t(id)
 {
@@ -303,6 +658,15 @@ void nwc_sync_t::do_command(karte_t *welt)
 	const sint16 yoff = welt->get_y_off();
 	// save active player
 	const uint8 active_player = welt->get_active_player_nr();
+	// save lock state
+	uint16 player_unlocked = 0;
+	for(uint8 i=0; i<PLAYER_UNOWNED; i++) {
+		if (spieler_t *sp = welt->get_spieler(i)) {
+			if (!sp->is_locked()) {
+				player_unlocked |= 1<<i;
+			}
+		}
+	}
 	// transfer game, all clients need to sync (save, reload, and pause)
 	// now save and send
 	chdir( umgebung_t::user_dir );
@@ -330,24 +694,29 @@ void nwc_sync_t::do_command(karte_t *welt)
 	}
 	else {
 		char fn[256];
-		sprintf( fn, "server%d-network.sve", umgebung_t::server );
+		// first save password hashes
+		sprintf( fn, "server%d-pwdhash.sve", umgebung_t::server );
+		loadsave_t file;
+		if(file.wr_open(fn, loadsave_t::save_mode, "hashes", SAVEGAME_VER_NR )) {
+			welt->rdwr_player_password_hashes( &file );
+			file.close();
+		}
 
-#ifdef DO_NOT_SEND_HASHES
 		// remove passwords before transfer on the server and set default client mask
-		pwd_hash_t player_hashes[PLAYER_UNOWNED];
-		uint16 default_hashes = 0;
+		// they will be restored in karte_t::laden
+		uint16 unlocked_players = 0;
 		for(  int i=0;  i<PLAYER_UNOWNED; i++  ) {
 			spieler_t *sp = welt->get_spieler(i);
-			if(  sp==NULL  ||  !sp->set_unlock(NULL)  ) {
-				default_hashes |= (1<<i);
+			if(  sp==NULL  ||  sp->access_password_hash().empty()  ) {
+				unlocked_players |= (1<<i);
 			}
 			else {
-				pwd_hash_t& p = sp->get_password_hash();
-				player_hashes[i] = p;
-				p.clear();
+				sp->access_password_hash().clear();
 			}
 		}
-#endif
+
+		// save game
+		sprintf( fn, "server%d-network.sve", umgebung_t::server );
 		bool old_restore_UI = umgebung_t::restore_UI;
 		umgebung_t::restore_UI = true;
 		welt->speichern( fn, SERVER_SAVEGAME_VER_NR, false );
@@ -358,22 +727,11 @@ void nwc_sync_t::do_command(karte_t *welt)
 		if (err) {
 			dbg->warning("nwc_sync_t::do_command","send game failed with: %s", err);
 		}
-		// TODO: send command queue to client
 
 		uint32 old_sync_steps = welt->get_sync_steps();
 		welt->laden( fn );
 		umgebung_t::restore_UI = old_restore_UI;
 
-#ifdef DO_NOT_SEND_HASHES
-		// restore password info
-		for(  int i=0;  i<PLAYER_UNOWNED; i++  ) {
-			spieler_t *sp = welt->get_spieler(i);
-			if(  sp  ) {
-				sp->get_password_hash() = player_hashes[i];
-			}
-		}
-		hashes_ok.insert_at(client_id,default_hashes);
-#endif
 		// restore steps
 		welt->network_game_set_pause( false, old_sync_steps);
 
@@ -387,6 +745,16 @@ void nwc_sync_t::do_command(karte_t *welt)
 			nwc_ready_t nwc( old_sync_steps, welt->get_map_counter(), welt->get_checklist_at(old_sync_steps) );
 			if (nwc.send(sock)) {
 				socket_list_t::change_state( client_id, socket_info_t::playing);
+				if (socket_list_t::is_valid_client_id(client_id)) {
+					socket_list_t::get_client(client_id).player_unlocked = unlocked_players;
+					// send information about locked state
+					nwc_auth_player_t nwc;
+					nwc.player_unlocked = unlocked_players;
+					nwc.send(sock);
+
+					// welcome message
+					nwc_nick_t::server_tools(welt, client_id, nwc_nick_t::WELCOME, NULL);
+				}
 			}
 			else {
 				dbg->warning( "nwc_sync_t::do_command", "send of NWC_READY failed" );
@@ -397,6 +765,12 @@ void nwc_sync_t::do_command(karte_t *welt)
 	// restore screen coordinates & offsets
 	welt->change_world_position(ij, xoff, yoff);
 	welt->switch_active_player(active_player,true);
+	// restore lock state
+	for(uint8 i=0; i<PLAYER_UNOWNED; i++) {
+		if (spieler_t *sp = welt->get_spieler(i)) {
+			sp->unlock(player_unlocked & (1<<i));
+		}
+	}
 }
 
 void nwc_check_t::rdwr()
@@ -411,7 +785,82 @@ void nwc_check_t::rdwr()
 }
 
 
-nwc_tool_t::nwc_tool_t() : network_world_command_t(NWC_TOOL, 0, 0)
+void network_broadcast_world_command_t::rdwr()
+{
+	network_world_command_t::rdwr();
+	packet->rdwr_bool(exec);
+
+	if (packet->is_loading()  &&  umgebung_t::server  &&  exec) {
+		// server does not receive exec-commands
+		packet->failed();
+	}
+}
+
+
+bool network_broadcast_world_command_t::execute(karte_t *welt)
+{
+	if (exec) {
+		pre_execute();
+		// append to command queue
+		return network_world_command_t::execute(welt);
+	}
+	else if (umgebung_t::server) {
+		// clone
+		network_broadcast_world_command_t *nwc = clone(welt);
+		if (nwc == NULL) {
+			return true;
+		}
+		// next call to execute will put it in command queue
+		nwc->exec = true;
+		nwc->sync_step = welt->get_sync_steps() + 1;
+		// broadcast
+		network_send_all(nwc, false);
+		// return true to delete this command only if clone() returned something new
+		return true;
+	}
+	else {
+		dbg->warning("network_broadcast_world_command_t::execute", "should not reach here!");
+	}
+	return true;
+}
+
+
+void nwc_chg_player_t::rdwr()
+{
+	network_broadcast_world_command_t::rdwr();
+	packet->rdwr_byte(cmd);
+	packet->rdwr_byte(player_nr);
+	packet->rdwr_short(param);
+}
+
+
+network_broadcast_world_command_t* nwc_chg_player_t::clone(karte_t *welt)
+{
+	if (!socket_list_t::is_valid_client_id(our_client_id)) {
+		return NULL;
+	}
+	socket_info_t const& info = socket_list_t::get_client(our_client_id); //.is_player_unlocked(player_nr)
+
+	if (welt->change_player_tool(cmd, player_nr, param, info.is_player_unlocked(1), false)) {
+		return new nwc_chg_player_t(get_sync_step(), get_map_counter(), cmd, player_nr, param);
+	}
+	return NULL;
+}
+
+
+void nwc_chg_player_t::do_command(karte_t *welt)
+{
+	welt->change_player_tool(cmd, player_nr, param, true, true);
+
+	// update the window
+	ki_kontroll_t* playerwin = (ki_kontroll_t*)win_get_magic(magic_ki_kontroll_t);
+	if (playerwin) {
+		playerwin->update_data();
+	}
+}
+
+
+nwc_tool_t::nwc_tool_t() : network_broadcast_world_command_t(NWC_TOOL, 0, 0)
 {
 	default_param = NULL;
 	custom_data = new memory_rw_t(custom_data_buf, lengthof(custom_data_buf), true);
@@ -419,14 +868,13 @@ nwc_tool_t::nwc_tool_t() : network_world_command_t(NWC_TOOL, 0, 0)
 
 
 nwc_tool_t::nwc_tool_t(spieler_t *sp, werkzeug_t *wkz, koord3d pos_, uint32 sync_steps, uint32 map_counter, bool init_)
-: network_world_command_t(NWC_TOOL, sync_steps, map_counter)
+: network_broadcast_world_command_t(NWC_TOOL, sync_steps, map_counter)
 {
 	pos = pos_;
 	player_nr = sp ? sp->get_player_nr() : -1;
 	wkz_id = wkz->get_id();
 	const char *dfp = wkz->get_default_param(sp);
 	default_param = dfp ? strdup(dfp) : NULL;
-	exec = false;
 	init = init_;
 	tool_client_id = 0;
 	flags = wkz->flags;
@@ -441,7 +889,7 @@ nwc_tool_t::nwc_tool_t(spieler_t *sp, werkzeug_t *wkz, koord3d pos_, uint32 sync
 
 
 nwc_tool_t::nwc_tool_t(const nwc_tool_t &nwt)
-: network_world_command_t(NWC_TOOL, nwt.get_sync_step(), nwt.get_map_counter())
+: network_broadcast_world_command_t(NWC_TOOL, nwt.get_sync_step(), nwt.get_map_counter())
 {
 	pos = nwt.pos;
 	player_nr = nwt.player_nr;
@@ -467,7 +915,7 @@ nwc_tool_t::~nwc_tool_t()
 
 void nwc_tool_t::rdwr()
 {
-	network_world_command_t::rdwr();
+	network_broadcast_world_command_t::rdwr();
 	packet->rdwr_long(last_sync_step);
 	last_checklist.rdwr(packet);
 	packet->rdwr_byte(player_nr);
@@ -477,7 +925,6 @@ void nwc_tool_t::rdwr()
 	packet->rdwr_short(wkz_id);
 	packet->rdwr_str(default_param);
 	packet->rdwr_bool(init);
-	packet->rdwr_bool(exec);
 	packet->rdwr_long(tool_client_id);
 	packet->rdwr_byte(flags);
 	// copy custom data of wkz to/from packet
@@ -491,17 +938,13 @@ void nwc_tool_t::rdwr()
 	}
 
 	//if (packet->is_loading()) {
-		dbg->warning("nwc_tool_t::rdwr", "rdwr id=%d client=%d plnr=%d pos=%s wkzid=%d defpar=%s init=%d exec=%d flags=%d", id, tool_client_id, player_nr, pos.get_str(), wkz_id, default_param, init, exec, flags);
+		dbg->warning("nwc_tool_t::rdwr", "rdwr id=%d client=%d plnr=%d pos=%s wkzid=%d defpar=%s init=%d flags=%d", id, tool_client_id, player_nr, pos.get_str(), wkz_id, default_param, init, flags);
 	//}
-	if (packet->is_loading()  &&  umgebung_t::server  &&  exec) {
-		// server does not receive exec-commands
-		packet->failed();
-	}
 }
 
-bool nwc_tool_t::execute(karte_t *welt)
+
+void nwc_tool_t::pre_execute()
 {
-	if (exec) {
 		if (custom_data->is_saving()) {
 			// create new memory_rw_t that is in reading mode to read wkz data
 			memory_rw_t *new_custom_data = new memory_rw_t(custom_data_buf, custom_data->get_current_index(), false);
@@ -509,15 +952,25 @@ bool nwc_tool_t::execute(karte_t *welt)
 			custom_data = new_custom_data;
 		}
 		// append to command queue
-		dbg->message("nwc_tool_t::execute", "append sync_step=%d current sync_step=%d  wkz=%d %s", get_sync_step(),welt->get_sync_steps(), wkz_id, init ? "init" : "work");
-		return network_world_command_t::execute(welt);
-	}
-	else if (umgebung_t::server) {
+		dbg->message("nwc_tool_t::pre_execute", "append sync_step=%d wkz=%d %s", get_sync_step(), wkz_id, init ? "init" : "work");
+}
+
+network_broadcast_world_command_t* nwc_tool_t::clone(karte_t *welt)
+{
 		if (map_counter != welt->get_map_counter()) {
 			// command from another world
 			// maybe sent before sync happened -> ignore
-			dbg->error("nwc_tool_t::execute", "wanted to execute(%d) from another world", get_id());
-			return true; // to delete cmd
+			dbg->warning("nwc_tool_t::clone", "wanted to execute(%d) from another world", get_id());
+			return NULL; // indicate failure
+		}
+		if ( player_nr < PLAYER_UNOWNED  &&  !socket_list_t::get_client(our_client_id).is_player_unlocked(player_nr) ) {
+			if (wkz_id == (WKZ_ADD_MESSAGE_TOOL|SIMPLE_TOOL)) {
+				player_nr = PLAYER_UNOWNED;
+			}
+			else {
+				dbg->warning("nwc_tool_t::clone", "client %d not allowed to act as player %d", our_client_id, player_nr);
+				return NULL; // indicate failure
+			}
 		}
 #if 0
 #error "Pause does not reset nwc_join_t::pending_join_client properly. Disabled for now here and in simwerkz.h (wkz_pause_t)"
@@ -531,12 +984,12 @@ bool nwc_tool_t::execute(karte_t *welt)
 				welt->network_game_set_pause(false, welt->get_sync_steps());
 				// reset pending_join_client to allow connection attempts again
 				nwc_join_t::pending_join_client = INVALID_SOCKET;
-				return true;
+				return NULL; // indicate failure
 			}
 			else {
 				// do not pause the game while a join process is active
 				if(  nwc_join_t::pending_join_client != INVALID_SOCKET  ) {
-					return true;
+					return NULL; // indicate failure
 				}
 				// set pending_join_client to block connection attempts during pause
 				nwc_join_t::pending_join_client = ~INVALID_SOCKET;
@@ -545,14 +998,10 @@ bool nwc_tool_t::execute(karte_t *welt)
 #endif
 		// copy data, sets tool_client_id to sender client_id
 		nwc_tool_t *nwt = new nwc_tool_t(*this);
-		nwt->exec = true;
-		nwt->sync_step = welt->get_sync_steps() + 1;
 		nwt->last_sync_step = welt->get_last_checklist_sync_step();
 		nwt->last_checklist = welt->get_last_checklist();
-		dbg->warning("nwc_tool_t::execute", "send sync_steps=%d  wkz=%d %s", nwt->get_sync_step(), wkz_id, init ? "init" : "work");
-		network_send_all(nwt, false);
-	}
-	return true;
+		dbg->warning("nwc_tool_t::clone", "send sync_steps=%d  wkz=%d %s", nwt->get_sync_step(), wkz_id, init ? "init" : "work");
+		return nwt;
 }
 
 
@@ -576,20 +1025,6 @@ bool nwc_tool_t::cmp_default_param(const char *d1, const char *d2)
 }
 
 
-void nwc_tool_t::tool_node_t::set_default_param(const char* param) {
-	if (param == default_param) {
-		return;
-	}
-	if (default_param) {
-		free( (void *)default_param );
-		default_param = NULL;
-	}
-	if (param) {
-		default_param = strdup(param);
-	}
-}
-
-
 void nwc_tool_t::tool_node_t::set_tool(werkzeug_t *wkz_) {
 	if (wkz == wkz_) {
 		return;
@@ -624,13 +1059,12 @@ void nwc_tool_t::tool_node_t::client_set_werkzeug(werkzeug_t* &wkz_new, const ch
 }
 
 
-vector_tpl<nwc_tool_t::tool_node_t> nwc_tool_t::tool_list;
+vector_tpl<nwc_tool_t::tool_node_t*> nwc_tool_t::tool_list;
 
 
 void nwc_tool_t::do_command(karte_t *welt)
 {
 	DBG_MESSAGE("nwc_tool_t::do_command", "steps %d wkz %d %s", get_sync_step(), wkz_id, init ? "init" : "work");
-	if (exec) {
 		// commands are treated differently if they come from this client or not
 		bool local = tool_client_id == network_get_client_id();
 
@@ -640,20 +1074,20 @@ void nwc_tool_t::do_command(karte_t *welt)
 		// pointer, where the active tool from a remote client is stored
 		tool_node_t *tool_node = NULL;
 
-		spieler_t *sp = player_nr < MAX_PLAYER_COUNT ? welt->get_spieler(player_nr) : NULL;
-
 		// do we have a tool for this client already?
-		tool_node_t new_tool_node(NULL, player_nr, tool_client_id);
-		uint32 index;
-		if (tool_list.is_contained(new_tool_node)) {
-			index = tool_list.index_of(new_tool_node);
-		}
-		else {
-			tool_list.append(new_tool_node);
-			index = tool_list.get_count()-1;
+		FOR(vector_tpl<nwc_tool_t::tool_node_t*>, i, tool_list) {
+			if (i->player_id == player_nr  &&  i->client_id  == tool_client_id) {
+				tool_node = i;
+				break;
+			}
 		}
 		// this node stores the tool and its default_param
-		tool_node = &(tool_list[index]);
+		if (tool_node == NULL) {
+			tool_node = new tool_node_t(NULL, player_nr, tool_client_id);
+			tool_list.append(tool_node);
+		}
+
+		spieler_t *sp = player_nr < PLAYER_UNOWNED ? welt->get_spieler(player_nr) : NULL;
 
 		wkz = tool_node->get_tool();
 		// create a new tool if necessary
@@ -701,7 +1135,6 @@ void nwc_tool_t::do_command(karte_t *welt)
 				wkz->flags = 0;
 			}
 		}
-	}
 }
 
 // static list of execution times
@@ -806,14 +1239,16 @@ bool nwc_service_t::execute(karte_t *welt)
 
 		case SRVC_ADMIN_MSG:
 			if (text) {
-				// add message via tool
-				cbuffer_t buf;
-				buf.printf(translator::translate("[Admin] %s", welt->get_settings().get_name_language_id()), text);
-				werkzeug_t *w = create_tool( WKZ_ADD_MESSAGE_TOOL | SIMPLE_TOOL );
-				w->set_default_param( buf );
-				welt->set_werkzeug( w, NULL );
-				// since init always returns false, it is save to delete immediately
-				delete w;
+				// Send message to all clients as Public Service
+				// with reserved username Admin
+				nwc_chat_t* nwchat = new nwc_chat_t( text, 1, "Admin" );
+				network_send_all( nwchat, false );
+
+				// Log chat message - please don't change order of fields
+				CSV_t csv;
+				csv.add_field( "adminmsg" );
+				csv.add_field( text );
+				dbg->warning( "__ChatLog__", "%s", csv.get_str() );
 			}
 			break;
 
@@ -834,4 +1269,3 @@ bool nwc_service_t::execute(karte_t *welt)
 	}
 	return true; // to delete
 }
-
