@@ -26,33 +26,33 @@
 // @author hsiegeln
 #include "../simlinemgmt.h"
 #include "../simline.h"
-#include "../boden/grund.h"
 #include "messagebox.h"
 
 #include "../player/simplay.h"
 
 #include "../utils/simstring.h"
 
-#include "components/list_button.h"
+
 
 #include "convoi_detail_t.h"
 
-static const char cost_type[MAX_CONVOI_COST][64] =
+static const char cost_type[convoi_t::MAX_CONVOI_COST][64] =
 {
-	"Free Capacity", "Transported", "Revenue", "Operation", "Profit", "Distance"
+	"Free Capacity", "Transported", "Revenue", "Operation", "Profit", "Distance", "Maxspeed"
 };
 
-static const int cost_type_color[MAX_CONVOI_COST] =
+static const int cost_type_color[convoi_t::MAX_CONVOI_COST] =
 {
 	COL_FREE_CAPACITY,
 	COL_TRANSPORTED,
 	COL_REVENUE,
 	COL_OPERATION,
 	COL_PROFIT,
-	COL_DISTANCE
+	COL_DISTANCE,
+	COL_MAXSPEED
 };
 
-static const bool cost_type_money[MAX_CONVOI_COST] =
+static const bool cost_type_money[convoi_t::MAX_CONVOI_COST] =
 {
 	false, false, true, true, true, false
 };
@@ -91,10 +91,10 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 	this->mean_convoi_speed = speed_to_kmh(cnv->get_akt_speed()*4);
 	this->max_convoi_speed = speed_to_kmh(cnv->get_min_top_speed()*4);
 
-	const sint16 offset_below_viewport = 21 + view.get_groesse().y;
-	const sint16 total_width = 3*(BUTTON_WIDTH + BUTTON_SPACER) + max(BUTTON_WIDTH + 2*BUTTON_SPACER, view.get_groesse().x + 32);
+	const sint16 offset_below_viewport = D_MARGIN_TOP + D_BUTTON_HEIGHT + D_V_SPACE + view.get_groesse().y;
+	const sint16 total_width = D_MARGIN_LEFT + 3*(D_BUTTON_WIDTH + D_H_SPACE) + max( D_BUTTON_WIDTH, view.get_groesse().x ) + D_MARGIN_RIGHT;
 
-	input.set_pos(koord(10,4));
+	input.set_pos( koord(D_MARGIN_LEFT,D_MARGIN_TOP) );
 	reset_cnv_name();
 	input.add_listener(this);
 	add_komponente(&input);
@@ -102,36 +102,36 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 	add_komponente(&view);
 
 	// this convoi doesn't belong to an AI
-	button.init(button_t::roundbox, "Fahrplan", koord(BUTTON1_X,offset_below_viewport), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	button.init(button_t::roundbox, "Fahrplan", koord(BUTTON1_X,offset_below_viewport), koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 	button.set_tooltip("Alters a schedule.");
 	button.add_listener(this);
 	add_komponente(&button);
 
-	go_home_button.init(button_t::roundbox, "go home", koord(BUTTON2_X,offset_below_viewport), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	go_home_button.init(button_t::roundbox, "go home", koord(BUTTON2_X,offset_below_viewport), koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 	go_home_button.set_tooltip("Sends the convoi to the last depot it departed from!");
 	go_home_button.add_listener(this);
 	add_komponente(&go_home_button);
 
-	no_load_button.init(button_t::roundbox, "no load", koord(BUTTON3_X,offset_below_viewport), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	no_load_button.init(button_t::roundbox, "no load", koord(BUTTON3_X,offset_below_viewport), koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 	no_load_button.set_tooltip("No goods are loaded onto this convoi.");
 	no_load_button.add_listener(this);
 	add_komponente(&no_load_button);
 
-	follow_button.init(button_t::roundbox_state, "follow me", koord(view.get_pos().x, view.get_groesse().y + 21), koord(view.get_groesse().x, BUTTON_HEIGHT));
+	follow_button.init(button_t::roundbox_state, "follow me", koord(view.get_pos().x, view.get_groesse().y + 21), koord(view.get_groesse().x, D_BUTTON_HEIGHT));
 	follow_button.set_tooltip("Follow the convoi on the map.");
 	follow_button.add_listener(this);
 	add_komponente(&follow_button);
 
-	chart.set_pos(koord(88,offset_below_viewport+BUTTON_HEIGHT+11));
-	chart.set_groesse(koord(total_width-88-10, 100));
+	chart.set_pos(koord(88,offset_below_viewport+D_BUTTON_HEIGHT+11));
+	chart.set_groesse(koord(total_width-88-10, 88));
 	chart.set_dimension(12, 10000);
 	chart.set_visible(false);
 	chart.set_background(MN_GREY1);
-	for (int cost = 0; cost<MAX_CONVOI_COST; cost++) {
-		chart.add_curve( cost_type_color[cost], cnv->get_finance_history(), MAX_CONVOI_COST, cost, MAX_MONTHS, cost_type_money[cost], false, true, cost_type_money[cost]*2 );
+	for (int cost = 0; cost<convoi_t::MAX_CONVOI_COST; cost++) {
+		chart.add_curve( cost_type_color[cost], cnv->get_finance_history(), convoi_t::MAX_CONVOI_COST, cost, MAX_MONTHS, cost_type_money[cost], false, true, cost_type_money[cost]*2 );
 		filterButtons[cost].init(button_t::box_state, cost_type[cost],
-			koord(BUTTON1_X+(BUTTON_WIDTH+BUTTON_SPACER)*(cost%4), view.get_groesse().y+164+(BUTTON_HEIGHT+2)*(cost/4)),
-			koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+			koord(BUTTON1_X+(D_BUTTON_WIDTH+D_H_SPACE)*(cost%4), view.get_groesse().y+164+(D_BUTTON_HEIGHT+2)*(cost/4)),
+			koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 		filterButtons[cost].add_listener(this);
 		filterButtons[cost].background = cost_type_color[cost];
 		filterButtons[cost].set_visible(false);
@@ -142,24 +142,25 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 
 	add_komponente(&sort_label);
 
-	const sint16 yoff = offset_below_viewport+46-BUTTON_HEIGHT-2;
+	const sint16 yoff = offset_below_viewport+46-D_BUTTON_HEIGHT-2;
 
-	sort_button.init(button_t::roundbox, sort_text[umgebung_t::default_sortmode], koord(BUTTON1_X,yoff), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	sort_button.init(button_t::roundbox, sort_text[umgebung_t::default_sortmode], koord(BUTTON1_X,yoff), koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 	sort_button.set_tooltip("Sort by");
 	sort_button.add_listener(this);
 	add_komponente(&sort_button);
 
-	toggler.init(button_t::roundbox_state, "Chart", koord(BUTTON3_X,yoff), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	toggler.init(button_t::roundbox_state, "Chart", koord(BUTTON3_X,yoff), koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 	toggler.set_tooltip("Show/hide statistics");
 	toggler.add_listener(this);
 	add_komponente(&toggler);
 
-	details_button.init(button_t::roundbox, "Details", koord(BUTTON4_X,yoff), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	details_button.init(button_t::roundbox, "Details", koord(BUTTON4_X,yoff), koord(D_BUTTON_WIDTH, D_BUTTON_HEIGHT));
 	details_button.set_tooltip("Vehicle details");
 	details_button.add_listener(this);
 	add_komponente(&details_button);
 
-	scrolly.set_pos(koord(0, offset_below_viewport+50));
+	text.set_pos( koord(0,D_V_SPACE) );
+	scrolly.set_pos(koord(0, yoff+D_BUTTON_HEIGHT));
 	scrolly.set_show_scroll_x(true);
 	add_komponente(&scrolly);
 
@@ -177,7 +178,7 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 	add_komponente(&route_bar);
 
 	// goto line button
-	line_button.init( button_t::posbutton, NULL, koord(10, 64) );
+	line_button.init( button_t::posbutton, NULL, koord(D_MARGIN_LEFT, D_MARGIN_TOP + D_BUTTON_HEIGHT + D_V_SPACE + LINESPACE*4 ) );
 	line_button.set_targetpos( koord(0,0) );
 	line_button.add_listener( this );
 	line_bound = false;
@@ -265,27 +266,32 @@ enable_home:
 		// all gui stuff set => display it
 		gui_frame_t::zeichnen(pos, gr);
 
-		PUSH_CLIP(pos.x+1,pos.y+16,gr.x-2,gr.y-16);
+		PUSH_CLIP(pos.x+1,pos.y+D_TITLEBAR_HEIGHT,gr.x-2,gr.y-D_TITLEBAR_HEIGHT);
 
 		// convoi information
 		char tmp[256];
 		cbuffer_t buf;
 		static cbuffer_t info_buf;
 
+		KOORD_VAL xpos = pos.x+D_MARGIN_LEFT;
+		KOORD_VAL ypos = pos.y+D_TITLEBAR_HEIGHT+D_MARGIN_TOP+D_BUTTON_HEIGHT+D_V_SPACE;
+
 		// use median speed to avoid flickering
 		mean_convoi_speed += speed_to_kmh(cnv->get_akt_speed()*4);
 		mean_convoi_speed /= 2;
 		buf.printf( translator::translate("%i km/h (max. %ikm/h)"), (mean_convoi_speed+3)/4, speed_to_kmh(cnv->get_min_top_speed()) );
-		display_proportional( pos.x+10, pos.y+16+20, buf, ALIGN_LEFT, COL_BLACK, true );
+		display_proportional( xpos, ypos, buf, ALIGN_LEFT, COL_BLACK, true );
+		ypos += LINESPACE;
 
 		// next important: income stuff
-		int len = display_proportional(pos.x + 10, pos.y + 16 + 20 + 1 * LINESPACE, translator::translate("Gewinn"), ALIGN_LEFT, COL_BLACK, true ) + 5;
+		int len = display_proportional( xpos, ypos, translator::translate("Gewinn"), ALIGN_LEFT, COL_BLACK, true ) + 5;
 		money_to_string( tmp, cnv->get_jahresgewinn()/100.0 );
-		len += display_proportional( pos.x+10+len, pos.y+16+20+1*LINESPACE, tmp, ALIGN_LEFT, cnv->get_jahresgewinn()>0?MONEY_PLUS:MONEY_MINUS, true )+5;
+		len += display_proportional( xpos+len, ypos, tmp, ALIGN_LEFT, cnv->get_jahresgewinn()>0?MONEY_PLUS:MONEY_MINUS, true )+5;
 		money_to_string( tmp+1, cnv->get_running_cost()/100.0 );
 		strcat( tmp, "/km)" );
 		tmp[0] = '(';
-		display_proportional( pos.x+10+len, pos.y+16+20+1*LINESPACE, tmp, ALIGN_LEFT, COL_BLACK, true );
+		display_proportional( xpos+len, ypos, tmp, ALIGN_LEFT, COL_BLACK, true );
+		ypos += LINESPACE;
 
 		// the weight entry
 		info_buf.clear();
@@ -295,28 +301,29 @@ enable_home:
 		info_buf.append( "t (" );
 		info_buf.append( cnv->get_sum_gesamtgewicht()-cnv->get_sum_gewicht(), 0 );
 		info_buf.append( "t)" );
-		display_proportional( pos.x+10, pos.y+16+20+2*LINESPACE, info_buf, ALIGN_LEFT, COL_BLACK, true );
+		display_proportional( xpos, ypos, info_buf, ALIGN_LEFT, COL_BLACK, true );
+		ypos += LINESPACE;
 
 		// next stop
 		const schedule_t * fpl = cnv->get_schedule();
 		info_buf.clear();
 		info_buf.append(translator::translate("Fahrtziel"));
-		fahrplan_gui_t::gimme_short_stop_name(info_buf, cnv->get_welt(), cnv->get_besitzer(), fpl, fpl->get_aktuell(), 34);
-		len = display_proportional_clip( pos.x+10, pos.y+16+20+3*LINESPACE, info_buf, ALIGN_LEFT, COL_BLACK, true );
+		fahrplan_gui_t::gimme_short_stop_name(info_buf, cnv->get_welt(), cnv->get_besitzer(), fpl->get_current_eintrag(), 34);
+		len = display_proportional_clip( xpos, ypos, info_buf, ALIGN_LEFT, COL_BLACK, true );
 
 		// convoi load indicator
-		const int offset = max( len+10, 167)+3;
-		route_bar.set_pos(koord(offset,22+3*LINESPACE));
-		route_bar.set_groesse(koord(view.get_pos().x-offset-5, 4));
+		const KOORD_VAL offset = max( len+D_MARGIN_LEFT, 167)+3;
+		route_bar.set_pos( koord( offset,ypos-pos.y-D_TITLEBAR_HEIGHT+(LINESPACE-D_INDICATOR_HEIGHT)/2) );
+		route_bar.set_groesse( koord(view.get_pos().x-offset-D_H_SPACE, D_INDICATOR_HEIGHT) );
+		ypos += LINESPACE;
 
 		/*
 		 * only show assigned line, if there is one!
 		 * @author hsiegeln
 		 */
 		if(  cnv->get_line().is_bound()  ) {
-			sint16 add_off = line_bound*12;
-			sint16 w = display_proportional( pos.x+10+add_off, pos.y+16+20+4*LINESPACE, translator::translate("Serves Line:"), ALIGN_LEFT, COL_BLACK, true );
-			display_proportional_clip( pos.x+10+w+5+add_off, pos.y+16+20+4*LINESPACE, cnv->get_line()->get_name(), ALIGN_LEFT, cnv->get_line()->get_state_color(), true );
+			len = display_proportional( xpos+D_BUTTON_HEIGHT, ypos, translator::translate("Serves Line:"), ALIGN_LEFT, COL_BLACK, true );
+			display_proportional_clip( xpos+D_BUTTON_HEIGHT+D_H_SPACE+len, ypos, cnv->get_line()->get_name(), ALIGN_LEFT, cnv->get_line()->get_state_color(), true );
 		}
 		POP_CLIP();
 	}
@@ -333,7 +340,7 @@ void convoi_info_t::show_hide_statistics( bool show )
 	chart.set_visible(show);
 	set_fenstergroesse(get_fenstergroesse() + offset + koord(0,show?LINESPACE:-LINESPACE));
 	resize(koord(0,0));
-	for (int i=0;i<MAX_CONVOI_COST;i++) {
+	for(  int i = 0;  i < convoi_t::MAX_CONVOI_COST;  i++  ) {
 		filterButtons[i].set_visible(toggler.pressed);
 	}
 }
@@ -404,12 +411,10 @@ DBG_MESSAGE("convoi_info_t::action_triggered()","convoi state %i => cannot chang
 			route_search_in_progress = true;
 
 			// iterate over all depots and try to find shortest route
-			slist_iterator_tpl<depot_t *> depot_iter(depot_t::get_depot_list());
 			route_t * shortest_route = new route_t();
 			route_t * route = new route_t();
 			koord3d home = koord3d(0,0,0);
-			while (depot_iter.next()) {
-				depot_t *depot = depot_iter.get_current();
+			FOR(slist_tpl<depot_t*>, const depot, depot_t::get_depot_list()) {
 				vehikel_t& v = *cnv->front();
 				if (depot->get_wegtyp()   != v.get_besch()->get_waytype() ||
 						depot->get_besitzer() != cnv->get_besitzer()) {
@@ -455,8 +460,8 @@ DBG_MESSAGE("convoi_info_t::action_triggered()","convoi state %i => cannot chang
 		return true;
 	}
 
-	for ( int i = 0; i<MAX_CONVOI_COST; i++) {
-		if (komp == &filterButtons[i]) {
+	for(  int i = 0;  i < convoi_t::MAX_CONVOI_COST;  i++  ) {
+		if(  komp == &filterButtons[i]  ) {
 			filterButtons[i].pressed = !filterButtons[i].pressed;
 			if(filterButtons[i].pressed) {
 				chart.show_curve(i);
@@ -514,26 +519,26 @@ void convoi_info_t::set_fenstergroesse(koord groesse)
 {
 	gui_frame_t::set_fenstergroesse(groesse);
 
-	input.set_groesse(koord(get_fenstergroesse().x - 20, 13));
+	input.set_groesse(koord(get_fenstergroesse().x - D_MARGIN_LEFT-D_MARGIN_RIGHT, D_BUTTON_HEIGHT));
 
-	view.set_pos(koord(get_fenstergroesse().x - view.get_groesse().x - 10 , 21));
-	follow_button.set_pos(koord(view.get_pos().x, view.get_groesse().y + 21));
+	view.set_pos( koord(get_fenstergroesse().x - view.get_groesse().x - D_MARGIN_LEFT, D_MARGIN_TOP+D_BUTTON_HEIGHT+D_V_SPACE ));
+	follow_button.set_pos( view.get_pos() + koord( 0, view.get_groesse().y ) );
 
-	scrolly.set_groesse(get_client_windowsize()-scrolly.get_pos());
+	scrolly.set_groesse( get_client_windowsize()-scrolly.get_pos() );
 
-	const sint16 yoff = scrolly.get_pos().y-BUTTON_HEIGHT-3;
+	const sint16 yoff = scrolly.get_pos().y-D_BUTTON_HEIGHT-3;
 	sort_button.set_pos(koord(BUTTON1_X,yoff));
 	toggler.set_pos(koord(BUTTON3_X,yoff));
 	details_button.set_pos(koord(BUTTON4_X,yoff));
 	sort_label.set_pos(koord(2,yoff-LINESPACE-1));
 
 	// convoi speed indicator
-	speed_bar.set_pos(koord(170,22+0*LINESPACE));
-	speed_bar.set_groesse(koord(view.get_pos().x - 175, 4));
+	speed_bar.set_pos( koord(170,view.get_pos().y+0*LINESPACE+(LINESPACE-D_INDICATOR_HEIGHT)/2 ));
+	speed_bar.set_groesse(koord(view.get_pos().x - 175, D_INDICATOR_HEIGHT));
 
 	// convoi load indicator
-	filled_bar.set_pos(koord(170,22+2*LINESPACE));
-	filled_bar.set_groesse(koord(view.get_pos().x - 175, 4));
+	filled_bar.set_pos(koord(170,view.get_pos().y+2*LINESPACE+(LINESPACE-D_INDICATOR_HEIGHT)/2 ));
+	filled_bar.set_groesse(koord(view.get_pos().x - 175, D_INDICATOR_HEIGHT));
 }
 
 
@@ -561,7 +566,7 @@ void convoi_info_t::rdwr(loadsave_t *file)
 	sint32 yoff = scrolly.get_scroll_y();
 	if(  file->is_saving()  ) {
 		cnv_pos = cnv->front()->get_pos();
-		for( int i = 0; i<MAX_CONVOI_COST; i++) {
+		for(  int i = 0;  i < convoi_t::MAX_CONVOI_COST;  i++  ) {
 			if(  filterButtons[i].pressed  ) {
 				flags |= (1<<i);
 			}
@@ -593,9 +598,9 @@ void convoi_info_t::rdwr(loadsave_t *file)
 		}
 		// we might be unlucky, then search all convois for a convoi with this name
 		if(  !cnv.is_bound()  ) {
-			for(  vector_tpl<convoihandle_t>::const_iterator i = welt->convois_begin(), end = welt->convois_end();  i != end;  ++i  ) {
-				if(  strcmp( (*i)->get_name(),name)==0  ) {
-					cnv = *i;
+			FOR(vector_tpl<convoihandle_t>, const i, welt->convoys()) {
+				if (strcmp(i->get_name(), name) == 0) {
+					cnv = i;
 					break;
 				}
 			}
@@ -607,18 +612,27 @@ void convoi_info_t::rdwr(loadsave_t *file)
 			return;
 		}
 		// now we can open the window ...
-		KOORD_VAL xpos = win_get_posx( this );
-		KOORD_VAL ypos = win_get_posy( this );
+		koord const& pos = win_get_pos(this);
 		convoi_info_t *w = new convoi_info_t(cnv);
-		create_win( xpos, ypos, w, w_info, magic_convoi_info+cnv.get_id() );
+		create_win(pos.x, pos.y, w, w_info, magic_convoi_info + cnv.get_id());
 		if(  stats  ) {
 			gr.y -= 170;
 		}
 		w->set_fenstergroesse( gr );
-		for( int i = 0; i<MAX_CONVOI_COST; i++) {
-			w->filterButtons[i].pressed = (flags>>i)&1;
-			if(w->filterButtons[i].pressed) {
-				w->chart.show_curve(i);
+		if(  file->get_version()<111001  ) {
+			for(  int i = 0;  i < 6;  i++  ) {
+				w->filterButtons[i].pressed = (flags>>i)&1;
+				if(w->filterButtons[i].pressed) {
+					w->chart.show_curve(i);
+				}
+			}
+		}
+		else {
+			for(  int i = 0;  i < convoi_t::MAX_CONVOI_COST;  i++  ) {
+				w->filterButtons[i].pressed = (flags>>i)&1;
+				if(w->filterButtons[i].pressed) {
+					w->chart.show_curve(i);
+				}
 			}
 		}
 		if(  stats  ) {
