@@ -6,7 +6,6 @@
  */
 
 #include <stdio.h>
-#include <math.h>
 
 #include "../simunits.h"
 #include "../simworld.h"
@@ -20,6 +19,7 @@
 #include "../simline.h"
 #include "../simlinemgmt.h"
 #include "../simmenu.h"
+#include "../simtools.h"
 
 #include "../tpl/slist_tpl.h"
 
@@ -87,7 +87,8 @@ depot_frame_t::depot_frame_t(depot_t* depot) :
 	scrolly_pas(&cont_pas),
 	scrolly_electrics(&cont_electrics),
 	scrolly_loks(&cont_loks),
-	scrolly_waggons(&cont_waggons)
+	scrolly_waggons(&cont_waggons),
+	lb_vehicle_filter("Filter:", COL_BLACK, gui_label_t::left)
 {
 DBG_DEBUG("depot_frame_t::depot_frame_t()","get_max_convoi_length()=%i",depot->get_max_convoi_length());
 	selected_line = depot->get_selected_line();
@@ -112,8 +113,8 @@ DBG_DEBUG("depot_frame_t::depot_frame_t()","get_max_convoi_length()=%i",depot->g
 	/*
 	* [SELECT ROUTE]:
 	*/
-	line_selector.set_highlight_color(PLAYER_FLAG | (depot->get_besitzer()->get_player_color1()+1));
 	line_selector.add_listener(this);
+	line_selector.set_highlight_color(depot->get_besitzer()->get_player_color1() + 1);
 	add_komponente(&line_selector);
 	depot->get_besitzer()->simlinemgmt.sort_lines();
 
@@ -193,6 +194,7 @@ DBG_DEBUG("depot_frame_t::depot_frame_t()","get_max_convoi_length()=%i",depot->g
 	add_komponente(&tabs);
 	add_komponente(&div_tabbottom);
 	add_komponente(&lb_veh_action);
+	add_komponente(&lb_vehicle_filter);
 
 	veh_action = va_append;
 	bt_veh_action.set_typ(button_t::roundbox);
@@ -214,6 +216,10 @@ DBG_DEBUG("depot_frame_t::depot_frame_t()","get_max_convoi_length()=%i",depot->g
 	bt_show_all.set_tooltip("Show also vehicles that do not match for current action.");
 	add_komponente(&bt_show_all);
 
+	vehicle_filter.set_highlight_color(depot->get_besitzer()->get_player_color1() + 1);
+	vehicle_filter.add_listener(this);
+	add_komponente(&vehicle_filter);
+
 	koord gr = koord(0,0);
 	build_vehicle_lists();
 	layout(&gr);
@@ -316,13 +322,13 @@ void depot_frame_t::layout(koord *gr)
 	/*
 	*	Structure of [VINFO] is one multiline text.
 	*/
-	int VINFO_HEIGHT = 9*LINESPACE-1;
+	int VINFO_HEIGHT = 11*LINESPACE-1;
 
 	/*
 	* Total width is the max from [CONVOI] and [ACTIONS] width.
 	*/
-	int MIN_TOTAL_WIDTH = max(CONVOI_WIDTH, ACTIONS_WIDTH);
-	int TOTAL_WIDTH = max(fgr.x,max(CONVOI_WIDTH, ACTIONS_WIDTH));
+	int MIN_DEPOT_FRAME_WIDTH = max(CONVOI_WIDTH, ACTIONS_WIDTH);
+	int DEPOT_FRAME_WIDTH = max(fgr.x,max(CONVOI_WIDTH, ACTIONS_WIDTH));
 
 	/*
 	*	Now we can do the first vertical adjustement:
@@ -356,12 +362,12 @@ void depot_frame_t::layout(koord *gr)
 	/*
 	* DONE with layout planning - now build everything.
 	*/
-	set_min_windowsize(koord(MIN_TOTAL_WIDTH, MIN_TOTAL_HEIGHT));
-	if(fgr.x<TOTAL_WIDTH) {
-		gui_frame_t::set_fenstergroesse(koord(MIN_TOTAL_WIDTH, max(fgr.y,MIN_TOTAL_HEIGHT) ));
+	set_min_windowsize(koord(MIN_DEPOT_FRAME_WIDTH, MIN_TOTAL_HEIGHT));
+	if(fgr.x<DEPOT_FRAME_WIDTH) {
+		gui_frame_t::set_fenstergroesse(koord(MIN_DEPOT_FRAME_WIDTH, max(fgr.y,MIN_TOTAL_HEIGHT) ));
 	}
 	if(gr  &&  gr->x==0) {
-		gr->x = TOTAL_WIDTH;
+		gr->x = DEPOT_FRAME_WIDTH;
 	}
 	if(gr  &&  gr->y==0) {
 		gr->y = TOTAL_HEIGHT;
@@ -375,49 +381,48 @@ void depot_frame_t::layout(koord *gr)
 	bt_prev.set_pos(koord(3, SELECT_VSTART + 2));
 
 	inp_name.set_pos(koord(3+12, SELECT_VSTART));
-	inp_name.set_groesse(koord(TOTAL_WIDTH - 26-3, 14));
+	inp_name.set_groesse(koord(DEPOT_FRAME_WIDTH - 26-3, 14));
 
-	bt_next.set_pos(koord(TOTAL_WIDTH - 12, SELECT_VSTART + 2));
+	bt_next.set_pos(koord(DEPOT_FRAME_WIDTH - 12, SELECT_VSTART + 2));
 
 	/*
 	 * [SELECT ROUTE]:
 	 * @author hsiegeln
 	 */
 	line_selector.set_pos(koord(3, SELECT_VSTART + 14));
-	line_selector.set_groesse(koord(TOTAL_WIDTH - 3, 14));
-	line_selector.set_max_size(koord(TOTAL_WIDTH - 3, LINESPACE*13+2+16));
-	line_selector.set_highlight_color(1);
+	line_selector.set_groesse(koord(DEPOT_FRAME_WIDTH - 3, 14));
+	line_selector.set_max_size(koord(DEPOT_FRAME_WIDTH - 3, LINESPACE*13+2+16));
 
 	/*
 	 * [CONVOI]
 	 */
 	convoi.set_grid(koord(grid.x - grid_dx, grid.y));
 	convoi.set_placement(koord(placement.x - placement_dx, placement.y));
-	convoi.set_pos(koord((TOTAL_WIDTH-CLIST_WIDTH)/2, CONVOI_VSTART));
+	convoi.set_pos(koord((DEPOT_FRAME_WIDTH-CLIST_WIDTH)/2, CONVOI_VSTART));
 	convoi.set_groesse(koord(CLIST_WIDTH, CLIST_HEIGHT));
 
 	lb_convoi_count.set_pos(koord(4, CINFO_VSTART));
 	lb_convoi_speed.set_pos(koord(4, CINFO_VSTART + LINESPACE));
-	lb_convoi_value.set_pos(koord(TOTAL_WIDTH-10, CINFO_VSTART));
+	lb_convoi_value.set_pos(koord(DEPOT_FRAME_WIDTH-10, CINFO_VSTART));
 	lb_convoi_line.set_pos(koord(4, CINFO_VSTART + LINESPACE * 2));
 
 	/*
 	 * [ACTIONS]
 	 */
 	bt_start.set_pos(koord(2, ACTIONS_VSTART));
-	bt_start.set_groesse(koord(TOTAL_WIDTH/4-2, ABUTTON_HEIGHT));
+	bt_start.set_groesse(koord(DEPOT_FRAME_WIDTH/4-2, ABUTTON_HEIGHT));
 	bt_start.set_text("Start");
 
-	bt_schedule.set_pos(koord(TOTAL_WIDTH/4+2, ACTIONS_VSTART));
-	bt_schedule.set_groesse(koord(TOTAL_WIDTH*2/4-TOTAL_WIDTH/4-3, ABUTTON_HEIGHT));
+	bt_schedule.set_pos(koord(DEPOT_FRAME_WIDTH/4+2, ACTIONS_VSTART));
+	bt_schedule.set_groesse(koord(DEPOT_FRAME_WIDTH*2/4-DEPOT_FRAME_WIDTH/4-3, ABUTTON_HEIGHT));
 	bt_schedule.set_text("Fahrplan");
 
-	bt_destroy.set_pos(koord(TOTAL_WIDTH*2/4+1, ACTIONS_VSTART));
-	bt_destroy.set_groesse(koord(TOTAL_WIDTH*3/4-TOTAL_WIDTH*2/4-2, ABUTTON_HEIGHT));
+	bt_destroy.set_pos(koord(DEPOT_FRAME_WIDTH*2/4+1, ACTIONS_VSTART));
+	bt_destroy.set_groesse(koord(DEPOT_FRAME_WIDTH*3/4-DEPOT_FRAME_WIDTH*2/4-2, ABUTTON_HEIGHT));
 	bt_destroy.set_text("Aufloesen");
 
-	bt_sell.set_pos(koord(TOTAL_WIDTH*3/4+1, ACTIONS_VSTART));
-	bt_sell.set_groesse(koord(TOTAL_WIDTH-TOTAL_WIDTH*3/4-3, ABUTTON_HEIGHT));
+	bt_sell.set_pos(koord(DEPOT_FRAME_WIDTH*3/4+1, ACTIONS_VSTART));
+	bt_sell.set_groesse(koord(DEPOT_FRAME_WIDTH-DEPOT_FRAME_WIDTH*3/4-3, ABUTTON_HEIGHT));
 	bt_sell.set_text("Verkauf");
 
 	/*
@@ -425,26 +430,26 @@ void depot_frame_t::layout(koord *gr)
 	 * @author hsiegeln
 	 */
 	bt_new_line.set_pos(koord(2, ACTIONS_VSTART+ABUTTON_HEIGHT));
-	bt_new_line.set_groesse(koord(TOTAL_WIDTH/4-2, ABUTTON_HEIGHT));
+	bt_new_line.set_groesse(koord(DEPOT_FRAME_WIDTH/4-2, ABUTTON_HEIGHT));
 	bt_new_line.set_text("New Line");
 
-	bt_apply_line.set_pos(koord(TOTAL_WIDTH/4+2, ACTIONS_VSTART+ABUTTON_HEIGHT));
-	bt_apply_line.set_groesse(koord(TOTAL_WIDTH*2/4-3-TOTAL_WIDTH/4, ABUTTON_HEIGHT));
+	bt_apply_line.set_pos(koord(DEPOT_FRAME_WIDTH/4+2, ACTIONS_VSTART+ABUTTON_HEIGHT));
+	bt_apply_line.set_groesse(koord(DEPOT_FRAME_WIDTH*2/4-3-DEPOT_FRAME_WIDTH/4, ABUTTON_HEIGHT));
 	bt_apply_line.set_text("Apply Line");
 
-	bt_change_line.set_pos(koord(TOTAL_WIDTH*2/4+1, ACTIONS_VSTART+ABUTTON_HEIGHT));
-	bt_change_line.set_groesse(koord(TOTAL_WIDTH*3/4-2-TOTAL_WIDTH*2/4, ABUTTON_HEIGHT));
+	bt_change_line.set_pos(koord(DEPOT_FRAME_WIDTH*2/4+1, ACTIONS_VSTART+ABUTTON_HEIGHT));
+	bt_change_line.set_groesse(koord(DEPOT_FRAME_WIDTH*3/4-2-DEPOT_FRAME_WIDTH*2/4, ABUTTON_HEIGHT));
 	bt_change_line.set_text("Update Line");
 
-	bt_copy_convoi.set_pos(koord(TOTAL_WIDTH*3/4+1, ACTIONS_VSTART+ABUTTON_HEIGHT));
-	bt_copy_convoi.set_groesse(koord(TOTAL_WIDTH-TOTAL_WIDTH*3/4-3, ABUTTON_HEIGHT));
+	bt_copy_convoi.set_pos(koord(DEPOT_FRAME_WIDTH*3/4+1, ACTIONS_VSTART+ABUTTON_HEIGHT));
+	bt_copy_convoi.set_groesse(koord(DEPOT_FRAME_WIDTH-DEPOT_FRAME_WIDTH*3/4-3, ABUTTON_HEIGHT));
 	bt_copy_convoi.set_text("Copy Convoi");
 
 	/*
 	* [PANEL]
 	*/
 	tabs.set_pos(koord(0, PANEL_VSTART));
-	tabs.set_groesse(koord(TOTAL_WIDTH, PANEL_HEIGHT));
+	tabs.set_groesse(koord(DEPOT_FRAME_WIDTH, PANEL_HEIGHT));
 
 	pas.set_grid(grid);
 	pas.set_placement(placement);
@@ -491,19 +496,25 @@ void depot_frame_t::layout(koord *gr)
 	scrolly_waggons.set_size_corner(false);
 
 	div_tabbottom.set_pos(koord(0,PANEL_VSTART+PANEL_HEIGHT));
-	div_tabbottom.set_groesse(koord(TOTAL_WIDTH,0));
+	div_tabbottom.set_groesse(koord(DEPOT_FRAME_WIDTH,0));
 
-	lb_veh_action.set_pos(koord(TOTAL_WIDTH-ABUTTON_WIDTH, PANEL_VSTART + PANEL_HEIGHT+4));
+	lb_veh_action.set_pos(koord(DEPOT_FRAME_WIDTH-ABUTTON_WIDTH, PANEL_VSTART + PANEL_HEIGHT + 4));
 
-	bt_veh_action.set_pos(koord(TOTAL_WIDTH-ABUTTON_WIDTH, PANEL_VSTART + PANEL_HEIGHT + 14));
-	bt_veh_action.set_groesse(koord(ABUTTON_WIDTH, ABUTTON_HEIGHT));
+	bt_veh_action.set_pos(koord(DEPOT_FRAME_WIDTH-ABUTTON_WIDTH, PANEL_VSTART + PANEL_HEIGHT + 16));
+	bt_veh_action.set_groesse(koord(ABUTTON_WIDTH-2, ABUTTON_HEIGHT));
 
-	bt_show_all.set_pos(koord(TOTAL_WIDTH-(ABUTTON_WIDTH*5)/2, PANEL_VSTART + PANEL_HEIGHT + 4 ));
+	bt_show_all.set_pos(koord(4, PANEL_VSTART + PANEL_HEIGHT + 14 ));
 	bt_show_all.pressed = show_all;
 
-	bt_obsolete.set_pos(koord(TOTAL_WIDTH-(ABUTTON_WIDTH*5)/2, PANEL_VSTART + PANEL_HEIGHT + 16));
+	bt_obsolete.set_pos(koord(4, PANEL_VSTART + PANEL_HEIGHT+28));
 	bt_obsolete.pressed = show_retired_vehicles;
 
+	lb_vehicle_filter.set_pos(koord(DEPOT_FRAME_WIDTH - (ABUTTON_WIDTH*5)/2 + 4, PANEL_VSTART + PANEL_HEIGHT + 4));
+
+	vehicle_filter.set_pos(koord(DEPOT_FRAME_WIDTH - (ABUTTON_WIDTH*5)/2 + 4, PANEL_VSTART + PANEL_HEIGHT + 16));
+	vehicle_filter.set_groesse(koord(ABUTTON_WIDTH + 30, 14));
+	vehicle_filter.set_max_size(koord(ABUTTON_WIDTH + 60, LINESPACE * 8));
+
 	const uint8 margin = 4;
 	img_bolt.set_pos(koord(get_fenstergroesse().x-skinverwaltung_t::electricity->get_bild(0)->get_pic()->w-margin,margin));
 }
@@ -540,12 +551,9 @@ void depot_frame_t::activate_convoi( convoihandle_t c )
 // true if already stored here
 bool depot_frame_t::is_contained(const vehikel_besch_t *info)
 {
-	if(depot->vehicle_count()>0) {
-		slist_iterator_tpl<vehikel_t *> iter(depot->get_vehicle_list());
-		while(iter.next()) {
-			if(iter.get_current()->get_besch()==info) {
-				return true;
-			}
+	FOR(slist_tpl<vehikel_t*>, const v, depot->get_vehicle_list()) {
+		if (v->get_besch() == info) {
+			return true;
 		}
 	}
 	return false;
@@ -558,6 +566,36 @@ void depot_frame_t::add_to_vehicle_list(const vehikel_besch_t *info)
 	// prissi: ist a non-electric track?
 	// Hajo: check for timeline
 	// prissi: and retirement date
+
+	// Check if vehicle should be filtered
+	const ware_besch_t *freight = info->get_ware();
+	// Only filter when required and never filter engines
+	if (depot->selected_filter > 0 && info->get_zuladung() > 0) {
+		if (depot->selected_filter == VEHICLE_FILTER_RELEVANT) {
+			if(freight->get_catg_index() >= 3) {
+				bool found = false;
+				FOR(vector_tpl<ware_besch_t const*>, const i, get_welt()->get_goods_list()) {
+					if (freight->get_catg_index() == i->get_catg_index()) {
+						found = true;
+						break;
+					}
+				}
+
+				// If no current goods can be transported by this vehicle, don't display it
+				if (!found) return;
+			}
+		} else if (depot->selected_filter > VEHICLE_FILTER_RELEVANT) {
+			// Filter on specific selected good
+			uint32 goods_index = depot->selected_filter - VEHICLE_FILTER_GOODS_OFFSET;
+			if (goods_index < get_welt()->get_goods_list().get_count()) {
+				const ware_besch_t *selected_good = get_welt()->get_goods_list()[goods_index];
+				if (freight->get_catg_index() != selected_good->get_catg_index()) {
+					return; // This vehicle can't transport the selected good
+				}
+			}
+		}
+	}
+
 	gui_image_list_t::image_data_t img_data;
 
 	img_data.image = info->get_basis_bild();
@@ -589,7 +627,7 @@ void depot_frame_t::add_to_vehicle_list(const vehikel_besch_t *info)
 // add all current vehicles
 void depot_frame_t::build_vehicle_lists()
 {
-	if(depot->get_vehicle_type()==NULL) {
+	if (depot->get_vehicle_type().empty()) {
 		// there are tracks etc. but no vehicles => do nothing
 		// at least initialize some data
 		update_data();
@@ -606,9 +644,7 @@ void depot_frame_t::build_vehicle_lists()
 	 */
 	if(electrics_vec.empty()  &&  pas_vec.empty()  &&  loks_vec.empty()  &&  waggons_vec.empty()) {
 		int loks = 0, waggons = 0, pax=0, electrics = 0;
-		slist_iterator_tpl<const vehikel_besch_t*> vehinfo(depot->get_vehicle_type());
-		while (vehinfo.next()) {
-			const vehikel_besch_t* info = vehinfo.get_current();
+		FOR(slist_tpl<vehikel_besch_t const*>, const info, depot->get_vehicle_type()) {
 			if(  info->get_engine_type() == vehikel_besch_t::electric  &&  (info->get_ware()==warenbauer_t::passagiere  ||  info->get_ware()==warenbauer_t::post)) {
 				electrics++;
 			}
@@ -644,19 +680,15 @@ void depot_frame_t::build_vehicle_lists()
 	// use this to show only sellable vehicles
 	if(!show_all  &&  veh_action==va_sell) {
 		// just list the one to sell
-		slist_iterator_tpl<vehikel_t *> iter2(depot->get_vehicle_list());
-		while(iter2.next()) {
-			if(vehicle_map.get(iter2.get_current()->get_besch())) {
-				continue;
-			}
-			add_to_vehicle_list( iter2.get_current()->get_besch() );
+		FOR(slist_tpl<vehikel_t*>, const v, depot->get_vehicle_list()) {
+			vehikel_besch_t const* const d = v->get_besch();
+			if (vehicle_map.get(d)) continue;
+			add_to_vehicle_list(d);
 		}
 	}
 	else {
 		// list only matching ones
-		slist_iterator_tpl<const vehikel_besch_t*> vehinfo(depot->get_vehicle_type());
-		while (vehinfo.next()) {
-			const vehikel_besch_t* info = vehinfo.get_current();
+		FOR(slist_tpl<vehikel_besch_t const*>, const info, depot->get_vehicle_type()) {
 			const vehikel_besch_t *veh = NULL;
 			convoihandle_t cnv = depot->get_convoi(icnv);
 			if(cnv.is_bound() && cnv->get_vehikel_anzahl()>0) {
@@ -782,14 +814,14 @@ void depot_frame_t::update_data()
 		veh = (veh_action == va_insert ? cnv->front() : cnv->back())->get_besch();
 	}
 
-	ptrhashtable_iterator_tpl<const vehikel_besch_t *, gui_image_list_t::image_data_t *> iter1(vehicle_map);
-	while(iter1.next()) {
-		const vehikel_besch_t *info = iter1.get_current_key();
+	FOR(vehicle_image_map, const& i, vehicle_map) {
+		vehikel_besch_t const* const    info = i.key;
+		gui_image_list_t::image_data_t& img  = *i.value;
 		const uint8 ok_color = info->is_future(month_now) || info->is_retired(month_now) ? COL_BLUE: COL_GREEN;
 
-		iter1.get_current_value()->count = 0;
-		iter1.get_current_value()->lcolor = ok_color;
-		iter1.get_current_value()->rcolor = ok_color;
+		img.count = 0;
+		img.lcolor = ok_color;
+		img.rcolor = ok_color;
 
 		/*
 		* color bars for current convoi:
@@ -801,31 +833,27 @@ void depot_frame_t::update_data()
 
 		if(veh_action == va_insert) {
 			if(!info->can_lead(veh)  ||  (veh  &&  !veh->can_follow(info))) {
-				iter1.get_current_value()->lcolor = COL_RED;
-				iter1.get_current_value()->rcolor = COL_RED;
+				img.lcolor = COL_RED;
+				img.rcolor = COL_RED;
 			} else if(!info->can_follow(NULL)) {
-				iter1.get_current_value()->lcolor = COL_YELLOW;
+				img.lcolor = COL_YELLOW;
 			}
 		} else if(veh_action == va_append) {
 			if(!info->can_follow(veh)  ||  (veh  &&  !veh->can_lead(info))) {
-				iter1.get_current_value()->lcolor = COL_RED;
-				iter1.get_current_value()->rcolor = COL_RED;
+				img.lcolor = COL_RED;
+				img.rcolor = COL_RED;
 			} else if(!info->can_lead(NULL)) {
-				iter1.get_current_value()->rcolor = COL_YELLOW;
+				img.rcolor = COL_YELLOW;
 			}
 		} else if( veh_action == va_sell ) {
-			iter1.get_current_value()->lcolor = COL_RED;
-			iter1.get_current_value()->rcolor = COL_RED;
+			img.lcolor = COL_RED;
+			img.rcolor = COL_RED;
 		}
-
-//DBG_DEBUG("depot_frame_t::update_data()","current %i = %s with color %i",info->get_name(),iter1.get_current_value()->lcolor);
 	}
 
-	slist_iterator_tpl<vehikel_t *> iter2(depot->get_vehicle_list());
-	while(iter2.next()) {
-		gui_image_list_t::image_data_t *imgdat=vehicle_map.get(iter2.get_current()->get_besch());
+	FOR(slist_tpl<vehikel_t*>, const v, depot->get_vehicle_list()) {
 		// can fail, if currently not visible
-		if(imgdat) {
+		if (gui_image_list_t::image_data_t* const imgdat = vehicle_map.get(v->get_besch())) {
 			imgdat->count++;
 			if(veh_action == va_sell) {
 				imgdat->lcolor = COL_GREEN;
@@ -845,13 +873,26 @@ void depot_frame_t::update_data()
 	// check all matching lines
 	vector_tpl<linehandle_t> lines;
 	get_line_list(depot, &lines);
-	for (vector_tpl<linehandle_t>::const_iterator i = lines.begin(), end = lines.end(); i != end; i++) {
-		linehandle_t line = *i;
+	FOR(vector_tpl<linehandle_t>, const line, lines) {
 		line_selector.append_element( new line_scrollitem_t(line) );
 		if(line==selected_line) {
 			line_selector.set_selection( line_selector.count_elements()-1 );
 		}
 	}
+
+	// Update vehicle filter
+	vehicle_filter.clear_elements();
+	vehicle_filter.append_element(new gui_scrolled_list_t::const_text_scrollitem_t(translator::translate("All"), COL_BLACK));
+	vehicle_filter.append_element(new gui_scrolled_list_t::const_text_scrollitem_t(translator::translate("Relevant"), COL_BLACK));
+
+	FOR(vector_tpl<ware_besch_t const*>, const i, get_welt()->get_goods_list()) {
+		vehicle_filter.append_element(new gui_scrolled_list_t::const_text_scrollitem_t(translator::translate(i->get_name()), COL_BLACK));
+	}
+
+	if (depot->selected_filter > vehicle_filter.count_elements()) {
+		depot->selected_filter = VEHICLE_FILTER_RELEVANT;
+	}
+	vehicle_filter.set_selection(depot->selected_filter);
 }
 
 
@@ -891,11 +932,9 @@ void depot_frame_t::rename_convoy(convoihandle_t cnv)
 sint32 depot_frame_t::calc_restwert(const vehikel_besch_t *veh_type)
 {
 	sint32 wert = 0;
-
-	slist_iterator_tpl<vehikel_t *> iter(depot->get_vehicle_list());
-	while(iter.next()) {
-		if(iter.get_current()->get_besch() == veh_type) {
-			wert += iter.get_current()->calc_restwert();
+	FOR(slist_tpl<vehikel_t*>, const v, depot->get_vehicle_list()) {
+		if (v->get_besch() == veh_type) {
+			wert += v->calc_restwert();
 		}
 	}
 	return wert;
@@ -1030,6 +1069,8 @@ bool depot_frame_t::action_triggered( gui_action_creator_t *komp,value_t p)
 				depot->set_selected_line(selected_line);
 				line_selector.set_selection( 0 );
 			}
+		} else if(komp == &vehicle_filter) {
+			depot->selected_filter = vehicle_filter.get_selection();
 		}
 		else {
 			return false;
@@ -1074,12 +1115,11 @@ bool depot_frame_t::infowin_event(const event_t *ev)
 			 * Replace our depot_frame_t with a new at the same position.
 			 * Volker Meyer
 			 */
-			int x = win_get_posx(this);
-			int y = win_get_posy(this);
+			koord const pos = win_get_pos(this);
 			destroy_win( this );
 
 			next_dep->zeige_info();
-			win_set_pos( win_get_magic((long)next_dep), x, y );
+			win_set_pos(win_get_magic((long)next_dep), pos.x, pos.y);
 			get_welt()->change_world_position(next_dep->get_pos());
 		}
 		else {
@@ -1122,53 +1162,55 @@ void depot_frame_t::zeichnen(koord pos, koord groesse)
 
 	if(cnv.is_bound()) {
 		if(cnv->get_vehikel_anzahl() > 0) {
-			uint32 total_power=0;
-			uint32 total_empty_weight=0;
-			uint32 total_max_weight=0;
-			uint32 total_min_weight=0;
-			sint32 empty_speed=0;
-			sint32 max_speed=0;
-			sint32 min_speed=0;
-			for( unsigned i=0;  i<cnv->get_vehikel_anzahl();  i++) {
-				const vehikel_besch_t *besch = cnv->get_vehikel(i)->get_besch();
+			sint32 empty_kmh, max_kmh, min_kmh;
+			if(  cnv->front()->get_waytype() == air_wt  ) {
+				// flying aircraft have 0 friction --> speed not limited by power, so just use top_speed
+				empty_kmh = max_kmh = min_kmh = speed_to_kmh( cnv->get_min_top_speed() );
+			}
+			else {
+				uint32 total_power = 0;
+				uint32 total_empty_weight = 0;
+				uint32 total_max_weight = 0;
+				uint32 total_min_weight = 0;
+				for(  unsigned i=0;  i<cnv->get_vehikel_anzahl();  i++  ) {
+					const vehikel_besch_t *besch = cnv->get_vehikel(i)->get_besch();
 
-				total_power += besch->get_leistung()*besch->get_gear()/64;
-				total_min_weight += besch->get_gewicht();
-				total_max_weight += besch->get_gewicht();
-				total_empty_weight += besch->get_gewicht();
+					total_power += besch->get_leistung()*besch->get_gear()/64;
 
-				uint32 max_weight =0;
-				uint32 min_weight =100000;
-				for(uint32 j=0; j<warenbauer_t::get_waren_anzahl(); j++) {
-					const ware_besch_t *ware = warenbauer_t::get_info(j);
+					uint32 max_weight = 0;
+					uint32 min_weight = 100000;
+					for(  uint32 j=0;  j<warenbauer_t::get_waren_anzahl();  j++  ) {
+						const ware_besch_t *ware = warenbauer_t::get_info(j);
 
-					if(besch->get_ware()->get_catg_index()==ware->get_catg_index()) {
-						max_weight = max(max_weight, ware->get_weight_per_unit());
-						min_weight = min(min_weight, ware->get_weight_per_unit());
+						if(  besch->get_ware()->get_catg_index() == ware->get_catg_index()  ) {
+							max_weight = max(max_weight, (uint32)ware->get_weight_per_unit());
+							min_weight = min(min_weight, (uint32)ware->get_weight_per_unit());
+						}
 					}
+					total_empty_weight += besch->get_gewicht();
+					total_max_weight += besch->get_gewicht() + (max_weight*besch->get_zuladung()+499)/1000;
+					total_min_weight += besch->get_gewicht() + (min_weight*besch->get_zuladung()+499)/1000;
 				}
-				total_max_weight += (max_weight*besch->get_zuladung()+499)/1000;
-				total_min_weight += (min_weight*besch->get_zuladung()+499)/1000;
+
+				const sint32 cnv_min_top_kmh = speed_to_kmh( cnv->get_min_top_speed() );
+				empty_kmh = total_power <= total_empty_weight ? 1 : min( cnv_min_top_kmh, sqrt_i32(((total_power<<8)/total_empty_weight-(1<<8))<<8)*50 >>8 );
+				max_kmh = total_power <= total_min_weight ? 1 : min( cnv_min_top_kmh, sqrt_i32(((total_power<<8)/total_min_weight-(1<<8))<<8)*50 >>8 );
+				min_kmh = total_power <= total_max_weight ? 1 : min( cnv_min_top_kmh, sqrt_i32(((total_power<<8)/total_max_weight-(1<<8))<<8)*50 >>8 );
 			}
-			// ensure that argument of sqrt is not negative
-			const sint32 cnv_min_top_speed = speed_to_kmh( cnv->get_min_top_speed() );
-			empty_speed = total_power < total_empty_weight ? 0 : min( cnv_min_top_speed, sqrt( (double)total_power/total_empty_weight - 1)*50 );
-			max_speed = total_power < total_min_weight ? 0 : min( cnv_min_top_speed, sqrt( (double)total_power/total_min_weight - 1)*50 );
-			min_speed = total_power < total_max_weight ? 0 : min( cnv_min_top_speed, sqrt( (double)total_power/total_max_weight - 1)*50 );
 
 			sprintf(txt_convoi_count, "%s %d (%s %i)",
 				translator::translate("Fahrzeuge:"), cnv->get_vehikel_anzahl(),
 				translator::translate("Station tiles:"), cnv->get_tile_length() );
-			if(  empty_speed != min_speed  ) {
-				if(  max_speed != min_speed  ) {
-					sprintf( txt_convoi_speed, "%s %d km/h, %d-%d km/h %s", translator::translate("Max. speed:"), empty_speed, min_speed, max_speed, translator::translate("loaded") );
+			if(  empty_kmh != min_kmh  ) {
+				if(  max_kmh != min_kmh  ) {
+					sprintf( txt_convoi_speed, "%s %d km/h, %d-%d km/h %s", translator::translate("Max. speed:"), empty_kmh, min_kmh, max_kmh, translator::translate("loaded") );
 				}
 				else {
-					sprintf( txt_convoi_speed, "%s %d km/h, %d km/h %s", translator::translate("Max. speed:"), empty_speed, min_speed, translator::translate("loaded") );
+					sprintf( txt_convoi_speed, "%s %d km/h, %d km/h %s", translator::translate("Max. speed:"), empty_kmh, min_kmh, translator::translate("loaded") );
 				}
 			}
 			else {
-					sprintf( txt_convoi_speed, "%s %d km/h", translator::translate("Max. speed:"), empty_speed );
+					sprintf( txt_convoi_speed, "%s %d km/h", translator::translate("Max. speed:"), empty_kmh );
 			}
 			sprintf(txt_convoi_value, "%s %ld$", translator::translate("Restwert:"), (long)(cnv->calc_restwert()/100) );
 			if(  cnv->get_line().is_bound()  &&  cnv->get_line()->get_schedule()->matches( get_welt(),cnv->get_schedule() )  ) {
@@ -1287,7 +1329,7 @@ void depot_frame_t::draw_vehicle_info_text(koord pos)
 		}
 	}
 
-	switch(depot->vehicle_count()) {
+	switch (uint32 const count = depot->get_vehicle_list().get_count()) {
 		case 0:
 			c = translator::translate("Keine Einzelfahrzeuge im Depot");
 			break;
@@ -1295,7 +1337,7 @@ void depot_frame_t::draw_vehicle_info_text(koord pos)
 			c = translator::translate("1 Einzelfahrzeug im Depot");
 			break;
 		default:
-			sprintf(buf, translator::translate("%d Einzelfahrzeuge im Depot"), depot->vehicle_count());
+			sprintf(buf, translator::translate("%d Einzelfahrzeuge im Depot"), count);
 			c = buf;
 			break;
 	}
@@ -1350,7 +1392,7 @@ void depot_frame_t::draw_vehicle_info_text(koord pos)
 				veh_type->get_geschw()
 				);
 		}
-		display_multiline_text( pos.x + 4, pos.y + tabs.get_pos().y + tabs.get_groesse().y + 31 + LINESPACE*1 + 4, buf,  COL_BLACK);
+		display_multiline_text( pos.x + 4, pos.y + tabs.get_pos().y + tabs.get_groesse().y + 31 + LINESPACE*1 + 4 + 16, buf,  COL_BLACK);
 
 		// column 2
 		int n = sprintf(buf, "%s %s %04d\n",
@@ -1389,7 +1431,7 @@ void depot_frame_t::draw_vehicle_info_text(koord pos)
 			sprintf(buf + strlen(buf), "%s %d Cr", translator::translate("Restwert:"), 	value);
 		}
 
-		display_multiline_text( pos.x + 200, pos.y + tabs.get_pos().y + tabs.get_groesse().y + 31 + LINESPACE*2 + 4, buf, COL_BLACK);
+		display_multiline_text( pos.x + 200, pos.y + tabs.get_pos().y + tabs.get_groesse().y + 31 + LINESPACE*2 + 4 + 16, buf, COL_BLACK);
 	}
 	POP_CLIP();
 }
