@@ -52,7 +52,7 @@ public:
 	factory_chart_t(const fabrik_t *_factory);
 	virtual ~factory_chart_t();
 
-	virtual bool action_triggered(gui_action_creator_t *komp, value_t extra);
+	bool action_triggered(gui_action_creator_t*, value_t) OVERRIDE;
 
 	virtual void zeichnen(koord pos);
 };
