@@ -6,26 +6,25 @@
  * Juli 2000
  */
 
-#include "../simconvoi.h"
 #include "../simline.h"
-#include "../simlinemgmt.h"
 #include "../simcolor.h"
-#include "../simwin.h"
 #include "../simhalt.h"
 #include "../simskin.h"
 #include "../simworld.h"
 #include "../simmenu.h"
 #include "../simgraph.h"
-#include "../simtools.h"
 
 #include "../utils/simstring.h"
 #include "../utils/cbuffer_t.h"
 
 #include "../boden/grund.h"
 
+#include "../dings/zeiger.h"
+
 #include "../dataobj/fahrplan.h"
 #include "../dataobj/loadsave.h"
 #include "../dataobj/translator.h"
+#include "../dataobj/umgebung.h"
 
 #include "../player/simplay.h"
 #include "../vehicle/simvehikel.h"
@@ -34,7 +33,7 @@
 
 #include "fahrplan_gui.h"
 #include "line_item.h"
-#include "components/list_button.h"
+
 #include "components/gui_button.h"
 #include "karte.h"
 
@@ -42,51 +41,87 @@ char fahrplan_gui_t::no_line[128];	// contains the current translation of "<no l
 karte_t *fahrplan_gui_t::welt = NULL;
 
 
-/**
- * Fills buf with description of schedule's i'th entry.
- *
- * @author Hj. Malthaner
- */
-void fahrplan_gui_t::gimme_stop_name(cbuffer_t & buf, karte_t *welt, const spieler_t *sp, const linieneintrag_t &entry )
+// shows/deletes highliting of tiles
+void fahrplan_gui_stats_t::highlight_schedule( schedule_t *markfpl, bool marking )
 {
-	halthandle_t halt = haltestelle_t::get_halt(welt, entry.pos, sp);
-	if(halt.is_bound()) {
-		if (entry.ladegrad != 0) {
-			buf.printf("%d%% %s (%s)", entry.ladegrad, halt->get_name(), entry.pos.get_str() );
-		}
-		else {
-			buf.printf("%s (%s)",
-				halt->get_name(),
-				entry.pos.get_str() );
+	marking &= umgebung_t::visualize_schedule;
+	FOR(minivec_tpl<linieneintrag_t>, const& i, markfpl->eintrag) {
+		if (grund_t* const gr = welt->lookup(i.pos)) {
+			for(  uint idx=0;  idx<gr->get_top();  idx++  ) {
+				ding_t *d = gr->obj_bei(idx);
+				if(  marking  ) {
+					if(  !d->is_moving()  ) {
+						d->set_flag( ding_t::highlight );
+					}
+				}
+				else {
+					d->clear_flag( ding_t::highlight );
+				}
+			}
+			gr->set_flag( grund_t::dirty );
+			// here on water
+			if(  gr->ist_wasser()  ||  gr->ist_natur()  ) {
+				if(  marking  ) {
+					gr->set_flag( grund_t::marked );
+				}
+				else {
+					gr->clear_flag( grund_t::marked );
+				}
+			}
+
 		}
 	}
-	else {
-		const grund_t* gr = welt->lookup(entry.pos);
-		if(gr==NULL) {
-			buf.printf("%s (%s)", translator::translate("Invalid coordinate"), entry.pos.get_str() );
-		}
-		else if(gr->get_depot() != NULL) {
-			buf.printf("%s (%s)", translator::translate("Depot"), entry.pos.get_str() );
-		}
-		else {
-			buf.printf("%s (%s)", translator::translate("Wegpunkt"), entry.pos.get_str() );
+	// always remove
+	if(  grund_t *old_gr = welt->lookup(aktuell_mark->get_pos())  ) {
+		aktuell_mark->mark_image_dirty( aktuell_mark->get_bild(), 0 );
+		old_gr->obj_remove( aktuell_mark );
+		old_gr->set_flag( grund_t::dirty );
+		aktuell_mark->set_pos( koord3d::invalid );
+	}
+	// add if required
+	if(  marking  &&  markfpl->get_aktuell() < markfpl->get_count() ) {
+		aktuell_mark->set_pos( markfpl->eintrag[markfpl->get_aktuell()].pos );
+		if(  grund_t *gr = welt->lookup(aktuell_mark->get_pos())  ) {
+			gr->obj_add( aktuell_mark );
+			aktuell_mark->set_flag( ding_t::dirty );
+			gr->set_flag( grund_t::dirty );
 		}
 	}
+	aktuell_mark->clear_flag( ding_t::highlight );
 }
 
 
 /**
- * Fills buf with description of schedule's i'th entry.
- * short version, without loading level and position ...
- * @author Hj. Malthaner
+ * Append description of entry to buf.
  */
-void fahrplan_gui_t::gimme_short_stop_name(cbuffer_t &buf, karte_t *welt, const spieler_t *sp, const schedule_t *fpl, int i, int max_chars)
+static void gimme_stop_name(cbuffer_t& buf, karte_t* const welt, spieler_t const* const sp, linieneintrag_t const& entry)
 {
-	if(i<0  ||  fpl==NULL  ||  i>=fpl->get_count()) {
-		dbg->warning("void fahrplan_gui_t::gimme_short_stop_name()","tried to receive unused entry %i in schedule %p.",i,fpl);
-		return;
+	char const* what;
+	halthandle_t const halt = haltestelle_t::get_halt(welt, entry.pos, sp);
+	if (halt.is_bound()) {
+		if (entry.ladegrad != 0) {
+			buf.printf("%d%% ", entry.ladegrad);
+		}
+		what = halt->get_name();
+	} else {
+		grund_t const* const gr = welt->lookup(entry.pos);
+		if (!gr) {
+			what = translator::translate("Invalid coordinate");
+		} else if (gr->get_depot()) {
+			what = translator::translate("Depot");
+		} else if (char const* const label_text = gr->get_text()) {
+			buf.printf("%s ", translator::translate("Wegpunkt"));
+			what = label_text;
+		} else {
+			what = translator::translate("Wegpunkt");
+		}
 	}
-	const linieneintrag_t& entry = fpl->eintrag[i];
+	buf.printf("%s (%s)", what, entry.pos.get_str());
+}
+
+
+void fahrplan_gui_t::gimme_short_stop_name(cbuffer_t& buf, karte_t* const welt, spieler_t const* const sp, linieneintrag_t const& entry, int const max_chars)
+{
 	const char *p;
 	halthandle_t halt = haltestelle_t::get_halt(welt, entry.pos, sp);
 	if(halt.is_bound()) {
@@ -115,34 +150,71 @@ void fahrplan_gui_t::gimme_short_stop_name(cbuffer_t &buf, karte_t *welt, const
 
 
 
+zeiger_t *fahrplan_gui_stats_t::aktuell_mark = NULL;
 karte_t *fahrplan_gui_stats_t::welt = NULL;
 cbuffer_t fahrplan_gui_stats_t::buf;
 
 void fahrplan_gui_stats_t::zeichnen(koord offset)
 {
-	if(fpl) {
-		sint16 width = get_groesse().x-16;
+	if (!fpl) return;
 
-		for (int i = 0; i < fpl->get_count(); i++) {
-
-			buf.clear();
-			buf.printf( "%i) ", i+1 );
-			fahrplan_gui_t::gimme_stop_name( buf, welt, sp, fpl->eintrag[i] );
-			sint16 w = display_proportional_clip(offset.x + 4 + 10, offset.y + i * (LINESPACE + 1), buf, ALIGN_LEFT, COL_BLACK, true);
-			if(  w>width  ) {
-				width = w;
+	if (fpl->empty()) {
+		buf.clear();
+		buf.append(translator::translate("Please click on the map to add\nwaypoints or stops to this\nschedule."));
+		sint16 const width = display_multiline_text(offset.x + 4, offset.y, buf, COL_WHITE);
+		set_groesse(koord(width + 4 + 16, 3 * LINESPACE));
+	} else {
+		int    i     = 0;
+		size_t sel   = fpl->get_aktuell();
+		sint16 width = get_groesse().x - 16;
+		FORX(minivec_tpl<linieneintrag_t>, const& e, fpl->eintrag, (--sel, offset.y += LINESPACE + 1)) {
+			if (sel == 0) {
+				// highlight current entry (width is just wide enough, scrolly will do clipping)
+				display_fillbox_wh_clip(offset.x, offset.y - 1, 2048, LINESPACE + 1, sp->get_player_color1() + 1, false);
 			}
 
+			buf.clear();
+			buf.printf("%i) ", ++i);
+			gimme_stop_name(buf, welt, sp, e);
+			PLAYER_COLOR_VAL const c = sel == 0 ? COL_BLACK : COL_WHITE;
+			sint16           const w = display_proportional_clip(offset.x + 4 + 10, offset.y, buf, ALIGN_LEFT, c, true);
+			if (width < w) width = w;
+
 			// the goto button (right arrow)
-			display_color_img( i!=fpl->get_aktuell() ? button_t::arrow_right_normal : button_t::arrow_right_pushed,
-				offset.x + 2, offset.y + i * (LINESPACE + 1), 0, false, true);
+			image_id const img = sel == 0 ? button_t::arrow_right_normal : button_t::arrow_right_pushed;
+			display_color_img(img, offset.x + 2, offset.y, 0, false, true);
 		}
-		set_groesse( koord(width+16, fpl->get_count() * (LINESPACE + 1) ) );
+		set_groesse(koord(width + 16, fpl->get_count() * (LINESPACE + 1)));
+		highlight_schedule(fpl, true);
 	}
 }
 
 
 
+fahrplan_gui_stats_t::fahrplan_gui_stats_t(karte_t* w, spieler_t *s)
+{
+	welt = w;
+	fpl = NULL;
+	sp = s;
+	if(  aktuell_mark==NULL  ) {
+		aktuell_mark = new zeiger_t( welt, koord3d::invalid, NULL );
+		aktuell_mark->set_bild( werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]->cursor );
+	}
+}
+
+
+
+fahrplan_gui_stats_t::~fahrplan_gui_stats_t()
+{
+	if(  grund_t *gr = welt->lookup(aktuell_mark->get_pos())  ) {
+		aktuell_mark->mark_image_dirty( aktuell_mark->get_bild(), 0 );
+		gr->obj_remove(aktuell_mark);
+	}
+	aktuell_mark->set_pos( koord3d::invalid );
+}
+
+
+
 fahrplan_gui_t::~fahrplan_gui_t()
 {
 	if(  sp  ) {
@@ -188,16 +260,16 @@ fahrplan_gui_t::fahrplan_gui_t(schedule_t* fpl_, spieler_t* sp_, convoihandle_t
 		lb_line.set_pos(koord(10, ypos+2));
 		add_komponente(&lb_line);
 
-		bt_promote_to_line.init(button_t::roundbox, "promote to line", koord( BUTTON3_X, ypos ), koord(BUTTON_WIDTH,BUTTON_HEIGHT) );
+		bt_promote_to_line.init(button_t::roundbox, "promote to line", koord( BUTTON3_X, ypos ), koord(D_BUTTON_WIDTH,D_BUTTON_HEIGHT) );
 		bt_promote_to_line.set_tooltip("Create a new line based on this schedule");
 		bt_promote_to_line.add_listener(this);
 		add_komponente(&bt_promote_to_line);
 
-		ypos += BUTTON_HEIGHT+1;
+		ypos += D_BUTTON_HEIGHT+1;
 
 		line_selector.set_pos(koord(2, ypos));
-		line_selector.set_groesse(koord(BUTTON4_X-2, BUTTON_HEIGHT));
-		line_selector.set_max_size(koord(BUTTON4_X-2, 13*LINESPACE+TITLEBAR_HEIGHT-1));
+		line_selector.set_groesse(koord(BUTTON4_X-2, D_BUTTON_HEIGHT));
+		line_selector.set_max_size(koord(BUTTON4_X-2, 13*LINESPACE+D_TITLEBAR_HEIGHT-1));
 		line_selector.set_highlight_color(sp->get_player_color1() + 1);
 		line_selector.clear_elements();
 
@@ -206,28 +278,28 @@ fahrplan_gui_t::fahrplan_gui_t(schedule_t* fpl_, spieler_t* sp_, convoihandle_t
 		line_selector.add_listener(this);
 		add_komponente(&line_selector);
 
-		ypos += BUTTON_HEIGHT+3;
+		ypos += D_BUTTON_HEIGHT+3;
 	}
 
 	// loading level and return tickets
 	lb_load.set_pos( koord( 10, ypos+2 ) );
 	add_komponente(&lb_load);
 
-	numimp_load.set_pos( koord( BUTTON_WIDTH*2-65, ypos+2 ) );
-	numimp_load.set_groesse( koord( 60, BUTTON_HEIGHT ) );
+	numimp_load.set_pos( koord( D_BUTTON_WIDTH*2-65, ypos+2 ) );
+	numimp_load.set_groesse( koord( 60, D_BUTTON_HEIGHT ) );
 	numimp_load.set_value( fpl->get_current_eintrag().ladegrad );
 	numimp_load.set_limits( 0, 100 );
 	numimp_load.set_increment_mode( gui_numberinput_t::PROGRESS );
 	numimp_load.add_listener(this);
 	add_komponente(&numimp_load);
 
-	ypos += BUTTON_HEIGHT;
+	ypos += D_BUTTON_HEIGHT;
 
 	// waiting in parts per month
 	lb_wait.set_pos( koord( 10, ypos+2 ) );
 	add_komponente(&lb_wait);
 
-	bt_wait_prev.set_pos( koord( BUTTON_WIDTH*2-65, ypos+2 ) );
+	bt_wait_prev.set_pos( koord( D_BUTTON_WIDTH*2-65, ypos+2 ) );
 	bt_wait_prev.set_typ(button_t::arrowleft);
 	bt_wait_prev.add_listener(this);
 	add_komponente(&bt_wait_prev);
@@ -239,40 +311,40 @@ fahrplan_gui_t::fahrplan_gui_t(schedule_t* fpl_, spieler_t* sp_, convoihandle_t
 		sprintf( str_parts_month, "1/%d",  1<<(16-fpl->get_current_eintrag().waiting_time_shift) );
 	}
 	lb_waitlevel.set_text_pointer( str_parts_month );
-	lb_waitlevel.set_pos( koord( BUTTON_WIDTH*2-20, ypos+2 ) );
+	lb_waitlevel.set_pos( koord( D_BUTTON_WIDTH*2-20, ypos+2 ) );
 	add_komponente(&lb_waitlevel);
 
-	bt_wait_next.set_pos( koord( BUTTON_WIDTH*2-15, ypos+2 ) );
+	bt_wait_next.set_pos( koord( D_BUTTON_WIDTH*2-15, ypos+2 ) );
 	bt_wait_next.set_typ(button_t::arrowright);
 	bt_wait_next.add_listener(this);
 	add_komponente(&bt_wait_next);
 
-	bt_return.init(button_t::roundbox, "return ticket", koord(BUTTON3_X, ypos ), koord(BUTTON_WIDTH,BUTTON_HEIGHT) );
+	bt_return.init(button_t::roundbox, "return ticket", koord(BUTTON3_X, ypos ), koord(D_BUTTON_WIDTH,D_BUTTON_HEIGHT) );
 	bt_return.set_tooltip("Add stops for backward travel");
 	bt_return.add_listener(this);
 	add_komponente(&bt_return);
 
-	ypos += BUTTON_HEIGHT;
+	ypos += D_BUTTON_HEIGHT;
 
-	bt_add.init(button_t::roundbox_state, "Add Stop", koord(BUTTON1_X, ypos ), koord(BUTTON_WIDTH,BUTTON_HEIGHT) );
+	bt_add.init(button_t::roundbox_state, "Add Stop", koord(BUTTON1_X, ypos ), koord(D_BUTTON_WIDTH,D_BUTTON_HEIGHT) );
 	bt_add.set_tooltip("Appends stops at the end of the schedule");
 	bt_add.add_listener(this);
 	bt_add.pressed = true;
 	add_komponente(&bt_add);
 
-	bt_insert.init(button_t::roundbox_state, "Ins Stop", koord(BUTTON2_X, ypos ), koord(BUTTON_WIDTH,BUTTON_HEIGHT) );
+	bt_insert.init(button_t::roundbox_state, "Ins Stop", koord(BUTTON2_X, ypos ), koord(D_BUTTON_WIDTH,D_BUTTON_HEIGHT) );
 	bt_insert.set_tooltip("Insert stop before the current stop");
 	bt_insert.add_listener(this);
 	bt_insert.pressed = false;
 	add_komponente(&bt_insert);
 
-	bt_remove.init(button_t::roundbox_state, "Del Stop", koord(BUTTON3_X, ypos ), koord(BUTTON_WIDTH,BUTTON_HEIGHT) );
+	bt_remove.init(button_t::roundbox_state, "Del Stop", koord(BUTTON3_X, ypos ), koord(D_BUTTON_WIDTH,D_BUTTON_HEIGHT) );
 	bt_remove.set_tooltip("Delete the current stop");
 	bt_remove.add_listener(this);
 	bt_remove.pressed = false;
 	add_komponente(&bt_remove);
 
-	ypos += BUTTON_HEIGHT+2;
+	ypos += D_BUTTON_HEIGHT+2;
 
 	scrolly.set_pos( koord( 0, ypos ) );
 	scrolly.set_show_scroll_x(true);
@@ -282,8 +354,8 @@ fahrplan_gui_t::fahrplan_gui_t(schedule_t* fpl_, spieler_t* sp_, convoihandle_t
 	mode = adding;
 	update_selection();
 
-	set_fenstergroesse( koord(BUTTON4_X, ypos+BUTTON_HEIGHT+min(15,fpl->get_count())*(LINESPACE + 1)+TITLEBAR_HEIGHT) );
-	set_min_windowsize( koord(BUTTON4_X, ypos+BUTTON_HEIGHT+3*(LINESPACE + 1)+TITLEBAR_HEIGHT) );
+	set_fenstergroesse( koord(BUTTON4_X, ypos+D_BUTTON_HEIGHT+(fpl->get_count()>0 ? min(15,fpl->get_count()) : 15)*(LINESPACE+1)+D_TITLEBAR_HEIGHT) );
+	set_min_windowsize( koord(BUTTON4_X, ypos+D_BUTTON_HEIGHT+3*(LINESPACE+1)+D_TITLEBAR_HEIGHT) );
 
 	set_resizemode(diagonal_resize);
 	resize( koord(0,0) );
@@ -374,6 +446,7 @@ bool fahrplan_gui_t::infowin_event(const event_t *ev)
 				else if(ev->mx<scrolly.get_groesse().x-11) {
 					fpl->set_aktuell( line );
 					if(mode == removing) {
+						stats.highlight_schedule( fpl, false );
 						fpl->remove();
 						action_triggered( &bt_add, value_t() );
 					}
@@ -384,6 +457,10 @@ bool fahrplan_gui_t::infowin_event(const event_t *ev)
 	}
 	else if(ev->ev_class == INFOWIN  &&  ev->ev_code == WIN_CLOSE  &&  fpl!=NULL  ) {
 
+		for(  int i=0;  i<fpl->get_count();  i++  ) {
+			stats.highlight_schedule( fpl, false );
+		}
+
 		update_werkzeug( false );
 		fpl->cleanup();
 		fpl->eingabe_abschliessen();
@@ -432,27 +509,28 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 		bt_insert.pressed = false;
 		bt_remove.pressed = false;
 		update_werkzeug( true );
-
-	} else if(komp == &bt_insert) {
+	}
+	else if(komp == &bt_insert) {
 		mode = inserting;
 		bt_add.pressed = false;
 		bt_insert.pressed = true;
 		bt_remove.pressed = false;
 		update_werkzeug( true );
-
-	} else if(komp == &bt_remove) {
+	}
+	else if(komp == &bt_remove) {
 		mode = removing;
 		bt_add.pressed = false;
 		bt_insert.pressed = false;
 		bt_remove.pressed = true;
 		update_werkzeug( false );
-
-	} else if(komp == &numimp_load) {
+	}
+	else if(komp == &numimp_load) {
 		if (!fpl->empty()) {
-			fpl->eintrag[fpl->get_aktuell()].ladegrad = p.i;
+			fpl->eintrag[fpl->get_aktuell()].ladegrad = (uint8)p.i;
 			update_selection();
 		}
-	} else if(komp == &bt_wait_prev) {
+	}
+	else if(komp == &bt_wait_prev) {
 		if (!fpl->empty()) {
 			sint8& wait = fpl->eintrag[fpl->get_aktuell()].waiting_time_shift;
 			if(wait>7) {
@@ -466,7 +544,8 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 			}
 			update_selection();
 		}
-	} else if(komp == &bt_wait_next) {
+	}
+	else if(komp == &bt_wait_next) {
 		if (!fpl->empty()) {
 			sint8& wait = fpl->eintrag[fpl->get_aktuell()].waiting_time_shift;
 			if(wait==0) {
@@ -480,13 +559,16 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 			}
 			update_selection();
 		}
-	} else if (komp == &bt_return) {
+	}
+	else if (komp == &bt_return) {
 		fpl->add_return_way();
-	} else if (komp == &line_selector) {
+	}
+	else if (komp == &line_selector) {
 		int selection = p.i;
 //DBG_MESSAGE("fahrplan_gui_t::action_triggered()","line selection=%i",selection);
 		if(  (uint32)(selection-1)<(uint32)line_selector.count_elements()  ) {
 			new_line = lines[selection - 1];
+			stats.highlight_schedule( fpl, false );
 			fpl->copy_from( new_line->get_schedule() );
 			fpl->eingabe_beginnen();
 		}
@@ -495,7 +577,8 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 			new_line = linehandle_t();
 			line_selector.set_selection( 0 );
 		}
-	} else if (komp == &bt_promote_to_line) {
+	}
+	else if (komp == &bt_promote_to_line) {
 		// update line schedule via tool!
 		werkzeug_t *w = create_tool( WKZ_LINE_TOOL | SIMPLE_TOOL );
 		cbuffer_t buf;
@@ -541,8 +624,7 @@ void fahrplan_gui_t::init_line_selector()
 		}
 	}
 
-	for (vector_tpl<linehandle_t>::const_iterator i = lines.begin(), end = lines.end(); i != end; i++) {
-		linehandle_t line = *i;
+	FOR(vector_tpl<linehandle_t>, const line, lines) {
 		line_selector.append_element( new line_scrollitem_t(line) );
 		if(  !new_line.is_bound()  ) {
 			if(  fpl->matches( sp->get_welt(), line->get_schedule() )  ) {
@@ -576,6 +658,8 @@ void fahrplan_gui_t::zeichnen(koord pos, koord gr)
 		cnv->call_convoi_tool( 's', "1" );
 	}
 
+	// always dirty, to cater for shortening of halt names and change of selections
+	set_dirty();
 	gui_frame_t::zeichnen(pos,gr);
 }
 
@@ -660,20 +744,21 @@ void fahrplan_gui_t::rdwr(loadsave_t *file)
 		}
 		if(  !cnv.is_bound() ) {
 			// not found (most likely convoi in depot ... )
-			for(  vector_tpl<convoihandle_t>::const_iterator i=welt->convois_begin(); i!=welt->convois_end();  ++i  ) {
-				if(  (*i)->get_besitzer()->get_player_nr()==player_nr  &&  strncmp( (*i)->get_name(), cnv_name, 256 )==0  &&  old_fpl->matches( welt, (*i)->get_schedule() )  ) {
+			FOR(vector_tpl<convoihandle_t>, const i, welt->convoys()) {
+				if (i->get_besitzer()->get_player_nr()    == player_nr &&
+						strncmp(i->get_name(), cnv_name, 256) == 0         &&
+						old_fpl->matches(welt, i->get_schedule())) {
 					// valid convoi found
-					cnv = *i;
+					cnv = i;
 					break;
 				}
 			}
 		}
 		if(  cnv.is_bound() ) {
 			// now we can open the window ...
-			KOORD_VAL xpos = win_get_posx( this );
-			KOORD_VAL ypos = win_get_posy( this );
+			koord const& pos = win_get_pos(this);
 			fahrplan_gui_t *w = new fahrplan_gui_t( cnv->get_schedule(), cnv->get_besitzer(), cnv );
-			create_win( xpos, ypos, w, w_info, (long)cnv->get_schedule() );
+			create_win(pos.x, pos.y, w, w_info, (long)cnv->get_schedule());
 			w->set_fenstergroesse( gr );
 			w->fpl->copy_from( fpl );
 			cnv->get_schedule()->eingabe_abschliessen();
