@@ -31,7 +31,8 @@ static char const* const version[] =
 	"0.102.5",
 	"0.110.0",
 	"0.110.1",
-	"0.111.0"
+	"0.111.0",
+	"0.111.1"
 };
 
 
@@ -71,6 +72,7 @@ void settings_general_stats_t::init(settings_t const* const sets)
 	INIT_NUM( "autosave", umgebung_t::autosave, 0, 12, gui_numberinput_t::AUTOLINEAR, false );
 	INIT_NUM( "frames_per_second",umgebung_t::fps, 10, 25, gui_numberinput_t::AUTOLINEAR, false );
 	INIT_NUM( "fast_forward", umgebung_t::max_acceleration, 1, 1000, gui_numberinput_t::AUTOLINEAR, false );
+	INIT_NUM( "simple_drawing_tile_size",umgebung_t::simple_drawing_tile_size, 0, 256, gui_numberinput_t::POWER2, false );
 	SEPERATOR
 	INIT_BOOL( "numbered_stations", sets->get_numbered_stations() );
 	INIT_NUM( "show_names", umgebung_t::show_names, 0, 3, gui_numberinput_t::AUTOLINEAR, true );
@@ -111,7 +113,7 @@ void settings_general_stats_t::init(settings_t const* const sets)
 	SEPERATOR
 	// combobox for savegame version
 	savegame.set_pos( koord(2,ypos-2) );
-	savegame.set_groesse( koord(70,BUTTON_HEIGHT) );
+	savegame.set_groesse( koord(70,D_BUTTON_HEIGHT) );
 	for(  uint32 i=0;  i<lengthof(version);  i++  ) {
 		savegame.append_element( new gui_scrolled_list_t::const_text_scrollitem_t( version[i]+2, COL_BLACK ) );
 		if(  strcmp(version[i],umgebung_t::savegame_version_str)==0  ) {
@@ -137,6 +139,7 @@ void settings_general_stats_t::read(settings_t* const sets)
 	READ_NUM_VALUE( umgebung_t::autosave );
 	READ_NUM_VALUE( umgebung_t::fps );
 	READ_NUM_VALUE( umgebung_t::max_acceleration );
+	READ_NUM_VALUE( umgebung_t::simple_drawing_tile_size );
 
 	READ_BOOL_VALUE( sets->numbered_stations );
 	READ_NUM_VALUE( umgebung_t::show_names );
@@ -234,6 +237,7 @@ void settings_economy_stats_t::init(settings_t const* const sets)
 	INIT_INIT
 	INIT_COST( "starting_money", sets->get_starting_money(sets->get_starting_year()), 1, 0x7FFFFFFFul, 10000, false );
 	INIT_NUM( "pay_for_total_distance", sets->get_pay_for_total_distance_mode(), 0, 2, gui_numberinput_t::AUTOLINEAR, true );
+	INIT_NUM( "bonus_basefactor", sets->get_bonus_basefactor(), 0, 1000, gui_numberinput_t::AUTOLINEAR, false );
 	INIT_BOOL_NEW( "first_beginner", sets->get_beginner_mode() );
 	INIT_NUM( "beginner_price_factor", sets->get_beginner_price_factor(), 1, 25000, gui_numberinput_t::AUTOLINEAR, false );
 	INIT_BOOL( "allow_buying_obsolete_vehicles", sets->get_allow_buying_obsolete_vehicles() );
@@ -308,6 +312,7 @@ void settings_economy_stats_t::read(settings_t* const sets)
 		sets->starting_money = start_money_temp;
 	}
 	READ_NUM_VALUE( sets->pay_for_total_distance );
+	READ_NUM_VALUE( sets->bonus_basefactor );
 	READ_BOOL_VALUE_NEW( sets->beginner_mode );
 	READ_NUM_VALUE( sets->beginner_price_factor );
 	READ_BOOL_VALUE( sets->allow_buying_obsolete_vehicles );
@@ -379,6 +384,7 @@ void settings_costs_stats_t::init(settings_t const* const sets)
 void settings_costs_stats_t::read(settings_t* const sets)
 {
 	READ_INIT
+	(void)booliter;
 	READ_NUM_VALUE( sets->maint_building );
 	READ_COST_VALUE( sets->cst_multiply_dock )*(-1);
 	READ_COST_VALUE( sets->cst_multiply_station )*(-1);
@@ -437,9 +443,8 @@ void settings_climates_stats_t::init(settings_t* const sets)
 	INIT_NUM_NEW( "minimum length of rivers", sets->get_min_river_length(), 0, max(16,sets->get_max_river_length())-16, gui_numberinput_t::AUTOLINEAR, false );
 	INIT_NUM_NEW( "maximum length of rivers", sets->get_max_river_length(), sets->get_min_river_length()+16, 8196, gui_numberinput_t::AUTOLINEAR, false );
 	// add listener to all of them
-	slist_iterator_tpl<gui_numberinput_t *>iter(numinp);
-	while(  iter.next()  ) {
-		iter.get_current()->add_listener( this );
+	FOR(slist_tpl<gui_numberinput_t*>, const n, numinp) {
+		n->add_listener(this);
 	}
 	// the following are independent and thus need no listener
 	SEPERATOR
@@ -500,9 +505,9 @@ bool settings_climates_stats_t::action_triggered(gui_action_creator_t *komp, val
 {
 	welt_gui_t *welt_gui = dynamic_cast<welt_gui_t *>(win_get_magic( magic_welt_gui_t ));
 	read( local_sets );
-	slist_iterator_tpl<gui_numberinput_t *>iter(numinp);
-	for(  uint i=0;  iter.next();  i++  ) {
-		if(  iter.get_current()==komp  &&  i<3  &&  welt_gui  ) {
+	uint i = 0;
+	FORX(slist_tpl<gui_numberinput_t*>, const n, numinp, ++i) {
+		if (n == komp && i < 3 && welt_gui) {
 			// update world preview
 			welt_gui->update_preview();
 		}
