@@ -35,16 +35,7 @@ class trafficlight_info_t : public ding_infowin_t, public action_listener_t
 	 */
 	const char *get_hilfe_datei() const {return "trafficlight_info.txt";}
 
-	/**
-	 * This method is called if an action is triggered
-	 * @author Hj. Malthaner
-	 *
-	 * Returns true, if action is done and no more
-	 * components should be triggered.
-	 * V.Meyer
-	 */
-	bool action_triggered( gui_action_creator_t *komp, value_t extra);
-
+	bool action_triggered(gui_action_creator_t*, value_t) OVERRIDE;
 
 	// called, after external change
 	void update_data();
