@@ -54,11 +54,12 @@ ai_goods_t::ai_goods_t(karte_t *wl, uint8 nr) : ai_t(wl,nr)
 	ship_vehicle = NULL;
 	road_weg = NULL;
 
-	next_contruction_steps = welt->get_steps()+ 50;
+	next_construction_steps = welt->get_steps()+ 50;
 
-	road_transport = nr<7;
+	road_transport = nr!=6;
 	rail_transport = nr>2;
 	ship_transport = true;
+	air_transport = false;
 }
 
 
@@ -138,14 +139,13 @@ bool ai_goods_t::get_factory_tree_lowest_missing( fabrik_t *fab )
 			continue;
 		}
 
-		const vector_tpl <koord> & sources = fab->get_suppliers();
-		for( unsigned q=0;  q<sources.get_count();  q++  ) {
-			fabrik_t *qfab = fabrik_t::get_fab(welt,sources[q]);
+		FOR(vector_tpl<koord>, const& q, fab->get_suppliers()) {
+			fabrik_t* const qfab = fabrik_t::get_fab(welt, q);
 			const fabrik_besch_t* const fb = qfab->get_besch();
 			for (uint qq = 0; qq < fb->get_produkte(); qq++) {
-				if (fb->get_produkt(qq)->get_ware() == ware
-					  &&  !is_forbidden( fabrik_t::get_fab(welt,sources[q]), fab, ware )
-					  &&  !is_connected( sources[q], fab->get_pos().get_2d(), ware )  ) {
+				if (fb->get_produkt(qq)->get_ware() == ware &&
+						!is_forbidden(qfab, fab, ware)          &&
+						!is_connected(q, fab->get_pos().get_2d(), ware)) {
 					// find out how much is there
 					const array_tpl<ware_production_t>& ausgang = qfab->get_ausgang();
 					uint ware_nr;
@@ -181,21 +181,21 @@ int ai_goods_t::get_factory_tree_missing_count( fabrik_t *fab )
 {
 	int numbers=0;	// how many missing?
 
+	fabrik_besch_t const& d = *fab->get_besch();
 	// ok, this is a source ...
-	if(fab->get_besch()->get_lieferanten()==0) {
+	if (d.is_producer_only()) {
 		return 0;
 	}
 
 	// now check for all
-	for( int i=0;  i<fab->get_besch()->get_lieferanten();  i++  ) {
-		const ware_besch_t *ware = fab->get_besch()->get_lieferant(i)->get_ware();
+	for (int i = 0; i < d.get_lieferanten(); ++i) {
+		ware_besch_t const* const ware = d.get_lieferant(i)->get_ware();
 
 		bool complete = false;	// found at least one factory
-		const vector_tpl <koord> & sources = fab->get_suppliers();
-		for( unsigned q=0;  q<sources.get_count();  q++  ) {
-			fabrik_t *qfab = fabrik_t::get_fab(welt,sources[q]);
+		FOR(vector_tpl<koord>, const& q, fab->get_suppliers()) {
+			fabrik_t* const qfab = fabrik_t::get_fab(welt, q);
 			if(!qfab) {
-				dbg->error( "fabrik_t::get_fab()","fab %s at %s does not find supplier at %s.", fab->get_name(), fab->get_pos().get_str(), sources[q].get_str() );
+				dbg->error("fabrik_t::get_fab()","fab %s at %s does not find supplier at %s.", fab->get_name(), fab->get_pos().get_str(), q.get_str());
 				continue;
 			}
 			if( !is_forbidden( qfab, fab, ware ) ) {
@@ -205,7 +205,7 @@ int ai_goods_t::get_factory_tree_missing_count( fabrik_t *fab )
 						int n = get_factory_tree_missing_count( qfab );
 						if(n>=0) {
 							complete = true;
-							if(  !is_connected( sources[q], fab->get_pos().get_2d(), ware )  ) {
+							if (!is_connected(q, fab->get_pos().get_2d(), ware)) {
 								numbers += 1;
 							}
 							numbers += n;
@@ -262,12 +262,12 @@ bool ai_goods_t::suche_platz1_platz2(fabrik_t *qfab, fabrik_t *zfab, int length
 				add_neighbourhood( one_more, 1 );
 				// Any halts here?
 				vector_tpl<koord> halts;
-				for( uint32 j = 0; j < one_more.get_count(); j++ ) {
-					halthandle_t halt = haltestelle_t::get_halt( welt, one_more[j], this );
+				FOR(vector_tpl<koord>, const& j, one_more) {
+					halthandle_t const halt = haltestelle_t::get_halt(welt, j, this);
 					if( halt.is_bound() && !halts.is_contained(halt->get_basis_pos()) ) {
 						bool halt_connected = halt->get_fab_list().is_contained( fab );
-						for( slist_tpl< haltestelle_t::tile_t >::const_iterator iter = halt->get_tiles().begin(); iter != halt->get_tiles().end(); ++iter ) {
-							koord pos = iter->grund->get_pos().get_2d();
+						FOR(slist_tpl<haltestelle_t::tile_t>, const& i, halt->get_tiles()) {
+							koord const pos = i.grund->get_pos().get_2d();
 							if( halt_connected || fab_tiles.is_contained(pos) ) {
 								halts.append_unique( pos );
 							}
@@ -278,8 +278,8 @@ bool ai_goods_t::suche_platz1_platz2(fabrik_t *qfab, fabrik_t *zfab, int length
 				vector_tpl<koord> *next = &halts;
 				for( uint8 k = 0; k < 2; k++ ) {
 					// On which tiles we can start?
-					for( uint32 j = 0; j < next->get_count(); j++ ) {
-						const grund_t* gr = welt->lookup_kartenboden( next->operator[](j) );
+					FOR(vector_tpl<koord>, const& j, *next) {
+						grund_t const* const gr = welt->lookup_kartenboden(j);
 						if(  gr  &&  gr->get_grund_hang() == hang_t::flach  &&  !gr->hat_wege()  &&  !gr->get_leitung()  ) {
 							tile_list[i].append_unique( gr->get_pos() );
 						}
@@ -361,7 +361,9 @@ bool ai_goods_t::create_ship_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 			if(ship_vehicle->get_nachfolger_count()>0  &&  ship_vehicle->get_nachfolger(0)!=NULL) {
 				v_second = ship_vehicle->get_nachfolger(0);
 			}
-			return false;
+			else {
+				return false;
+			}
 		}
 		else {
 			ship_vehicle = v_second->get_vorgaenger(0);
@@ -749,7 +751,7 @@ void ai_goods_t::step()
 	}
 
 	// one route per month ...
-	if(  welt->get_steps() < next_contruction_steps  ) {
+	if(  welt->get_steps() < next_construction_steps  ) {
 		return;
 	}
 
@@ -768,11 +770,9 @@ void ai_goods_t::step()
 			if(root==NULL) {
 				// find a tree root to complete
 				weighted_vector_tpl<fabrik_t *> start_fabs(20);
-				slist_iterator_tpl<fabrik_t *> fabiter( welt->get_fab_list() );
-				while(fabiter.next()) {
-					fabrik_t *fab = fabiter.get_current();
+				FOR(slist_tpl<fabrik_t*>, const fab, welt->get_fab_list()) {
 					// consumer and not completely overcrowded
-					if(fab->get_besch()->get_produkte()==0  &&  fab->get_status()!=fabrik_t::bad) {
+					if (!fab->get_besch()->is_consumer_only() && fab->get_status() != fabrik_t::bad) {
 						int missing = get_factory_tree_missing_count( fab );
 						if(missing>0) {
 							start_fabs.append( fab, 100/(missing+1)+1, 20 );
@@ -970,7 +970,7 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 					length = (rail_engine->get_length() + count_rail*rail_vehicle->get_length()+CARUNITS_PER_TILE-1)/CARUNITS_PER_TILE;
 					if(suche_platz1_platz2(start, ziel, length)) {
 						state = ship_vehicle ? NR_BAUE_WATER_ROUTE : NR_BAUE_SIMPLE_SCHIENEN_ROUTE;
-						next_contruction_steps += 80;
+						next_construction_steps += 10;
 					}
 				}
 				// if state is still NR_BAUE_ROUTE1 then there are no sutiable places
@@ -978,7 +978,7 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 					// rail was too expensive or not successfull
 					count_rail = 255;
 					state = ship_vehicle ? NR_BAUE_WATER_ROUTE : NR_BAUE_STRASSEN_ROUTE;
-					next_contruction_steps += 80;
+					next_construction_steps += 10;
 				}
 			}
 			// no success at all?
@@ -1175,10 +1175,10 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 		// remove stucked vehicles (only from roads!)
 		case CHECK_CONVOI:
 		{
-			next_contruction_steps = welt->get_steps() + simrand( 8000 )+1000;
+			next_construction_steps = welt->get_steps() + simrand( ai_t::construction_speed ) + 25;
 
-			for( int i = welt->get_convoi_count()-1;  i>=0;  i--  ) {
-				const convoihandle_t cnv = welt->get_convoi(i);
+			for (size_t i = welt->convoys().get_count(); i-- != 0;) {
+				convoihandle_t const cnv = welt->convoys()[i];
 				if(!cnv.is_bound()  ||  cnv->get_besitzer()!=this) {
 					continue;
 				}
@@ -1190,7 +1190,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 
 				sint64 gewinn = 0;
 				for( int j=0;  j<12;  j++  ) {
-					gewinn += cnv->get_finance_history( j, CONVOI_PROFIT );
+					gewinn += cnv->get_finance_history( j, convoi_t::CONVOI_PROFIT );
 				}
 
 				// apparently we got the toatlly wrong vehicle here ...
@@ -1203,7 +1203,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 					sint64 goods=0;
 					// no goods for six months?
 					for( int i=0;  i<6;  i ++) {
-						goods += cnv->get_finance_history(i,CONVOI_TRANSPORTED_GOODS);
+						goods += cnv->get_finance_history( i, convoi_t::CONVOI_TRANSPORTED_GOODS );
 					}
 					delete_this = (goods==0);
 				}
@@ -1235,8 +1235,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 							vector_tpl<linehandle_t> lines;
 							koord water_stop = koord::invalid;
 							simlinemgmt.get_lines( simline_t::shipline, &lines );
-							for (vector_tpl<linehandle_t>::const_iterator iter2 = lines.begin(), end = lines.end(); iter2 != end; iter2++) {
-								linehandle_t line = *iter2;
+							FOR(vector_tpl<linehandle_t>, const line, lines) {
 								schedule_t *fpl=line->get_schedule();
 								if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos,this)==start_halt) {
 									water_stop = koord( (start_pos.x+fpl->eintrag[0].pos.x)/2, (start_pos.y+fpl->eintrag[0].pos.y)/2 );
@@ -1295,7 +1294,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 		break;
 
 		default:
-			dbg->fatal("ai_goods_t::step()","Illegal state!", state );
+			dbg->warning("ai_goods_t::step()","Illegal state!", state );
 			state = NR_INIT;
 	}
 }
@@ -1312,8 +1311,7 @@ void ai_goods_t::rdwr(loadsave_t *file)
 
 	xml_tag_t t( file, "ai_goods_t" );
 
-	// first: do all the administration
-	spieler_t::rdwr(file);
+	ai_t::rdwr(file);
 
 	// then check, if we have to do something or the game is too old ...
 	if(file->get_version()<101000) {
@@ -1324,12 +1322,7 @@ void ai_goods_t::rdwr(loadsave_t *file)
 			road_vehicle = NULL;
 			road_weg = NULL;
 
-			next_contruction_steps = welt->get_steps()+simrand(400);
-
-			road_transport = player_nr!=7;
-			rail_transport = player_nr>3;
-			ship_transport = true;
-
+			next_construction_steps = welt->get_steps()+simrand(400);
 			root = start = ziel = NULL;
 		}
 		return;
@@ -1344,13 +1337,16 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	file->rdwr_long(count_rail);
 	file->rdwr_long(count_road);
 	file->rdwr_long(count);
-	file->rdwr_bool(road_transport);
-	file->rdwr_bool(rail_transport);
-	file->rdwr_bool(ship_transport);
+	if(  file->get_version()<111001  ) {
+		file->rdwr_bool(road_transport);
+		file->rdwr_bool(rail_transport);
+		file->rdwr_bool(ship_transport);
+		air_transport = false;
+	}
 
 	if(file->is_saving()) {
 		// save current pointers
-		sint32 delta_steps = next_contruction_steps-welt->get_steps();
+		sint32 delta_steps = next_construction_steps-welt->get_steps();
 		file->rdwr_long(delta_steps);
 		koord3d k3d = root ? root->get_pos() : koord3d::invalid;
 		k3d.rdwr(file);
@@ -1378,7 +1374,7 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	}
 	else {
 		// since steps in loaded game == 0
-		file->rdwr_long(next_contruction_steps);
+		file->rdwr_long(next_construction_steps);
 		// reinit current pointers
 		koord3d k3d;
 		k3d.rdwr(file);
@@ -1411,9 +1407,7 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	sint32 cnt = forbidden_connections.get_count();
 	file->rdwr_long(cnt);
 	if(file->is_saving()) {
-		slist_iterator_tpl<fabconnection_t*> iter(forbidden_connections);
-		while(  iter.next()  ) {
-			fabconnection_t *fc = iter.get_current();
+		FOR(slist_tpl<fabconnection_t*>, const fc, forbidden_connections) {
 			fc->rdwr(file);
 		}
 	}
@@ -1438,12 +1432,11 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	}
 }
 
+
 bool ai_goods_t::is_forbidden( fabrik_t *fab1, fabrik_t *fab2, const ware_besch_t *w ) const
 {
 	fabconnection_t fc(fab1, fab2, w);
-	slist_iterator_tpl<fabconnection_t*> iter(forbidden_connections);
-	while(  iter.next()  ) {
-		fabconnection_t *test_fc = iter.get_current();
+	FOR(slist_tpl<fabconnection_t*>, const test_fc, forbidden_connections) {
 		if (fc == (*test_fc)) {
 			return true;
 		}
@@ -1451,6 +1444,7 @@ bool ai_goods_t::is_forbidden( fabrik_t *fab1, fabrik_t *fab2, const ware_besch_
 	return false;
 }
 
+
 void ai_goods_t::fabconnection_t::rdwr(loadsave_t *file)
 {
 	koord3d k3d;
@@ -1474,8 +1468,6 @@ void ai_goods_t::fabconnection_t::rdwr(loadsave_t *file)
 }
 
 
-
-
 /**
  * Dealing with stucked  or lost vehicles:
  * - delete lost ones
