@@ -14,6 +14,7 @@
 
 #include "../simconvoi.h"
 #include "../simdebug.h"
+#include "../simdepot.h"
 #include "../simhalt.h"
 #include "../simintr.h"
 #include "../simline.h"
@@ -45,7 +46,6 @@
 #include "../dings/tunnel.h"
 
 #include "../gui/messagebox.h"
-#include "../gui/money_frame.h"
 
 #include "../utils/cbuffer_t.h"
 #include "../utils/simstring.h"
@@ -82,6 +82,7 @@ spieler_t::spieler_t(karte_t *wl, uint8 nr) :
 	konto_ueberzogen = 0;
 	automat = false;		// Start nicht als automatischer Spieler
 	locked = false;	/* allowe to change anything */
+	unlock_pending = false;
 
 	headquarter_pos = koord::invalid;
 	headquarter_level = 0;
@@ -144,26 +145,6 @@ void spieler_t::set_name(const char *new_name)
 }
 
 
-/* returns FALSE when unlocking!
- */
-bool spieler_t::set_unlock( const uint8 *hash )
-{
-	if(  pwd_hash.empty()  ) {
-		locked = false;
-	}
-	else if(  hash!=NULL  ) {
-		// matches password?
-		locked = (pwd_hash != hash);
-	}
-	if(  !locked  &&  player_nr==1  ) {
-		// public player unlocked:
-		// allow to change active player
-		welt->get_settings().set_allow_player_change(true);
-	}
-	return locked;
-}
-
-
 /**
  * floating massages for all players here
  */
@@ -197,10 +178,7 @@ void spieler_t::display_messages()
 	const sint16 raster = get_tile_raster_width();
 	const sint16 yoffset = welt->get_y_off()+((display_get_width()/raster)&1)*(raster/4);
 
-	slist_iterator_tpl<income_message_t *>iter(messages);
-	while(iter.next()) {
-		income_message_t *m = iter.get_current();
-
+	FOR(slist_tpl<income_message_t*>, const m, messages) {
 		const koord ij = m->pos - welt->get_world_position()-welt->get_ansicht_ij_offset();
 		const sint16 x = (ij.x-ij.y)*(raster/2) + welt->get_x_off();
 		const sint16 y = (ij.x+ij.y)*(raster/4) + (m->alter >> 4) - tile_raster_scale_y( welt->lookup_hgt(m->pos)*TILE_HEIGHT_STEP, raster) + yoffset;
@@ -281,15 +259,7 @@ void spieler_t::neuer_monat()
 	}
 
 	// new month has started => recalculate vehicle value
-	sint64 assets = 0;
-	for(  vector_tpl<convoihandle_t>::const_iterator i = welt->convois_begin(), end = welt->convois_end();  i != end;  ++i  ) {
-		convoihandle_t cnv = *i;
-		if(cnv->get_besitzer()==this) {
-			assets += cnv->calc_restwert();
-		}
-	}
-	finance_history_year[0][COST_ASSETS] = finance_history_month[0][COST_ASSETS] = assets;
-	finance_history_year[0][COST_NETWEALTH] = finance_history_month[0][COST_NETWEALTH] = assets+konto;
+	calc_assets();
 
 	calc_finance_history();
 
@@ -316,21 +286,25 @@ void spieler_t::neuer_monat()
 	}
 
 	// Bankrott ?
-	if(konto < 0) {
+	if(  konto < 0  ) {
 		konto_ueberzogen++;
-		if (!welt->get_settings().is_freeplay() && player_nr != 1) {
+		if(  !welt->get_settings().is_freeplay()  &&  player_nr != 1  ) {
 			if(  welt->get_active_player_nr()==player_nr  &&  !umgebung_t::networkmode  ) {
-				if(finance_history_year[0][COST_NETWEALTH]<0) {
+				if(  finance_history_year[0][COST_NETWEALTH] < 0 ) {
 					destroy_all_win(true);
 					create_win( display_get_width()/2-128, 40, new news_img("Bankrott:\n\nDu bist bankrott.\n"), w_info, magic_none);
 					ticker::add_msg( translator::translate("Bankrott:\n\nDu bist bankrott.\n"), koord::invalid, PLAYER_FLAG + kennfarbe1 + 1 );
 					welt->beenden(false);
 				}
+				else if(  get_finance_history_year(0, COST_NETWEALTH)*10 < welt->get_settings().get_starting_money(welt->get_current_month()/12)  ){
+					// tell the player (problem!)
+					welt->get_message()->add_message( translator::translate("Net wealth less than 10% of starting capital!"), koord::invalid, message_t::problems, player_nr, IMG_LEER );
+				}
 				else {
-					// tell the player
+					// tell the player (just warning)
 					buf.clear();
 					buf.printf( translator::translate("On loan since %i month(s)"), konto_ueberzogen );
-					welt->get_message()->add_message( buf, koord::invalid, message_t::problems, player_nr, IMG_LEER );
+					welt->get_message()->add_message( buf, koord::invalid, message_t::ai, player_nr, IMG_LEER );
 				}
 			}
 			// no assets => nothing to go bankrupt about again
@@ -340,11 +314,18 @@ void spieler_t::neuer_monat()
 				if(finance_history_year[0][COST_NETWEALTH]<0) {
 					ai_bankrupt();
 				}
-				// tell the current player
+				// tell the current player (even during networkgames)
 				if(  welt->get_active_player_nr()==player_nr  ) {
-					buf.clear();
-					buf.printf( translator::translate("On loan since %i month(s)"), konto_ueberzogen );
-					welt->get_message()->add_message(buf,koord::invalid,message_t::problems,player_nr,IMG_LEER);
+					if(  get_finance_history_year(0, COST_NETWEALTH)*10 < welt->get_settings().get_starting_money(welt->get_current_month()/12)  ){
+						// netweath nearly spent (problem!)
+						welt->get_message()->add_message( translator::translate("Net wealth near zero"), koord::invalid, message_t::problems, player_nr, IMG_LEER );
+					}
+					else {
+						// just minus in account (just tell)
+						buf.clear();
+						buf.printf( translator::translate("On loan since %i month(s)"), konto_ueberzogen );
+						welt->get_message()->add_message( buf, koord::invalid, message_t::ai, player_nr, IMG_LEER );
+					}
 				}
 			}
 		}
@@ -402,13 +383,15 @@ void spieler_t::calc_finance_history()
 	*/
 	sint64 profit, mprofit;
 	profit = mprofit = 0;
-	for (int i=0; i<MAX_PLAYER_COST; i++) {
+	for (int i=0; i<COST_ASSETS; i++) {
 		// all costs < COST_ASSETS influence profit, so we must sum them up
-		if(i<COST_ASSETS) {
-			profit += finance_history_year[0][i];
-			mprofit += finance_history_month[0][i];
-		}
+		profit += finance_history_year[0][i];
+		mprofit += finance_history_month[0][i];
 	}
+	profit += finance_history_year[0][COST_POWERLINES];
+	profit += finance_history_year[0][COST_WAY_TOLLS];
+	mprofit += finance_history_month[0][COST_POWERLINES];
+	mprofit += finance_history_month[0][COST_WAY_TOLLS];
 
 	finance_history_year[0][COST_PROFIT] = profit;
 	finance_history_month[0][COST_PROFIT] = mprofit;
@@ -426,6 +409,40 @@ void spieler_t::calc_finance_history()
 }
 
 
+void spieler_t::calc_assets()
+{
+	sint64 assets = 0;
+	// all convois
+	FOR(vector_tpl<convoihandle_t>, const cnv, welt->convoys()) {
+		if(  cnv->get_besitzer() == this  ) {
+			assets += cnv->calc_restwert();
+		}
+	}
+
+	// all vehikels stored in depot not part of a convoi
+	FOR(slist_tpl<depot_t*>, const depot, depot_t::get_depot_list()) {
+		if(  depot->get_player_nr() == player_nr  ) {
+			FOR(slist_tpl<vehikel_t*>, const veh, depot->get_vehicle_list()) {
+				assets += veh->calc_restwert();
+			}
+		}
+	}
+
+	finance_history_year[0][COST_ASSETS] = finance_history_month[0][COST_ASSETS] = assets;
+	finance_history_year[0][COST_NETWEALTH] = finance_history_month[0][COST_NETWEALTH] = assets+konto;
+}
+
+
+void spieler_t::update_assets(sint64 const delta)
+{
+	finance_history_year[0][COST_ASSETS] += delta;
+	finance_history_year[0][COST_NETWEALTH] += delta;
+
+	finance_history_month[0][COST_ASSETS] += delta;
+	finance_history_month[0][COST_NETWEALTH] += delta;
+}
+
+
 // add and amount, including the display of the message and some other things ...
 void spieler_t::buche(sint64 const betrag, koord const pos, player_cost const type)
 {
@@ -434,17 +451,11 @@ void spieler_t::buche(sint64 const betrag, koord const pos, player_cost const ty
 	if(betrag != 0) {
 		if(  koord_distance(welt->get_world_position(),pos)<2*(uint32)(display_get_width()/get_tile_raster_width())+3  ) {
 			// only display, if near the screen ...
-			add_message(pos, betrag);
+			add_message(pos, (sint32)betrag);
 
 			// and same for sound too ...
 			if(  betrag>=10000  &&  !welt->is_fast_forward()  ) {
-				struct sound_info info;
-
-				info.index = SFX_CASH;
-				info.volume = 255;
-				info.pri = 0;
-
-				welt->play_sound_area_clipped(pos, info);
+				welt->play_sound_area_clipped(pos, SFX_CASH);
 			}
 		}
 	}
@@ -526,8 +537,8 @@ void spieler_t::ai_bankrupt()
 {
 	DBG_MESSAGE("spieler_t::ai_bankrupt()","Removing convois");
 
-	for( int i = welt->get_convoi_count()-1;  i>=0;  i--  ) {
-		const convoihandle_t cnv = welt->get_convoi(i);
+	for (size_t i = welt->convoys().get_count(); i-- != 0;) {
+		convoihandle_t const cnv = welt->convoys()[i];
 		if(cnv->get_besitzer()!=this) {
 			continue;
 		}
@@ -559,13 +570,11 @@ void spieler_t::ai_bankrupt()
 	}
 
 	// transfer all ways in public stops belonging to me to no one
-	slist_iterator_tpl<halthandle_t>iter(haltestelle_t::get_alle_haltestellen());
-	while(  iter.next()  ) {
-		halthandle_t halt = iter.get_current();
+	FOR(slist_tpl<halthandle_t>, const halt, haltestelle_t::get_alle_haltestellen()) {
 		if(  halt->get_besitzer()==welt->get_spieler(1)  ) {
 			// only concerns public stops tiles
-			for(  slist_tpl<haltestelle_t::tile_t>::const_iterator iter_tiles = halt->get_tiles().begin(), end = halt->get_tiles().end();  iter_tiles != end;  ++iter_tiles  ) {
-				const grund_t *gr = iter_tiles->grund;
+			FOR(slist_tpl<haltestelle_t::tile_t>, const& i, halt->get_tiles()) {
+				grund_t const* const gr = i.grund;
 				for(  uint8 wnr=0;  wnr<2;  wnr++  ) {
 					weg_t *w = gr->get_weg_nr(wnr);
 					if(  w  &&  w->get_besitzer()==this  ) {
@@ -587,7 +596,7 @@ void spieler_t::ai_bankrupt()
 	for( int y=0;  y<welt->get_groesse_y();  y++  ) {
 		for( int x=0;  x<welt->get_groesse_x();  x++  ) {
 			planquadrat_t *plan = welt->access(x,y);
-			for(  int b=plan->get_boden_count()-1;  b>=0;  b--  ) {
+			for (size_t b = plan->get_boden_count(); b-- != 0;) {
 				grund_t *gr = plan->get_boden_bei(b);
 				// remove tunnel and bridges first
 				if(  gr->get_top()>0  &&  gr->obj_bei(0)->get_besitzer()==this   &&  (gr->ist_bruecke()  ||  gr->ist_tunnel())  ) {
@@ -608,7 +617,7 @@ void spieler_t::ai_bankrupt()
 					}
 				}
 				bool count_signs = false;
-				for(  int i=gr->get_top()-1;  i>=0;  i--  ) {
+				for (size_t i = gr->get_top(); i-- != 0;) {
 					ding_t *dt = gr->obj_bei(i);
 					if(dt->get_besitzer()==this) {
 						switch(dt->get_typ()) {
@@ -874,14 +883,15 @@ void spieler_t::rdwr(loadsave_t *file)
 		for(  int year=0;  year<MAX_PLAYER_HISTORY_YEARS;  year++  ) {
 			finance_history_year[year][COST_NETWEALTH] = finance_history_year[year][COST_CASH]+finance_history_year[year][COST_ASSETS];
 			// only revnue minus running costs
-			finance_history_year[year][COST_OPERATING_PROFIT] = finance_history_year[year][COST_INCOME]+finance_history_year[year][COST_VEHICLE_RUN]+finance_history_year[year][COST_MAINTENANCE];
+			finance_history_year[year][COST_OPERATING_PROFIT] = finance_history_year[year][COST_INCOME] + finance_history_year[year][COST_POWERLINES] + finance_history_year[year][COST_VEHICLE_RUN] + finance_history_year[year][COST_MAINTENANCE] + finance_history_year[year][COST_WAY_TOLLS];
+
 			// including also investements into vehicles/infrastructure
 			finance_history_year[year][COST_PROFIT] = finance_history_year[year][COST_INCOME]+finance_history_year[year][COST_VEHICLE_RUN]+finance_history_year[year][COST_MAINTENANCE]+finance_history_year[year][COST_CONSTRUCTION]+finance_history_year[year][COST_NEW_VEHICLE];
 			finance_history_year[year][COST_MARGIN] = calc_margin(finance_history_year[year][COST_OPERATING_PROFIT], finance_history_year[year][COST_INCOME]);
 		}
 		for(  int month=0;  month<MAX_PLAYER_HISTORY_MONTHS;  month++  ) {
 			finance_history_month[month][COST_NETWEALTH] = finance_history_month[month][COST_CASH]+finance_history_month[month][COST_ASSETS];
-			finance_history_month[month][COST_OPERATING_PROFIT] = finance_history_month[month][COST_INCOME]+finance_history_month[month][COST_VEHICLE_RUN]+finance_history_month[month][COST_MAINTENANCE];
+			finance_history_month[month][COST_OPERATING_PROFIT] = finance_history_month[month][COST_INCOME] + finance_history_month[month][COST_POWERLINES] + finance_history_month[month][COST_VEHICLE_RUN] + finance_history_month[month][COST_MAINTENANCE] + finance_history_month[month][COST_WAY_TOLLS];
 			finance_history_month[month][COST_PROFIT] = finance_history_month[month][COST_INCOME]+finance_history_month[month][COST_VEHICLE_RUN]+finance_history_month[month][COST_MAINTENANCE]+finance_history_month[month][COST_CONSTRUCTION]+finance_history_month[month][COST_NEW_VEHICLE];
 			finance_history_month[month][COST_MARGIN] = calc_margin(finance_history_month[month][COST_OPERATING_PROFIT], finance_history_month[month][COST_INCOME]);
 		}
@@ -930,9 +940,8 @@ DBG_DEBUG("spieler_t::rdwr()","player %i: loading %i halts.",welt->sp2num( this
 			file->rdwr_byte(pwd_hash[i]);
 		}
 		if(  file->is_loading()  ) {
-			locked = true;
-			// disallow all actions, if password set (might be unlocked by karte_t::set_werkzeug() )
-			set_unlock( NULL );
+			// disallow all actions, if password set (might be unlocked by password gui )
+			locked = !pwd_hash.empty();
 		}
 	}
 
@@ -943,7 +952,7 @@ DBG_DEBUG("spieler_t::rdwr()","player %i: loading %i halts.",welt->sp2num( this
 }
 
 
-/*
+/**
  * called after game is fully loaded;
  */
 void spieler_t::laden_abschliessen()
@@ -951,15 +960,7 @@ void spieler_t::laden_abschliessen()
 	simlinemgmt.laden_abschliessen();
 	display_set_player_color_scheme( player_nr, kennfarbe1, kennfarbe2 );
 	// recalculate vehicle value
-	sint64 assets = 0;
-	for(  vector_tpl<convoihandle_t>::const_iterator i = welt->convois_begin(), end = welt->convois_end();  i != end;  ++i  ) {
-		convoihandle_t cnv = *i;
-		if(cnv->get_besitzer()==this) {
-			assets += cnv->calc_restwert();
-		}
-	}
-	finance_history_year[0][COST_ASSETS] = finance_history_month[0][COST_ASSETS] = assets;
-	finance_history_year[0][COST_NETWEALTH] = finance_history_month[0][COST_NETWEALTH] = assets+konto;
+	calc_assets();
 }
 
 
@@ -1040,8 +1041,8 @@ sint64 spieler_t::undo()
 		return false;
 	}
 	// check, if we can still do undo
-	for(unsigned short i=0;  i<last_built.get_count();  i++  ) {
-		grund_t* gr = welt->lookup(last_built[i]);
+	FOR(vector_tpl<koord3d>, const& i, last_built) {
+		grund_t* const gr = welt->lookup(i);
 		if(gr==NULL  ||  gr->get_typ()!=grund_t::boden) {
 			// well, something was built here ... so no undo
 			last_built.clear();
@@ -1087,8 +1088,8 @@ sint64 spieler_t::undo()
 
 	// ok, now remove everything last built
 	sint64 cost=0;
-	for(  uint32 i=0;  i<last_built.get_count();  i++  ) {
-		grund_t* gr = welt->lookup(last_built[i]);
+	FOR(vector_tpl<koord3d>, const& i, last_built) {
+		grund_t* const gr = welt->lookup(i);
 		if(  undo_type != powerline_wt  ) {
 			cost += gr->weg_entfernen(undo_type,true);
 		}
@@ -1114,14 +1115,12 @@ void spieler_t::tell_tool_result(werkzeug_t *tool, koord3d, const char *err, boo
 	if (welt->get_active_player()==this  &&  local) {
 		if(err==NULL) {
 			if(tool->ok_sound!=NO_SOUND) {
-				struct sound_info info = {tool->ok_sound,255,0};
-				sound_play(info);
+				sound_play(tool->ok_sound);
 			}
 		}
 		else if(*err!=0) {
 			// something went really wrong
-			struct sound_info info = {SFX_FAILURE,255,0};
-			sound_play(info);
+			sound_play(SFX_FAILURE);
 			create_win( new news_img(err), w_time_delete, magic_none);
 		}
 	}
