@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 1997 - 2001 Hj. Malthaner
  *
- * This file is part of the Simutrans project under the artistic licence.
- * (see licence.txt)
+ * This file is part of the Simutrans project under the artistic license.
+ * (see license.txt)
  *
  * construction of cities, creation of passengers
  *
@@ -49,6 +49,7 @@
 
 #include "sucher/bauplatz_sucher.h"
 #include "bauer/wegbauer.h"
+#include "bauer/brueckenbauer.h"
 #include "bauer/hausbauer.h"
 #include "bauer/fabrikbauer.h"
 #include "utils/cbuffer_t.h"
@@ -170,8 +171,7 @@ bool stadt_t::bewerte_loc(const koord pos, const rule_t &regel, int rotation)
 	//printf("Test for (%s) in rotation %d\n", pos.get_str(), rotation);
 	koord k;
 
-	for(uint32 i=0; i<regel.rule.get_count(); i++){
-		const rule_entry_t &r = regel.rule[i];
+	FOR(vector_tpl<rule_entry_t>, const& r, regel.rule) {
 		uint8 x,y;
 		switch (rotation) {
 			default:
@@ -692,9 +692,9 @@ void stadt_t::recalc_city_size()
 {
 	lo = pos;
 	ur = pos;
-	for(  uint32 i=0;  i<buildings.get_count();  i++  ) {
-		if(buildings[i]->get_tile()->get_besch()->get_utyp()!=haus_besch_t::firmensitz) {
-			const koord gb_pos = buildings[i]->get_pos().get_2d();
+	FOR(weighted_vector_tpl<gebaeude_t*>, const i, buildings) {
+		if (i->get_tile()->get_besch()->get_utyp() != haus_besch_t::firmensitz) {
+			koord const& gb_pos = i->get_pos().get_2d();
 			if (lo.x > gb_pos.x) {
 				lo.x = gb_pos.x;
 			}
@@ -766,15 +766,14 @@ void stadt_t::factory_entry_t::rdwr(loadsave_t *file)
 void stadt_t::factory_entry_t::resolve_factory()
 {
 	factory = fabrik_t::get_fab( welt, koord(factory_pos_x, factory_pos_y) );
-	assert( factory );
 }
 
 
 const stadt_t::factory_entry_t* stadt_t::factory_set_t::get_entry(const fabrik_t *const factory) const
 {
-	for(  uint32 e=0;  e<entries.get_count();  ++e  ) {
-		if(  entries[e].factory==factory  ) {
-			return &entries[e];
+	FOR(vector_tpl<factory_entry_t>, const& e, entries) {
+		if (e.factory == factory) {
+			return &e;
 		}
 	}
 	return NULL;	// not found
@@ -785,8 +784,7 @@ stadt_t::factory_entry_t* stadt_t::factory_set_t::get_random_entry()
 {
 	if(  total_remaining>0  ) {
 		sint32 weight = simrand(total_remaining);
-		for(  uint32 e=0;  e<entries.get_count();  ++e  ) {
-			factory_entry_t &entry = entries[e];
+		FOR(vector_tpl<factory_entry_t>, & entry, entries) {
 			if(  entry.remaining>0  ) {
 				if(  weight<entry.remaining  ) {
 					return &entry;
@@ -869,8 +867,7 @@ void stadt_t::factory_set_t::recalc_generation_ratio(const sint32 default_percen
 		const sint64 supply_promille = ( ( (average_generated << 10) * (sint64)default_percent ) / 100 ) / (sint64)target_supply;
 		if(  supply_promille<1024  ) {
 			// expected supply is really smaller than target supply
-			for(  uint32 e=0;  e<entries.get_count();  ++e  ) {
-				factory_entry_t &entry = entries[e];
+			FOR(vector_tpl<factory_entry_t>, & entry, entries) {
 				const sint32 new_supply = (sint32)( ( (sint64)entry.demand * SUPPLY_FACTOR * supply_promille + ((1<<(DEMAND_BITS+SUPPLY_BITS+10))-1) ) >> (DEMAND_BITS+SUPPLY_BITS+10) );
 				const sint32 delta_supply = new_supply - entry.supply;
 				if(  delta_supply==0  ) {
@@ -892,8 +889,7 @@ void stadt_t::factory_set_t::recalc_generation_ratio(const sint32 default_percen
 		}
 	}
 	// expected supply is unknown or sufficient to meet target supply
-	for(  uint32 e=0;  e<entries.get_count();  ++e  ) {
-		factory_entry_t &entry = entries[e];
+	FOR(vector_tpl<factory_entry_t>, & entry, entries) {
 		const sint32 new_supply = ( entry.demand * SUPPLY_FACTOR + ((1<<(DEMAND_BITS+SUPPLY_BITS))-1) ) >> (DEMAND_BITS+SUPPLY_BITS);
 		const sint32 delta_supply = new_supply - entry.supply;
 		if(  delta_supply==0  ) {
@@ -916,8 +912,8 @@ void stadt_t::factory_set_t::recalc_generation_ratio(const sint32 default_percen
 
 void stadt_t::factory_set_t::new_month()
 {
-	for(  uint32 e=0;  e<entries.get_count();  ++e  ) {
-		entries[e].new_month();
+	FOR(vector_tpl<factory_entry_t>, & e, entries) {
+		e.new_month();
 	}
 	total_remaining = 0;
 	total_generated = 0;
@@ -952,8 +948,15 @@ void stadt_t::factory_set_t::rdwr(loadsave_t *file)
 
 void stadt_t::factory_set_t::resolve_factories()
 {
-	for(  uint32 e=0;  e<entries.get_count();  ++e  ) {
-		entries[e].resolve_factory();
+	uint32 remove_count = 0;
+	FOR(vector_tpl<factory_entry_t>, & e, entries) {
+		e.resolve_factory();
+		if (!e.factory) {
+			remove_count ++;
+		}
+	}
+	for(  uint32 e=0;  e<remove_count;  ++e  ) {
+		this->remove_factory( NULL );
 	}
 }
 
@@ -991,7 +994,17 @@ stadt_t::~stadt_t()
 			}
 		}
 	}
-	free( (void *)name );
+}
+
+
+static bool name_used(weighted_vector_tpl<stadt_t*> const& cities, char const* const name)
+{
+	FOR(weighted_vector_tpl<stadt_t*>, const i, cities) {
+		if (strcmp(i->get_name(), name) == 0) {
+			return true;
+		}
+	}
+	return false;
 }
 
 
@@ -1026,28 +1039,21 @@ stadt_t::stadt_t(spieler_t* sp, koord pos, sint32 citizens) :
 	DBG_MESSAGE("stadt_t::stadt_t()", "Welt %p", welt);
 
 	/* get a unique cityname */
-	/* 9.1.2005, prissi */
-	const weighted_vector_tpl<stadt_t*>& staedte = welt->get_staedte();
-	vector_tpl<char *> city_names( translator::get_city_name_list() );
-
-	name = NULL;
-	while(  !city_names.empty()  &&  name==NULL  ) {
-		uint32 idx = simrand(city_names.get_count());
-		for (weighted_vector_tpl<stadt_t*>::const_iterator i = staedte.begin(), end = staedte.end(); i != end; ++i) {
-			if(  strcmp( (*i)->get_name(), city_names[idx] )==0  ) {
-				city_names.remove_at( idx );
-				idx = city_names.get_count();
-				break;
-			}
-		}
-		if(  idx < city_names.get_count()  ) {
-			name = strdup( city_names[idx] );
+	char                          const* n       = "simcity";
+	weighted_vector_tpl<stadt_t*> const& staedte = welt->get_staedte();
+	for (vector_tpl<char*> city_names(translator::get_city_name_list()); !city_names.empty();) {
+		size_t      const idx  = simrand(city_names.get_count());
+		char const* const cand = city_names[idx];
+		if (name_used(staedte, cand)) {
+			city_names[idx] = city_names.back();
+			city_names.pop_back();
+		} else {
+			n = cand;
+			break;
 		}
 	}
-	if(  name==NULL  ) {
-		name = strdup( "simcity" );
-	}
-	DBG_MESSAGE("stadt_t::stadt_t()", "founding new city named '%s'", name);
+	DBG_MESSAGE("stadt_t::stadt_t()", "founding new city named '%s'", n);
+	name = n;
 
 	// 1. Rathaus bei 0 Leuten bauen
 	check_bau_rathaus(true);
@@ -1095,7 +1101,6 @@ stadt_t::stadt_t(karte_t* wl, loadsave_t* file) :
 	has_low_density = false;
 
 	wachstum = 0;
-	name = NULL;
 	stadtinfo_options = 3;
 
 	rdwr(file);
@@ -1236,7 +1241,6 @@ void stadt_t::rdwr(loadsave_t* file)
 }
 
 
-
 /**
  * Wird am Ende der Laderoutine aufgerufen, wenn die Welt geladen ist
  * und nur noch die Datenstrukturenneu verknuepft werden muessen.
@@ -1249,7 +1253,7 @@ void stadt_t::laden_abschliessen()
 	next_bau_step = 0;
 
 	// there might be broken savegames
-	if(name==NULL) {
+	if (!name) {
 		set_name( "simcity" );
 	}
 
@@ -1341,10 +1345,7 @@ void stadt_t::rotate90( const sint16 y_size )
 
 void stadt_t::set_name(const char *new_name)
 {
-	if(name==NULL  ||  strcmp(name,new_name)) {
-		free( (void *)name );
-		name = strdup( new_name );
-	}
+	name = new_name;
 	grund_t *gr = welt->lookup_kartenboden(pos);
 	if(gr) {
 		gr->set_text( new_name );
@@ -1373,9 +1374,7 @@ void stadt_t::verbinde_fabriken()
 	assert( target_factories_pax.get_entries().empty() );
 	assert( target_factories_mail.get_entries().empty() );
 
-	slist_iterator_tpl<fabrik_t*> fab_iter(welt->get_fab_list());
-	while (fab_iter.next()) {
-		fabrik_t* fab = fab_iter.get_current();
+	FOR(slist_tpl<fabrik_t*>, const fab, welt->get_fab_list()) {
 		const uint32 count = fab->get_target_cities().get_count();
 		settings_t const& s = welt->get_settings();
 		if (count < s.get_factory_worker_maximum_towns() && koord_distance(fab->get_pos(), pos) < s.get_factory_worker_radius()) {
@@ -1549,7 +1548,7 @@ void stadt_t::neuer_monat( bool recalc_destinations )
 		// computes (1.0 - s[0]/s[1]) * city_history_month[1][HIST_CITICENS]
 #		define comp_factor(s) (city_history_month[1][HIST_CITICENS] *( s[1]-s[0] )) / s[1]
 
-		uint32 factor = comp_stats(pax_stat, mail_stat) ? (comp_stats(good_stat, pax_stat) ? comp_factor(good_stat) : comp_factor(pax_stat)) : comp_factor(mail_stat);
+		uint32 factor = (uint32)( comp_stats(pax_stat, mail_stat) ? (comp_stats(good_stat, pax_stat) ? comp_factor(good_stat) : comp_factor(pax_stat)) : comp_factor(mail_stat) );
 		factor = log10(factor);
 
 		uint16 number_of_cars = simrand( factor * s.get_verkehr_level() ) / 16;
@@ -1582,8 +1581,8 @@ void stadt_t::calc_growth()
 {
 	// now iterate over all factories to get the ratio of producing version nonproducing factories
 	// we use the incoming storage as a measure und we will only look for end consumers (power stations, markets)
-	for(  vector_tpl<factory_entry_t>::const_iterator iter = target_factories_pax.get_entries().begin(), end = target_factories_pax.get_entries().end();  iter!=end;  ++iter  ) {
-		fabrik_t *const fab = iter->factory;
+	FOR(vector_tpl<factory_entry_t>, const& i, target_factories_pax.get_entries()) {
+		fabrik_t *const fab = i.factory;
 		if (fab->get_lieferziele().empty() && !fab->get_suppliers().empty()) {
 			// consumer => check for it storage
 			const fabrik_besch_t *const besch = fab->get_besch();
@@ -1609,10 +1608,10 @@ void stadt_t::calc_growth()
 	 */
 	sint64     const(& h)[MAX_CITY_HISTORY] = city_history_month[0];
 	settings_t const&  s           = welt->get_settings();
-	sint32     const   pas         = (h[HIST_PAS_TRANSPORTED]  * (s.get_passenger_multiplier() << 6)) / (h[HIST_PAS_GENERATED]  + 1);
-	sint32     const   mail        = (h[HIST_MAIL_TRANSPORTED] * (s.get_mail_multiplier()      << 6)) / (h[HIST_MAIL_GENERATED] + 1);
+	sint32     const   pas         = (sint32)( (h[HIST_PAS_TRANSPORTED]  * (s.get_passenger_multiplier() << 6)) / (h[HIST_PAS_GENERATED]  + 1) );
+	sint32     const   mail        = (sint32)( (h[HIST_MAIL_TRANSPORTED] * (s.get_mail_multiplier()      << 6)) / (h[HIST_MAIL_GENERATED] + 1) );
 	sint32     const   electricity = 0;
-	sint32     const   goods       = h[HIST_GOODS_NEEDED] == 0 ? 0 : (h[HIST_GOODS_RECIEVED] * (s.get_goods_multiplier() << 6)) / (h[HIST_GOODS_NEEDED]);
+	sint32     const   goods       = (sint32)( h[HIST_GOODS_NEEDED] == 0 ? 0 : (h[HIST_GOODS_RECIEVED] * (s.get_goods_multiplier() << 6)) / (h[HIST_GOODS_NEEDED]) );
 
 	// smaller towns should growth slower to have villages for a longer time
 	sint32 const weight_factor =
@@ -1761,20 +1760,30 @@ void stadt_t::step_passagiere()
 					factory_entry->factory->liefere_an(wtyp, pax_left_to_do);
 				}
 				// people who walk or mail delivered by hand do not count as transported or happy
+				start_halt->add_pax_walked(num_pax);
 			}
 			else if(  route_result==haltestelle_t::ROUTE_OVERCROWDED  ) {
 				merke_passagier_ziel(dest_pos, COL_ORANGE );
-				start_halt->add_pax_unhappy(pax_left_to_do);
-				if(  will_return  ) {
-					pax.get_ziel()->add_pax_unhappy(pax_left_to_do);
+				if (start_halt.is_bound()) {
+					start_halt->add_pax_unhappy(pax_left_to_do);
+					if(  will_return  ) {
+						pax.get_ziel()->add_pax_unhappy(pax_left_to_do);
+					}
+				}
+				else {
+					// all routes to goal are overcrowded -> register at all start halts
+					FOR(vector_tpl<halthandle_t>, const s, start_halts) {
+						s->add_pax_unhappy(pax_left_to_do);
+						merke_passagier_ziel(dest_pos, COL_ORANGE);
+					}
 				}
 			}
 			else {
 				// since there is no route from any start halt -> register no route at all start halts
-				for(  uint32 s=0;  s<start_halts.get_count();  ++s  ) {
-					start_halts[s]->add_pax_no_route(pax_left_to_do);
-					merke_passagier_ziel(dest_pos, COL_DARK_ORANGE);
+				FOR(vector_tpl<halthandle_t>, const s, start_halts) {
+					s->add_pax_no_route(pax_left_to_do);
 				}
+				merke_passagier_ziel(dest_pos, COL_DARK_ORANGE);
 #ifdef DESTINATION_CITYCARS
 				//citycars with destination
 				erzeuge_verkehrsteilnehmer(start_halt->get_basis_pos(), step_count, ziel);
@@ -1879,9 +1888,8 @@ void stadt_t::add_target_city(stadt_t *const city)
 void stadt_t::recalc_target_cities()
 {
 	target_cities.clear();
-	const weighted_vector_tpl<stadt_t *> &cities = welt->get_staedte();
-	for(  uint32 c=0;  c<cities.get_count();  ++c  ) {
-		add_target_city( cities[c] );
+	FOR(weighted_vector_tpl<stadt_t*>, const c, welt->get_staedte()) {
+		add_target_city(c);
 	}
 }
 
@@ -1900,9 +1908,8 @@ void stadt_t::add_target_attraction(gebaeude_t *const attraction)
 void stadt_t::recalc_target_attractions()
 {
 	target_attractions.clear();
-	const weighted_vector_tpl<gebaeude_t *> &attractions = welt->get_ausflugsziele();
-	for(  uint32 a=0;  a<attractions.get_count();  ++a  ) {
-		add_target_attraction( attractions[a] );
+	FOR(weighted_vector_tpl<gebaeude_t*>, const a, welt->get_ausflugsziele()) {
+		add_target_attraction(a);
 	}
 }
 
@@ -1965,8 +1972,8 @@ class bauplatz_mit_strasse_sucher_t: public bauplatz_sucher_t
 		{
 			const weighted_vector_tpl<gebaeude_t*>& attractions = welt->get_ausflugsziele();
 			int dist = welt->get_groesse_x() * welt->get_groesse_y();
-			for (weighted_vector_tpl<gebaeude_t*>::const_iterator i = attractions.begin(), end = attractions.end(); i != end; ++i) {
-				int d = koord_distance((*i)->get_pos(), pos);
+			FOR(weighted_vector_tpl<gebaeude_t*>, const i, attractions) {
+				int const d = koord_distance(i->get_pos(), pos);
 				if (d < dist) {
 					dist = d;
 				}
@@ -2117,7 +2124,7 @@ void stadt_t::check_bau_rathaus(bool new_town)
 		koord k;
 		int old_layout(0);
 
-		DBG_MESSAGE("check_bau_rathaus()", "bev=%d, new=%d name=%s", bev, neugruendung, name);
+		DBG_MESSAGE("check_bau_rathaus()", "bev=%d, new=%d name=%s", bev, neugruendung, name.c_str());
 
 		if(  bev!=0  ) {
 
@@ -2272,7 +2279,7 @@ void stadt_t::check_bau_rathaus(bool new_town)
 		// if not during initialization
 		if (!new_town) {
 			cbuffer_t buf;
-			buf.printf( translator::translate("%s wasted\nyour money with a\nnew townhall\nwhen it reached\n%i inhabitants."), name, get_einwohner() );
+			buf.printf(translator::translate("%s wasted\nyour money with a\nnew townhall\nwhen it reached\n%i inhabitants."), name.c_str(), get_einwohner());
 			welt->get_message()->add_message(buf, best_pos, message_t::city, CITY_KI, besch->get_tile(layout, 0, 0)->get_hintergrund(0, 0, 0));
 		}
 		else {
@@ -2589,7 +2596,7 @@ void stadt_t::renoviere_gebaeude(gebaeude_t* gb)
 		}
 	}
 	// check for industry, also if we wanted com, but there was no com good enough ...
-	if ((sum_industrie > sum_industrie && sum_industrie > sum_wohnung) || (sum_gewerbe > sum_wohnung && will_haben == gebaeude_t::unbekannt)) {
+	if(  (sum_industrie > sum_gewerbe  &&  sum_industrie > sum_wohnung) || (sum_gewerbe > sum_wohnung  &&  will_haben == gebaeude_t::unbekannt)  ) {
 		// we must check, if we can really update to higher level ...
 		const int try_level = (alt_typ == gebaeude_t::industrie ? level + 1 : level);
 		h = hausbauer_t::get_industrie(try_level , current_month, cl);
@@ -2760,11 +2767,17 @@ bool stadt_t::baue_strasse(const koord k, spieler_t* sp, bool forced)
 		if (ribi_t::nsow[r] & allowed_dir) {
 			// now we have to check for several problems ...
 			grund_t* bd2;
-			if(bd->get_neighbour(bd2, invalid_wt, koord::nsow[r])) {
-				if(bd2->get_typ()==grund_t::fundament) {
+			if(bd->get_neighbour(bd2, invalid_wt, ribi_t::nsow[r])) {
+				if(bd2->get_typ()==grund_t::fundament  ||  bd2->get_typ()==grund_t::wasser) {
 					// not connecting to a building of course ...
-				} else if (bd2->get_typ()!=grund_t::boden  &&  ribi_t::nsow[r]!=ribi_typ(bd2->get_grund_hang())) {
-					// not the same slope => tunnel or bridge
+				} else if (!bd2->ist_karten_boden()) {
+					// do not connect to elevated ways / bridges
+				} else if (bd2->get_typ()==grund_t::tunnelboden  &&  ribi_t::nsow[r]!=ribi_typ(bd2->get_grund_hang())) {
+					// not the correct slope
+				} else if (bd2->get_typ()==grund_t::brueckenboden
+					&&  (bd2->get_grund_hang()==hang_t::flach  ?  ribi_t::nsow[r]!=ribi_typ(bd2->get_weg_hang())
+					                                           :  ribi_t::rueckwaerts(ribi_t::nsow[r])!=ribi_typ(bd2->get_grund_hang()))) {
+					// not the correct slope
 				} else if(bd2->hat_weg(road_wt)) {
 					const gebaeude_t* gb = bd2->find<gebaeude_t>();
 					if(gb) {
@@ -2824,6 +2837,35 @@ bool stadt_t::baue_strasse(const koord k, spieler_t* sp, bool forced)
 			bd->neuen_weg_bauen(weg, connection_roads, sp);
 			bd->calc_bild();	// otherwise the
 		}
+		// check to bridge a river
+		if (ribi_t::ist_einfach(connection_roads)) {
+			koord zv = koord(ribi_t::rueckwaerts(connection_roads));
+			grund_t *bd_next = welt->lookup_kartenboden( k + zv );
+			if (bd_next  &&  (bd_next->ist_wasser()  ||  (bd_next->hat_weg(water_wt)  &&  bd_next->get_weg(water_wt)->get_besch()->get_styp()==255))) {
+				// ok there is a river
+				const bruecke_besch_t *bridge = brueckenbauer_t::find_bridge(road_wt, 50, welt->get_timeline_year_month() );
+				if(  bridge==NULL  ) {
+					// does not have a bridge available ...
+					return false;
+				}
+				const char *err = NULL;
+				koord3d end = brueckenbauer_t::finde_ende(welt, bd->get_pos(), zv, bridge, err, false);
+				if (err  ||   koord_distance( k, end.get_2d())>3) {
+					// try to find shortest possible
+					end = brueckenbauer_t::finde_ende(welt, bd->get_pos(), zv, bridge, err, true);
+				}
+				if (err==NULL  &&   koord_distance( k, end.get_2d())<=3) {
+					brueckenbauer_t::baue_bruecke(welt, welt->get_spieler(1), bd->get_pos(), end, zv, bridge, welt->get_city_road());
+					// try to build one connecting piece of road
+					baue_strasse( (end+zv).get_2d(), NULL, false);
+					// try to build a house near the bridge end
+					uint32 old_count = buildings.get_count();
+					for(uint8 i=0; i<lengthof(koord::neighbours)  &&  buildings.get_count() == old_count; i++) {
+						baue_gebaeude(end.get_2d()+zv+koord::neighbours[i]);
+					}
+				}
+			}
+		}
 		return true;
 	}
 
@@ -2927,8 +2969,9 @@ vector_tpl<koord>* stadt_t::random_place(const karte_t* wl, const sint32 anzahl,
 	weighted_vector_tpl<koord> index_to_places(xmax*ymax);
 	for(uint32 i=0; i<xmax; i++) {
 		for(uint32 j=0; j<ymax; j++) {
-			if (!places.at(i,j).empty()) {
-				index_to_places.append( koord(i,j), places.at(i,j).get_count());
+			vector_tpl<koord> const& p = places.at(i, j);
+			if (!p.empty()) {
+				index_to_places.append(koord(i,j), p.get_count());
 			}
 		}
 	}
@@ -2946,40 +2989,36 @@ vector_tpl<koord>* stadt_t::random_place(const karte_t* wl, const sint32 anzahl,
 			koord const ip = pick_any_weighted(index_to_places);
 			// remove this cell from index list
 			index_to_places.remove(ip);
+			vector_tpl<koord>& p = places.at(ip);
 			// get random place in the cell
-			if (places.at(ip).empty()) continue;
-			const uint32 j = simrand(places.at(ip).get_count());
-			const koord k = places.at(ip)[j];
-
-			// check minimum distance
-			bool ok = true;
+			if (p.empty()) continue;
+			uint32 const j = simrand(p.get_count());
+			koord  const k = p[j];
 
 			const koord k2mcd = koord( k.x/minimum_city_distance, k.y/minimum_city_distance );
-			for(sint32 i=k2mcd.x-1; ok && i<=k2mcd.x+1; i++) {
-				for(sint32 j=k2mcd.y-1; ok && j<=k2mcd.y+1; j++) {
+			for (sint32 i = k2mcd.x - 1; i <= k2mcd.x + 1; ++i) {
+				for (sint32 j = k2mcd.y - 1; j <= k2mcd.y + 1; ++j) {
 					if (i>=0 && i<(sint32)xmax2 && j>=0 && j<(sint32)ymax2) {
-						for(uint32 l=0; ok && l<result_places.at(i,j).get_count(); l++) {
-							if (koord_distance(k, result_places.at(i,j)[l]) < minimum_city_distance){
-								ok = false;
+						FOR(vector_tpl<koord>, const& l, result_places.at(i, j)) {
+							if (koord_distance(k, l) < minimum_city_distance) {
+								goto too_close;
 							}
 						}
 					}
 				}
 			}
 
-			if (ok){ //minimum_dist > minimum_city_distance) {
-				// all citys are far enough => ok, find next place
-				result->append(k);
-				result_places.at(k2mcd).append(k);
-				break;
-			}
-			else {
-				// remove the place from the list
-				places.at(ip).remove_at(j);
-				// re-insert in index list with new weight
-				if (!places.at(ip).empty()) {
-					index_to_places.append( ip, places.at(ip).get_count());
-				}
+			// all citys are far enough => ok, find next place
+			result->append(k);
+			result_places.at(k2mcd).append(k);
+			break;
+
+too_close:
+			// remove the place from the list
+			p.remove_at(j);
+			// re-insert in index list with new weight
+			if (!p.empty()) {
+				index_to_places.append(ip, p.get_count());
 			}
 			// if we reached here, the city was not far enough => try again
 		}
