@@ -21,13 +21,6 @@
 #include "halthandle_t.h"
 
 #define MAX_MONTHS               12 // Max history
-#define CONVOI_CAPACITY           0 // the amount of ware that could be transported, theoretically
-#define CONVOI_TRANSPORTED_GOODS  1 // the amount of ware that has been transported
-#define CONVOI_REVENUE            2 // the income this CONVOI generated
-#define CONVOI_OPERATIONS         3 // the cost of operations this CONVOI generated
-#define CONVOI_PROFIT             4 // total profit of this convoi
-#define CONVOI_DISTANCE           5 // total distance traveled this month
-#define MAX_CONVOI_COST           6 // Total number of cost items
 
 class weg_t;
 class depot_t;
@@ -48,9 +41,20 @@ class cbuffer_t;
 class convoi_t : public sync_steppable, public overtaker_t
 {
 public:
+	enum {
+		CONVOI_CAPACITY = 0,       // the amount of ware that could be transported, theoretically
+		CONVOI_TRANSPORTED_GOODS,  // the amount of ware that has been transported
+		CONVOI_REVENUE,            // the income this CONVOI generated
+		CONVOI_OPERATIONS,         // the cost of operations this CONVOI generated
+		CONVOI_PROFIT,             // total profit of this convoi
+		CONVOI_DISTANCE,           // total distance traveled this month
+		CONVOI_MAXSPEED,           // average max. possible speed
+		MAX_CONVOI_COST            // Total number of cost items
+	};
+
 	/* Konstanten
-	* @author prissi
-	*/
+	 * @author prissi
+	 */
 	enum { max_vehicle=4, max_rail_vehicle = 24 };
 
 	enum states {INITIAL,
@@ -220,12 +224,11 @@ private:
 	// cached values
 	// will be recalculated if
 	// recalc_data is true
-	bool recalc_brake_soll;
-	bool recalc_data;
+	bool recalc_data_front; // true when front vehicle in convoi hops
+	bool recalc_data; // true when any vehicle in convoi hops
 	sint32 sum_friction_weight;
 	sint32 speed_limit;
 
-
 	/**
 	* Lowest top speed of all vehicles. Doesn't get saved, but calculated
 	* from the vehicles data
@@ -234,18 +237,26 @@ private:
 	sint32 min_top_speed;
 
 	/**
-	* this give the index of the next signal or the end of the route
-	* convois will slow down before it, if this is not a waypoint or the cannot pass
-	* The slowdown ist done by the vehicle routines
-	* @author prissi
-	*/
+	 * this give the index of the next signal or the end of the route
+	 * convois will slow down before it, if this is not a waypoint or the cannot pass
+	 * The slowdown ist done by the vehicle routines
+	 * @author prissi
+	 */
 	uint16 next_stop_index;
 
 	/**
-	* manchmal muss eine bestimmte Zeit gewartet werden.
-	* wait_lock bestimmt wie lange gewartet wird (in ms).
-	* @author Hanjsrg Malthaner
-	*/
+	 * this give the index until which the route has been reserved. It is used for
+	 * restoring reservations after loading a game.
+	 * @author prissi
+	 */
+	uint16 next_reservation_index;
+
+	/**
+	 * The coinvoi is not processed every sync step for various actions
+	 * (like waiting before signals, loading etc.) Such action will only
+	 * continue after a waiting time larger than wait_lock
+	 * @author Hanjsrg Malthaner
+	 */
 	sint32 wait_lock;
 
 	/**
@@ -263,6 +274,12 @@ private:
 	/* the odometer */
 	sint64 total_distance_traveled;
 
+	uint32 distance_since_last_stop; // number of tiles entered since last stop
+	uint32 sum_speed_limit; // sum of the speed limits encountered since the last stop
+
+	sint32 speedbonus_kmh; // speed used for speedbonus calculation in km/h
+	sint32 maxspeed_average_count;	// just a simple count to average for statsitics
+
 	/**
 	* Set, when there was a income calculation (avoids some cheats)
 	* Since 99.15 it will stored directly in the vehikel_t
@@ -312,6 +329,9 @@ private:
 	 */
 	void unreserve_route();
 
+	// reseverse route until next_reservation_index
+	void reserve_route();
+
 	/**
 	* Mark first and last vehicle.
 	* @author Hanjsrg Malthaner
@@ -396,12 +416,6 @@ public:
 	const route_t* get_route() const { return &route; }
 	route_t* access_route() { return &route; }
 
-	/**
-	* Checks if this convoi has a driveable route
-	* @author Hanjsrg Malthaner
-	*/
-	bool hat_keine_route() const { return (state==NO_ROUTE); }
-
 	/**
 	* get line
 	* @author hsiegeln
@@ -801,14 +815,20 @@ public:
 	koord3d get_home_depot() { return home_depot; }
 
 	/**
-	* this give the index of the next signal or the end of the route
-	* convois will slow down before it, if this is not a waypoint or the cannot pass
-	* The slowdown ist done by the vehicle routines
-	* @author prissi
-	*/
+	 * this give the index of the next signal or the end of the route
+	 * convois will slow down before it, if this is not a waypoint or the cannot pass
+	 * The slowdown ist done by the vehicle routines
+	 * @author prissi
+	 */
 	uint16 get_next_stop_index() const {return next_stop_index;}
 	void set_next_stop_index(uint16 n);
 
+	/* including this route_index, the route was reserved the laste time
+	 * currently only used for tracks
+	 */
+	uint16 get_next_reservation_index() const {return next_reservation_index;}
+	void set_next_reservation_index(uint16 n);
+
 	/* the current state of the convoi */
 	uint8 get_status_color() const;
 
@@ -826,10 +846,14 @@ public:
 	void set_no_load(bool new_no_load) { no_load = new_no_load; }
 
 	void must_recalc_data() { recalc_data = true; }
-	void must_recalc_brake_soll() { recalc_brake_soll = true; }
+	void must_recalc_data_front() { recalc_data_front = true; }
+
+	// calculates the speed used for the speedbonus base, and the max achievable speed at current power/weight for overtakers
+	void calc_speedbonus_kmh();
+	sint32 get_speedbonus_kmh() const;
 
 	// Overtaking for convois
-	virtual bool can_overtake(overtaker_t *other_overtaker, int other_speed, int steps_other, int diagonal_vehicle_steps_per_tile);
+	virtual bool can_overtake(overtaker_t *other_overtaker, sint32 other_speed, sint16 steps_other);
 };
 
 #endif
