@@ -1,18 +1,13 @@
 /*
  * Copyright (c) 1997 - 2001 Hansjrg Malthaner
  *
- * This file is part of the Simutrans project under the artistic licence.
+ * This file is part of the Simutrans project under the artistic license.
  */
 
 #ifndef _WIN32
 #error "Only Windows has GDI!"
 #endif
 
-#ifndef _MSC_VER
-#include <unistd.h>
-#include <sys/time.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -39,10 +34,9 @@
 // 16 Bit may be much slower than 15 unfourtunately on some hardware
 #define USE_16BIT_DIB
 
-// for redraws in another thread
-//#define MULTI_THREAD
+// for redraws in another thread (gives some accelleration on larger screens)
+#define MULTI_THREAD
 
-#include "simmain.h"
 #include "simmem.h"
 #include "simsys_w32_png.h"
 #include "simversion.h"
@@ -51,13 +45,7 @@
 #include "simdebug.h"
 #include "macros.h"
 
-#if COLOUR_DEPTH == 8
-typedef unsigned char PIXVAL;
-#elif COLOUR_DEPTH == 16
 typedef unsigned short PIXVAL;
-#else
-#	error unknown COLOUR_DEPTH
-#endif
 
 static HWND hwnd;
 static bool is_fullscreen = false;
@@ -72,27 +60,12 @@ static PIXVAL*     AllDibData;
 
 volatile HDC hdc = NULL;
 
-const wchar_t* const title =
-#ifdef _MSC_VER
-#define TOW_(x) L#x
-#define TOW(x) TOW_(x)
-			L"Simutrans " WIDE_VERSION_NUMBER
-#ifdef REVISION
-			L" - r" TOW(REVISION)
-#endif
-#else
-			L"" SAVEGAME_PREFIX " " VERSION_NUMBER " - " VERSION_DATE
-#ifdef REVISION
-			" - r" QUOTEME(REVISION)
-#endif
-#endif
-;
-
 #ifdef MULTI_THREAD
 
-
 HANDLE	hFlushThread=0;
-#define WAIT_FOR_SCREEN() {while(hdc) Sleep(5);}
+static volatile int flushcount = 0;
+
+#define WAIT_FOR_SCREEN() {while(flushcount) Sleep(5);}
 
 #else
 
@@ -101,19 +74,18 @@ HANDLE	hFlushThread=0;
 #endif
 
 
-
 /*
  * Hier sind die Basisfunktionen zur Initialisierung der
  * Schnittstelle untergebracht
  * -> init,open,close
  */
-int dr_os_init(const int* /*parameter*/)
+bool dr_os_init(int const* /*parameter*/)
 {
 	// prepare for next event
 	sys_event.type = SIM_NOEVENT;
 	sys_event.code = 0;
 
-	return TRUE;
+	return true;
 }
 
 
@@ -126,6 +98,15 @@ resolution dr_query_screen_resolution()
 }
 
 
+static void create_window(DWORD const ex_style, DWORD const style, int const x, int const y, int const w, int const h)
+{
+	RECT r = { 0, 0, w, h };
+	AdjustWindowRectEx(&r, style, false, ex_style);
+	hwnd = CreateWindowExA(ex_style, "Simu", SIM_TITLE, style, x, y, r.right - r.left, r.bottom - r.top, 0, 0, hInstance, 0);
+	ShowWindow(hwnd, SW_SHOW);
+}
+
+
 // open the window
 int dr_os_open(int const w, int const h, int fullscreen)
 {
@@ -140,11 +121,7 @@ int dr_os_open(int const w, int const h, int fullscreen)
 		MEMZERO(settings);
 		settings.dmSize = sizeof(settings);
 		settings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
-#if COLOUR_DEPTH != 16 || defined USE_16BIT_DIB
 		settings.dmBitsPerPel = COLOUR_DEPTH;
-#else
-		settings.dmBitsPerPel = 16;
-#endif
 		settings.dmPelsWidth  = w;
 		settings.dmPelsHeight = h;
 		settings.dmDisplayFrequency = 0;
@@ -160,36 +137,15 @@ int dr_os_open(int const w, int const h, int fullscreen)
 		is_fullscreen = fullscreen;
 	}
 	if(  fullscreen  ) {
-		hwnd = CreateWindowEx(
-			WS_EX_TOPMOST,
-			L"Simu", title,
-			WS_POPUP,
-			0, 0,
-			w, h,
-			NULL, NULL, hInstance, NULL
-		);
+		create_window(WS_EX_TOPMOST, WS_POPUP, 0, 0, w, h);
 	} else {
-		hwnd = CreateWindow(
-			L"Simu", title,
-			WS_OVERLAPPEDWINDOW,
-			CW_USEDEFAULT, CW_USEDEFAULT,
-			w + GetSystemMetrics(SM_CXFRAME),
-			h - 1 + 2 * GetSystemMetrics(SM_CYFRAME) + GetSystemMetrics(SM_CYCAPTION),
-			NULL, NULL, hInstance, NULL
-		);
+		create_window(0, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, w, h);
 	}
-	ShowWindow(hwnd, SW_SHOW);
 
 	WindowSize.right  = w;
 	WindowSize.bottom = h;
 
-#if COLOUR_DEPTH == 8
-	DWORD const clr = 224 + 15;
-	AllDib = MALLOCF(BITMAPINFO, bmiColors, 256);
-#else
-	DWORD const clr = 0;
 	AllDib = MALLOCF(BITMAPINFO, bmiColors, 3);
-#endif
 	BITMAPINFOHEADER& header = AllDib->bmiHeader;
 	header.biSize          = sizeof(BITMAPINFOHEADER);
 	header.biWidth         = w;
@@ -200,28 +156,32 @@ int dr_os_open(int const w, int const h, int fullscreen)
 	header.biSizeImage     = 0;
 	header.biXPelsPerMeter = 0;
 	header.biYPelsPerMeter = 0;
-	header.biClrUsed       = clr;
-	header.biClrImportant  = clr;
-#if COLOUR_DEPTH == 16
+	header.biClrUsed       = 0;
+	header.biClrImportant  = 0;
 	DWORD* const masks = (DWORD*)AllDib->bmiColors;
-#	ifdef USE_16BIT_DIB
+#ifdef USE_16BIT_DIB
 	header.biCompression   = BI_BITFIELDS;
 	masks[0]               = 0x0000F800;
 	masks[1]               = 0x000007E0;
 	masks[2]               = 0x0000001F;
-#	else
+#else
 	masks[0]               = 0x01;
 	masks[1]               = 0x02;
 	masks[2]               = 0x03;
-#	endif
 #endif
 
 	return MaxSize.right;
 }
 
 
-int dr_os_close(void)
+void dr_os_close()
 {
+#ifdef MULTI_THREAD
+	WAIT_FOR_SCREEN();
+	if(  hFlushThread  ) {
+		TerminateThread( hFlushThread, 0 );
+	}
+#endif
 	if (hwnd != NULL) {
 		DestroyWindow(hwnd);
 	}
@@ -229,8 +189,9 @@ int dr_os_close(void)
 	AllDibData = NULL;
 	free(AllDib);
 	AllDib = NULL;
-	ChangeDisplaySettings(NULL, 0);
-	return TRUE;
+	if(  is_fullscreen  ) {
+		ChangeDisplaySettings(NULL, 0);
+	}
 }
 
 
@@ -288,18 +249,6 @@ unsigned int get_system_color(unsigned int r, unsigned int g, unsigned int b)
 }
 
 
-void dr_setRGB8multi(int first, int count, unsigned char* data)
-{
-	// set color in DibHeader ...
-	RGBQUAD *pRGB = (RGBQUAD *)((char *)AllDib + sizeof(BITMAPINFOHEADER));
-	for(  int i=first;  i<first+count;  i++  ) {
-		pRGB[i].rgbRed = data[i*3];
-		pRGB[i].rgbGreen = data[i*3+1];
-		pRGB[i].rgbBlue = data[i*3+2];
-	}
-}
-
-
 #ifdef MULTI_THREAD
 // multhreaded screen copy ...
 DWORD WINAPI dr_flush_screen(LPVOID lpParam)
@@ -310,6 +259,7 @@ DWORD WINAPI dr_flush_screen(LPVOID lpParam)
 		display_flush_buffer();
 		ReleaseDC(hwnd, hdc);
 		hdc = NULL;
+		flushcount --;
 		// suspend myself after one update
 		SuspendThread( hFlushThread );
 	}
@@ -333,6 +283,7 @@ void dr_flush()
 {
 #ifdef MULTI_THREAD
 	// just let the thread do its work
+	flushcount ++;
 	ResumeThread( hFlushThread );
 #else
 	assert(hdc==NULL);
@@ -385,16 +336,14 @@ void set_pointer(int loading)
  */
 int dr_screenshot(const char *filename)
 {
-#if COLOUR_DEPTH != 16 || defined USE_16BIT_DIB
+#if defined USE_16BIT_DIB
 	int const bpp = COLOUR_DEPTH;
 #else
 	int const bpp = 15;
 #endif
 	if (!dr_screenshot_png(filename, display_get_width() - 1, WindowSize.bottom + 1, AllDib->bmiHeader.biWidth, (unsigned short*)AllDibData, bpp)) {
-#if COLOUR_DEPTH != 8
 		// not successful => save as BMP
-		FILE *fBmp = fopen(filename, "wb");
-		if (fBmp) {
+		if (FILE* const fBmp = fopen(filename, "wb")) {
 			BITMAPFILEHEADER bf;
 
 			// since the number of drawn pixel can be smaller than the actual width => only use the drawn pixel for bitmap
@@ -421,7 +370,6 @@ int dr_screenshot(const char *filename)
 		else {
 			return -1;
 		}
-#endif
 	}
 	return 0;
 }
@@ -438,7 +386,6 @@ static inline unsigned int ModifierKeys()
 		(GetKeyState(VK_CONTROL) < 0  ? 2 : 0); // highest bit set or return value<0 -> key is pressed
 }
 
-struct sys_event sys_event;
 
 /* Windows eventhandler: does most of the work */
 LRESULT WINAPI WindowProc(HWND this_hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
@@ -462,7 +409,7 @@ LRESULT WINAPI WindowProc(HWND this_hwnd, UINT msg, WPARAM wParam, LPARAM lParam
 					MEMZERO(settings);
 					settings.dmSize = sizeof(settings);
 					settings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
-#if COLOUR_DEPTH != 16 || defined USE_16BIT_DIB
+#if defined USE_16BIT_DIB
 					settings.dmBitsPerPel = COLOUR_DEPTH;
 #else
 					settings.dmBitsPerPel = 15;
@@ -477,15 +424,7 @@ LRESULT WINAPI WindowProc(HWND this_hwnd, UINT msg, WPARAM wParam, LPARAM lParam
 
 					Beep( 110, 250 );
 					// must reshow window, otherwise startbar will be topmost ...
-					hwnd = CreateWindowEx(
-						WS_EX_TOPMOST,
-						L"Simu", title,
-						WS_POPUP,
-						0, 0,
-						MaxSize.right, MaxSize.bottom,
-						NULL, NULL, hInstance, NULL
-					);
-					ShowWindow( hwnd, SW_SHOW );
+					create_window(WS_EX_TOPMOST, WS_POPUP, 0, 0, MaxSize.right, MaxSize.bottom);
 					DestroyWindow( this_hwnd );
 					while_handling = false;
 					return true;
@@ -655,7 +594,6 @@ LRESULT WINAPI WindowProc(HWND this_hwnd, UINT msg, WPARAM wParam, LPARAM lParam
 			if (AllDibData != NULL) {
 				sys_event.type = SIM_SYSTEM;
 				sys_event.code = SIM_SYSTEM_QUIT;
-				return FALSE;
 			}
 			break;
 
@@ -665,18 +603,17 @@ LRESULT WINAPI WindowProc(HWND this_hwnd, UINT msg, WPARAM wParam, LPARAM lParam
 			if (AllDibData == NULL) {
 				PostQuitMessage(0);
 				hwnd = NULL;
-				return TRUE;
 			}
-			return FALSE;
+			break;
 
 		default:
 			return DefWindowProc(this_hwnd, msg, wParam, lParam);
 	}
-	return FALSE;
+	return 0;
 }
 
 
-static void internal_GetEvents(int wait)
+static void internal_GetEvents(bool const wait)
 {
 	do {
 		// wait for keybord/mouse event
@@ -691,7 +628,7 @@ void GetEvents()
 {
 	// already even processed?
 	if(sys_event.type==SIM_NOEVENT) {
-		internal_GetEvents(TRUE);
+		internal_GetEvents(true);
 	}
 }
 
@@ -699,7 +636,7 @@ void GetEvents()
 void GetEventsNoWait()
 {
 	if (sys_event.type==SIM_NOEVENT  &&  PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
-		internal_GetEvents(FALSE);
+		internal_GetEvents(false);
 	}
 }
 
@@ -736,19 +673,6 @@ void dr_sleep(uint32 millisec)
 }
 
 
-
-bool dr_fatal_notify(const char* msg, int choices)
-{
-	if(choices==0) {
-		MessageBoxA( hwnd, msg, "Fatal Error", MB_ICONEXCLAMATION|MB_OK );
-		return 0;
-	}
-	else {
-		return MessageBoxA( hwnd, msg, "Fatal Error", MB_ICONEXCLAMATION|MB_RETRYCANCEL	)==IDRETRY;
-	}
-}
-
-
 int CALLBACK WinMain(HINSTANCE const hInstance, HINSTANCE, LPSTR, int)
 {
 	WNDCLASSW wc;
@@ -766,12 +690,6 @@ int CALLBACK WinMain(HINSTANCE const hInstance, HINSTANCE, LPSTR, int)
 
 	RegisterClass(&wc);
 
-	int    const argc = __argc;
-	char** const argv = __argv;
-	char         pathname[1024];
-	GetModuleFileNameA(hInstance, pathname, lengthof(pathname));
-	argv[0] = pathname;
-
 	GetWindowRect(GetDesktopWindow(), &MaxSize);
 
 	// maybe set timer to 1ms intervall on Win2k upwards ...
@@ -783,7 +701,7 @@ int CALLBACK WinMain(HINSTANCE const hInstance, HINSTANCE, LPSTR, int)
 		}
 	}
 
-	simu_main(argc, argv);
+	int const res = sysmain(__argc, __argv);
 	timeEndPeriod(1);
 
 #ifdef MULTI_THREAD
@@ -791,5 +709,5 @@ int CALLBACK WinMain(HINSTANCE const hInstance, HINSTANCE, LPSTR, int)
 		TerminateThread( hFlushThread, 0 );
 	}
 #endif
-	return 0;
+	return res;
 }
