@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 1997 - 2003 Hansjrg Malthaner
  *
- * This file is part of the Simutrans project under the artistic licence.
- * (see licence.txt)
+ * This file is part of the Simutrans project under the artistic license.
+ * (see license.txt)
  */
 
 /* Subfenster fuer Sim
@@ -114,6 +114,8 @@ static vector_tpl<simwin_t> kill_list(MAX_WIN);
 
 static karte_t* wl = NULL; // Zeiger auf aktuelle Welt, wird in win_set_welt gesetzt
 
+static int top_win(int win, bool keep_state );
+static void display_win(int win);
 
 
 // Hajo: tooltip data
@@ -323,9 +325,9 @@ static void win_draw_window_title(const koord pos, const koord gr,
 	// Draw the gadgets and then move left and draw text.
 	flags.gotopos = (welt_pos != koord3d::invalid);
 	int width = display_gadget_boxes( &flags, pos.x+(REVERSE_GADGETS?0:gr.x-20), pos.y, titel_farbe, closing, sticky );
-	int titlewidth = display_proportional_clip( pos.x + (REVERSE_GADGETS?width+4:4), pos.y+(16-large_font_height)/2, text, ALIGN_LEFT, text_farbe, false );
+	int titlewidth = display_proportional_clip( pos.x + (REVERSE_GADGETS?width+4:4), pos.y+(16-LINEASCENT)/2, text, ALIGN_LEFT, text_farbe, false );
 	if(  flags.gotopos  ) {
-		display_proportional_clip( pos.x + (REVERSE_GADGETS?width+4:4)+titlewidth+8, pos.y+(16-large_font_height)/2, welt_pos.get_2d().get_fullstr(), ALIGN_LEFT, text_farbe, false );
+		display_proportional_clip( pos.x + (REVERSE_GADGETS?width+4:4)+titlewidth+8, pos.y+(16-LINEASCENT)/2, welt_pos.get_2d().get_fullstr(), ALIGN_LEFT, text_farbe, false );
 	}
 	POP_CLIP();
 }
@@ -361,10 +363,10 @@ gui_frame_t *win_get_magic(long magic)
 {
 	if(magic!=-1  &&  magic!=0) {
 		// es kann nur ein fenster fuer jede pos. magic number geben
-		for(  uint i=0;  i<wins.get_count();  i++  ) {
-			if(wins[i].magic_number == magic) {
+		FOR(vector_tpl<simwin_t>, const& i, wins) {
+			if (i.magic_number == magic) {
 				// if 'special' magic number, return it
-				return wins[i].gui;
+				return i.gui;
 			}
 		}
 	}
@@ -400,11 +402,11 @@ int win_get_open_count()
 
 
 // brings a window to front, if open
-bool top_win(const gui_frame_t *gui)
+bool top_win( const gui_frame_t *gui, bool keep_rollup )
 {
 	for(  uint i=0;  i<wins.get_count()-1;  i++  ) {
 		if(wins[i].gui==gui) {
-			top_win(i);
+			top_win(i,keep_rollup);
 			return true;
 		}
 	}
@@ -430,15 +432,15 @@ void rdwr_all_win(loadsave_t *file)
 {
 	if(  file->get_version()>102003  ) {
 		if(  file->is_saving()  ) {
-			for ( uint32 i=0;  i < wins.get_count();  i++ ) {
-				uint32 id = wins[i].gui->get_rdwr_id();
+			FOR(vector_tpl<simwin_t>, & i, wins) {
+				uint32 id = i.gui->get_rdwr_id();
 				if(  id!=magic_reserved  ) {
 					file->rdwr_long( id );
-					wins[i].pos.rdwr( file );
-					file->rdwr_byte( wins[i].wt );
-					file->rdwr_bool( wins[i].sticky );
-					file->rdwr_bool( wins[i].rollup );
-					wins[i].gui->rdwr( file );
+					i.pos.rdwr(file);
+					file->rdwr_byte(i.wt);
+					file->rdwr_bool(i.sticky);
+					file->rdwr_bool(i.rollup);
+					i.gui->rdwr(file);
 				}
 			}
 			uint32 end = magic_none;
@@ -631,9 +633,9 @@ int create_win(int x, int y, gui_frame_t* const gui, wintype const wt, long cons
  */
 static void process_kill_list()
 {
-	for(uint i = 0; i < kill_list.get_count(); i++) {
-		wins.remove(kill_list[i]);
-		destroy_framed_win(&kill_list[i]);
+	FOR(vector_tpl<simwin_t>, & i, kill_list) {
+		wins.remove(i);
+		destroy_framed_win(&i);
 	}
 	kill_list.clear();
 }
@@ -730,7 +732,7 @@ void destroy_all_win(bool destroy_sticky)
 }
 
 
-int top_win(int win)
+int top_win(int win, bool keep_state )
 {
 	if(  (uint32)win==wins.get_count()-1  ) {
 		return win;
@@ -741,6 +743,9 @@ int top_win(int win)
 
 	simwin_t tmp = wins[win];
 	wins.remove_at(win);
+	if(  !keep_state  ) {
+		tmp.rollup = false;	// make visible when topping
+	}
 	wins.append(tmp);
 
 	 // mark new dirty
@@ -824,7 +829,7 @@ void display_all_win()
 	const sint16 x = get_maus_x();
 	const sint16 y = get_maus_y();
 	tooltip_element = NULL;
-	for(  int i=wins.get_count()-1;  i>=0;  i--  ) {
+	for(  uint32 i = wins.get_count(); i-- != 0;  ) {
 		if(  (!wins[i].rollup  &&  wins[i].gui->getroffen(x-wins[i].pos.x,y-wins[i].pos.y))  ||
 		     (wins[i].rollup  &&  x>=wins[i].pos.x  &&  x<wins[i].pos.x+wins[i].gui->get_fenstergroesse().x  &&  y>=wins[i].pos.y  &&  y<wins[i].pos.y+16)
 		) {
@@ -847,8 +852,8 @@ void display_all_win()
 
 void win_rotate90( sint16 new_ysize )
 {
-	for(  uint i=0;  i<wins.get_count();  i++  ) {
-		wins[i].gui->map_rotate90( new_ysize );
+	FOR(vector_tpl<simwin_t>, const& i, wins) {
+		i.gui->map_rotate90(new_ysize);
 	}
 }
 
@@ -857,7 +862,7 @@ void win_rotate90( sint16 new_ysize )
 static void remove_old_win()
 {
 	// alte fenster entfernen, falls dauer abgelaufen
-	for(  int i=wins.get_count()-1;  i>=0;  i=min(i,wins.get_count())-1  ) {
+	for(  int i=wins.get_count()-1;  i>=0;  i=min(i,(int)wins.get_count())-1  ) {
 		if(wins[i].dauer > 0) {
 			wins[i].dauer --;
 			if(wins[i].dauer == 0) {
@@ -1053,10 +1058,10 @@ void resize_win(int win, event_t *ev)
 // returns true, if gui is a open window handle
 bool win_is_open(gui_frame_t *gui)
 {
-	for(  uint i=0;  i<wins.get_count();  i++  ) {
-		if(  wins[i].gui == gui  ) {
-			for(  uint j = 0;  j < kill_list.get_count();  j++  ) {
-				if(  kill_list[j].gui == gui  ) {
+	FOR(vector_tpl<simwin_t>, const& i, wins) {
+		if (i.gui == gui) {
+			FOR(vector_tpl<simwin_t>, const& j, kill_list) {
+				if (j.gui == gui) {
 					return false;
 				}
 			}
@@ -1068,32 +1073,22 @@ bool win_is_open(gui_frame_t *gui)
 
 
 
-int win_get_posx(gui_frame_t *gui)
+koord const& win_get_pos(gui_frame_t const* const gui)
 {
-	for(  int i=wins.get_count()-1;  i>=0;  i--  ) {
-		if(wins[i].gui == gui) {
-			return wins[i].pos.x;
+	for(  uint32 i = wins.get_count(); i-- != 0;  ) {
+		if(  wins[i].gui == gui  ) {
+			return wins[i].pos;
 		}
 	}
-	return -1;
-}
-
-
-int win_get_posy(gui_frame_t *gui)
-{
-	for(  int i=wins.get_count()-1;  i>=0;  i--  ) {
-		if(wins[i].gui == gui) {
-			return wins[i].pos.y;
-		}
-	}
-	return -1;
+	static koord const bad(-1, -1);
+	return bad;
 }
 
 
 void win_set_pos(gui_frame_t *gui, int x, int y)
 {
-	for(  int i=wins.get_count()-1;  i>=0;  i--  ) {
-		if(wins[i].gui == gui) {
+	for(  uint32 i = wins.get_count(); i-- != 0;  ) {
+		if(  wins[i].gui == gui  ) {
 			wins[i].pos.x = x;
 			wins[i].pos.y = y;
 			const koord gr = wins[i].gui->get_fenstergroesse();
@@ -1154,11 +1149,13 @@ bool check_pos_win(event_t *ev)
 		return true;
 	}
 
-	// cursor event only go to top window
+	// cursor event only go to top window (but not if rolled up)
 	if(  ev->ev_class == EVENT_KEYBOARD  &&  !wins.empty()  ) {
 		simwin_t&               win  = wins.back();
-		inside_event_handling = win.gui;
-		swallowed = win.gui->infowin_event(ev);
+		if(  !win.rollup  )  {
+			inside_event_handling = win.gui;
+			swallowed = win.gui->infowin_event(ev);
+		}
 		inside_event_handling = NULL;
 		process_kill_list();
 		return swallowed;
@@ -1177,13 +1174,13 @@ bool check_pos_win(event_t *ev)
 	}
 
 	// swallow all other events in the infobar
-	if(  y > display_get_height()-16  ) {
+	if(  ev->ev_class != EVENT_KEYBOARD  &&  y > display_get_height()-16  ) {
 		// swallow event
 		return true;
 	}
 
 	// swallow all other events in ticker (if there)
-	if(  show_ticker  &&  y > display_get_height()-32  ) {
+	if(  ev->ev_class != EVENT_KEYBOARD  &&  show_ticker  &&  y > display_get_height()-32  ) {
 		if(  IS_LEFTCLICK(ev)  ) {
 			// goto infowin koordinate, if ticker is active
 			koord p = ticker::get_welt_pos();
@@ -1196,7 +1193,7 @@ bool check_pos_win(event_t *ev)
 	}
 
 	// handle all the other events
-	for(  int i=wins.get_count()-1;  i>=0  &&  !swallowed;  i=min(i,wins.get_count())-1  ) {
+	for(  int i=wins.get_count()-1;  i>=0  &&  !swallowed;  i=min(i,(int)wins.get_count())-1  ) {
 
 		if(  wins[i].gui->getroffen( x-wins[i].pos.x, y-wins[i].pos.y )  ) {
 
@@ -1207,7 +1204,7 @@ bool check_pos_win(event_t *ev)
 
 			// Top window first
 			if(  (int)wins.get_count()-1>i  &&  IS_LEFTCLICK(ev)  &&  (!wins[i].rollup  ||  ev->cy<wins[i].pos.y+16)  ) {
-				i = top_win(i);
+				i = top_win(i,false);
 			}
 
 			// Hajo: if within title bar && window needs decoration
@@ -1242,7 +1239,7 @@ bool check_pos_win(event_t *ev)
 						break;
 					case GADGET_HELP :
 						if (IS_LEFTCLICK(ev)) {
-							create_win(new help_frame_t(wins[i].gui->get_hilfe_datei()), w_info, (long)(wins[i].gui->get_hilfe_datei()) );
+							help_frame_t::open_help_on( wins[i].gui->get_hilfe_datei() );
 							inside_event_handling = 0;
 						}
 						break;
@@ -1275,7 +1272,7 @@ bool check_pos_win(event_t *ev)
 						break;
 					default : // Title
 						if (IS_LEFTDRAG(ev)) {
-							i = top_win(i);
+							i = top_win(i,false);
 							move_win(i, ev);
 							is_moving = i;
 						}
@@ -1331,13 +1328,13 @@ bool check_pos_win(event_t *ev)
 }
 
 
-void win_get_event(struct event_t *ev)
+void win_get_event(event_t* const ev)
 {
 	display_get_event(ev);
 }
 
 
-void win_poll_event(struct event_t *ev)
+void win_poll_event(event_t* const ev)
 {
 	display_poll_event(ev);
 	// main window resized
@@ -1630,7 +1627,7 @@ bool win_change_zoom_factor(bool magnify)
 		ev.cy = 0;
 		ev.button_state = 0;
 
-		for(  sint32 i=wins.get_count()-1;  i>=0;  i=min(i,wins.get_count())-1  ) {
+		for(  sint32 i=wins.get_count()-1;  i>=0;  i=min(i,(int)wins.get_count())-1  ) {
 			wins[i].gui->infowin_event(&ev);
 		}
 	}
