@@ -20,11 +20,19 @@ private:
 
 	char * buf;
 
-	cbuffer_t(const cbuffer_t &);
-	cbuffer_t & operator=(const cbuffer_t &);
+	/**
+	 * Implementation for copy constructor and copy assignment operator
+	 * @author Timothy Baldock <tb@entropy.me.uk>
+	 */
+	void copy (const cbuffer_t& cbx);
+
+	/**
+	 * Implementation for destructor and copy assignment operator
+	 * @author Timothy Baldock <tb@entropy.me.uk>
+	 */
+	void free ();
 
 public:
-
 	/**
 	 * Number of characters without(!) trailing '\0'
 	 * @author Hj. Malthaner
@@ -33,12 +41,21 @@ public:
 
 	/**
 	 * Creates a new cbuffer with capacity cap
-	 * @param cap the capacity
-	 * @author Hj. Malthaner
 	 */
 	cbuffer_t();
 	~cbuffer_t();
 
+	/**
+	 * Copy constructor
+	 * @author Timothy Baldock <tb@entropy.me.uk>
+	 */
+	cbuffer_t (const cbuffer_t& cbx);
+
+	/**
+	 * Copy assignment operator
+	 * @author Timothy Baldock <tb@entropy.me.uk>
+	 */
+	cbuffer_t& operator= (const cbuffer_t& cbx);
 
 	/**
 	 * Clears the buffer
@@ -53,6 +70,20 @@ public:
 	 */
 	void append(const char * text);
 
+	/**
+	 * Appends text, at most n characters worth. Buffer will be extended if needed.
+	 * maxchars should NOT include the null at the end of the string!
+	 *  (e.g. it should be equivalent to the output of strlen())
+	 * @author Timothy Baldock <tb@entropy.me.uk>
+	 */
+	void append (const char* text, size_t maxchars);
+
+	/**
+	 * Return contents of buffer
+	 * @author Timothy Baldock <tb@entropy.me.uk>
+	 */
+	const char* get_str () const;
+
 	/**
 	 * Appends a number. Buffer will be extended if it does not have enough capacity.
 	 * @author Hj. Malthaner
