@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 1997 - 2001 Hansjrg Malthaner
  *
- * This file is part of the Simutrans project under the artistic licence.
- * (see licence.txt)
+ * This file is part of the Simutrans project under the artistic license.
+ * (see license.txt)
  */
 
 /*
@@ -101,6 +101,7 @@ public:
 	// only called during load time: set some offsets
 	static void set_diagonal_multiplier( uint32 multiplier, uint32 old_multiplier );
 	static uint16 get_diagonal_multiplier() { return diagonal_multiplier; }
+	static uint8 get_diagonal_vehicle_steps_per_tile() { return diagonal_vehicle_steps_per_tile; }
 
 	static void set_overtaking_offsets( bool driving_on_the_left );
 
@@ -120,7 +121,7 @@ public:
 
 	virtual void rotate90();
 
-	ribi_t::ribi calc_richtung(koord start, koord ende) const;
+	static ribi_t::ribi calc_richtung(koord start, koord ende);
 	ribi_t::ribi calc_set_richtung(koord start, koord ende);
 
 	ribi_t::ribi get_fahrtrichtung() const {return fahrtrichtung;}
@@ -286,7 +287,7 @@ public:
 	const vehikel_besch_t *get_besch() const {return besch; }
 
 	/**
-	* @return die Betriebskosten in Cr/100Km
+	* @return die running_cost in Cr/100Km
 	* @author Hj. Malthaner
 	*/
 	int get_betriebskosten() const { return besch->get_betriebskosten(); }
@@ -458,6 +459,11 @@ template<> inline vehikel_t* ding_cast<vehikel_t>(ding_t* const d)
  */
 class automobil_t : public vehikel_t
 {
+private:
+	// called internally only from ist_weg_frei()
+	// returns true on success
+	bool choose_route( int &restart_speed, ribi_t::dir richtung, uint16 index );
+
 protected:
 	bool ist_befahrbar(const grund_t *bd) const;
 
