@@ -74,8 +74,6 @@ verkehrsteilnehmer_t::~verkehrsteilnehmer_t()
 }
 
 
-
-
 verkehrsteilnehmer_t::verkehrsteilnehmer_t(karte_t *welt, koord3d pos) :
 	vehikel_basis_t(welt, pos)
 {
@@ -92,7 +90,7 @@ verkehrsteilnehmer_t::verkehrsteilnehmer_t(karte_t *welt, koord3d pos) :
 
 	// verfgbare ribis in liste eintragen
 	for(int r = 0; r < 4; r++) {
-		if(from->get_neighbour(to, road_wt, koord::nsow[r])) {
+		if(from->get_neighbour(to, road_wt, ribi_t::nsow[r])) {
 			liste[count++] = ribi_t::nsow[r];
 		}
 	}
@@ -119,7 +117,8 @@ verkehrsteilnehmer_t::verkehrsteilnehmer_t(karte_t *welt, koord3d pos) :
 	if(count) {
 		from->get_neighbour(to, road_wt, fahrtrichtung);
 		pos_next = to->get_pos();
-	} else {
+	}
+	else {
 		pos_next = welt->lookup_kartenboden(pos.get_2d() + koord(fahrtrichtung))->get_pos();
 	}
 	set_besitzer( welt->get_spieler(1) );
@@ -170,7 +169,7 @@ void verkehrsteilnehmer_t::hop()
 	int ribi = weg->get_ribi_unmasked();
 	for(int r = 0; r < 4; r++) {
 		if(  (ribi & ribi_t::nsow[r])!=0  &&  (ribi_t::nsow[r]&gegenrichtung)==0 &&
-			from->get_neighbour(to, road_wt, koord::nsow[r])
+			from->get_neighbour(to, road_wt, ribi_t::nsow[r])
 		) {
 			// check, if this is just a single tile deep
 			int next_ribi =  to->get_weg(road_wt)->get_ribi_unmasked();
@@ -344,9 +343,8 @@ void stadtauto_t::built_timeline_liste(karte_t *welt)
 //DBG_DEBUG("stadtauto_t::built_timeline_liste()","year=%i, month=%i", month_now/12, month_now%12+1);
 
 		// check for every citycar, if still ok ...
-		stringhashtable_iterator_tpl<const stadtauto_besch_t *> iter(table);
-		while(   iter.next()  ) {
-			const stadtauto_besch_t* info = iter.get_current_value();
+		FOR(stringhashtable_tpl<stadtauto_besch_t const*>, const& i, table) {
+			stadtauto_besch_t const* const info = i.value;
 			const int intro_month = info->get_intro_year_month();
 			const int retire_month = info->get_retire_year_month();
 
@@ -356,8 +354,8 @@ void stadtauto_t::built_timeline_liste(karte_t *welt)
 		}
 	}
 	liste_timeline.resize( temp_liste.get_count() );
-	for (vector_tpl<const stadtauto_besch_t*>::const_iterator i = temp_liste.begin(), end = temp_liste.end(); i != end; ++i) {
-		liste_timeline.append( (*i), (*i)->get_gewichtung() );
+	FOR(vector_tpl<stadtauto_besch_t const*>, const i, temp_liste) {
+		liste_timeline.append(i, i->get_gewichtung());
 	}
 }
 
@@ -415,11 +413,12 @@ bool stadtauto_t::sync_step(long delta_t)
 		return false;
 	}
 
-	if(current_speed==0) {
+	if(  current_speed==0  ) {
 		// stuck in traffic jam
 		uint32 old_ms_traffic_jam = ms_traffic_jam;
 		ms_traffic_jam += delta_t;
-		if(  (ms_traffic_jam>>7) != (old_ms_traffic_jam>>7  )) {
+		// check only every 1.024 s if stopped
+		if(  (ms_traffic_jam>>10) != (old_ms_traffic_jam>>10)  ) {
 			pos_next_next = koord3d::invalid;
 			if(  hop_check(  )) {
 				ms_traffic_jam = 0;
@@ -450,7 +449,6 @@ bool stadtauto_t::sync_step(long delta_t)
 }
 
 
-
 void stadtauto_t::rdwr(loadsave_t *file)
 {
 	xml_tag_t s( file, "stadtauto_t" );
@@ -526,6 +524,8 @@ bool stadtauto_t::ist_weg_frei(grund_t *gr)
 		// already too many things here
 		return false;
 	}
+
+	// road still there?
 	weg_t * str = gr->get_weg(road_wt);
 	if(str==NULL) {
 		time_to_life = 0;
@@ -536,7 +536,7 @@ bool stadtauto_t::ist_weg_frei(grund_t *gr)
 	// are we just turning around?
 	const uint8 this_fahrtrichtung = get_fahrtrichtung();
 	bool frei = false;
-	if(get_pos()==pos_next_next) {
+	if(  get_pos()==pos_next_next  ) {
 		// turning around => single check
 		const uint8 next_fahrtrichtung = ribi_t::rueckwaerts(this_fahrtrichtung);
 		frei = (NULL == no_cars_blocking( gr, NULL, next_fahrtrichtung, next_fahrtrichtung, next_fahrtrichtung ));
@@ -555,14 +555,14 @@ bool stadtauto_t::ist_weg_frei(grund_t *gr)
 		if(ribi_t::is_threeway(str->get_ribi_unmasked())) {
 			// but leaving from railroad crossing is more important
 			grund_t *gr_here = welt->lookup(get_pos());
-			if(gr_here  &&  gr_here->ist_uebergang()) {
+			if(  gr_here  &&  gr_here->ist_uebergang()  ) {
 				return true;
 			}
 			grund_t *test = welt->lookup(pos_next_next);
-			if(test) {
+			if(  test  ) {
 				uint8 next_90fahrtrichtung = this->calc_richtung(pos_next.get_2d(), pos_next_next.get_2d());
 				frei = (NULL == no_cars_blocking( gr, NULL, this_fahrtrichtung, next_fahrtrichtung, next_90fahrtrichtung ));
-				if(frei) {
+				if(  frei  ) {
 					// check, if it can leave this crossings
 					frei = (NULL == no_cars_blocking( test, NULL, next_fahrtrichtung, next_90fahrtrichtung, next_90fahrtrichtung ));
 				}
@@ -587,12 +587,12 @@ bool stadtauto_t::ist_weg_frei(grund_t *gr)
 							if(!over->is_overtaking()) {
 								// otherwise the overtaken car would stop for us ...
 								if(  automobil_t const* const car = ding_cast<automobil_t>(dt)  ) {
-									convoi_t* const cnv = car->get_convoi();
-									if(  cnv==NULL  ||  !can_overtake( cnv, (cnv->get_state()==convoi_t::LOADING ? 0 : cnv->get_min_top_speed()), cnv->get_length_in_steps(), diagonal_vehicle_steps_per_tile)  ) {
+									convoi_t* const ocnv = car->get_convoi();
+									if(  ocnv==NULL  ||  !can_overtake( ocnv, (ocnv->get_state()==convoi_t::LOADING ? 0 : over->get_max_power_speed()), ocnv->get_length_in_steps()+ocnv->get_vehikel(0)->get_steps())  ) {
 										frei = false;
 									}
 								} else if(  stadtauto_t* const caut = ding_cast<stadtauto_t>(dt)  ) {
-									if(  !can_overtake(caut, caut->get_besch()->get_geschw(), VEHICLE_STEPS_PER_TILE, diagonal_vehicle_steps_per_tile)  ) {
+									if(  !can_overtake(caut, caut->get_besch()->get_geschw(), VEHICLE_STEPS_PER_TILE)  ) {
 										frei = false;
 									}
 								}
@@ -608,7 +608,7 @@ bool stadtauto_t::ist_weg_frei(grund_t *gr)
 		}
 
 		// do not block railroad crossing
-		if(frei  &&  str->is_crossing()) {
+		if(  frei  &&  str->is_crossing()  ) {
 			// can we cross?
 			crossing_t* cr = gr->find<crossing_t>(2);
 			if(  cr && !cr->request_crossing(this)) {
@@ -713,7 +713,7 @@ bool stadtauto_t::hop_check()
 	// traffic light phase check (since this is on next tile, it will always be neccessary!)
 	const ribi_t::ribi fahrtrichtung90 = ribi_typ(get_pos().get_2d(),pos_next.get_2d());
 
-	if(weg->has_sign()) {
+	if(  weg->has_sign(  )) {
 		const roadsign_t* rs = from->find<roadsign_t>();
 		const roadsign_besch_t* rs_besch = rs->get_besch();
 		if(rs_besch->is_traffic_light()  &&  (rs->get_dir()&fahrtrichtung90)==0) {
@@ -754,10 +754,10 @@ bool stadtauto_t::hop_check()
 #endif
 			if(  (ribi&ribi_t::nsow[r])!=0  ) {
 				grund_t *to;
-				if(from->get_neighbour(to, road_wt, koord::nsow[r])) {
+				if(  from->get_neighbour(to, road_wt, ribi_t::nsow[r])  ) {
 					// check, if this is just a single tile deep after a crossing
-					weg_t *w=to->get_weg(road_wt);
-					if(ribi_t::ist_einfach(w->get_ribi())  &&  (w->get_ribi()&ribi_t::nsow[r])==0  &&  !ribi_t::ist_einfach(ribi)) {
+					weg_t *w = to->get_weg(road_wt);
+					if(  ribi_t::ist_einfach(w->get_ribi())  &&  (w->get_ribi()&ribi_t::nsow[r])==0  &&  !ribi_t::ist_einfach(ribi)  ) {
 						ribi &= ~ribi_t::nsow[r];
 						continue;
 					}
@@ -923,7 +923,7 @@ void stadtauto_t::get_screen_offset( int &xoff, int &yoff, const sint16 raster_w
  * The city car is not overtaking/being overtaken.
  * @author isidoro
  */
-bool stadtauto_t::can_overtake( overtaker_t *other_overtaker, int other_speed, int steps_other, int diagonal_vehicle_steps_per_tile)
+bool stadtauto_t::can_overtake( overtaker_t *other_overtaker, sint32 other_speed, sint16 steps_other)
 {
 	if(  !other_overtaker->can_be_overtaken()  ) {
 		return false;
@@ -1071,7 +1071,7 @@ bool stadtauto_t::can_overtake( overtaker_t *other_overtaker, int other_speed, i
 				if(  (rib&ribi_t::nsow[r])==0  ||  check_pos.get_2d()+koord::nsow[r]==pos_prev) {
 					continue;
 				}
-				if(gr->get_neighbour(to, road_wt, koord::nsow[r])) {
+				if(gr->get_neighbour(to, road_wt, ribi_t::nsow[r])) {
 					if(found_one) {
 						// two directions to go: unexpected cars may occurs => abort
 						return false;
@@ -1139,7 +1139,7 @@ bool stadtauto_t::can_overtake( overtaker_t *other_overtaker, int other_speed, i
 				if(check_pos.get_2d()+koord::nsow[r]==pos_prev) {
 					continue;
 				}
-				if(gr->get_neighbour(to, road_wt, koord::nsow[r])) {
+				if(gr->get_neighbour(to, road_wt, ribi_t::nsow[r])) {
 					if(found_one) {
 						return false;
 					}
